@@ -21,23 +21,33 @@
 # -------------------------------------------------------------------------
 SRCDIR=${SRCDIR-"."}
 TOP_SRCDIR=${TOP_SRCDIR-"../.."}
+GEODIR="${TOP_SRCDIR}/nfem/ptst"
+ROUNDER="../../nfem/pbin/field - -field -round"
 NPROC_MAX=${NPROC_MAX-"8"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
 status=0
 
 L="
-line-1-bdry
-line-2-bdry
-line-3-bdry
-line-20-bdry
-square-10-bdry
-cube-5-bdry
+$GEODIR/line-1-bdry
+$GEODIR/line-2-bdry
+$GEODIR/line-3-bdry
+$GEODIR/line-20-bdry
+$SRCDIR/square-10-bdry
+$SRCDIR/cube-5-bdry
+$GEODIR/cube-P-5-dom-v2
+$GEODIR/my_cube_PH-5-v2
+$GEODIR/my_cube_TP-5-v2
+$GEODIR/my_cube_TPH-5-v2
 "
 
-for geo in $L; do
-  loop_mpirun "./dirichlet ${SRCDIR}/$geo P1 2>/dev/null | ./rounder 2>/dev/null | diff -w -B $SRCDIR/dirichlet-$geo-P1.field - >/dev/null"
+for fullgeo in $L; do
+ geo=`basename $fullgeo`
+ dir=`dirname $fullgeo`
+ for Pk in P1 P2 P3; do
+   loop_mpirun "./dirichlet $fullgeo $Pk 2>/dev/null | RHEOPATH=${dir} $ROUNDER 1e-7 2>/dev/null | diff -w -B $SRCDIR/dirichlet-$geo-$Pk.field - >/dev/null"
   if test $? -ne 0; then status=1; fi
+ done
 done
 
 exit $status
