@@ -21,27 +21,34 @@
 # -------------------------------------------------------------------------
 SRCDIR=${SRCDIR-"."}
 TOP_SRCDIR=${TOP_SRCDIR-"../.."}
+DATADIR=$TOP_SRCDIR/nfem/ptst
 NPROC_MAX=${NPROC_MAX-"3"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
 status=0
 
+# geo			err_P1	err_P2 	err_P3
 L="
-line-20-bdry
-square-10-bdry
-cube-5-bdry
+line-bdry-v2		8e-3	2e-5	3e-6
+carre-bamg-v2		4e-2	6e-4	4e-5
+carre-bamg-q-dom-v2	7e-3	6e-5    2e-6
+carre-tq-10-dom-v2	4e-2	4e-4	3e-5
+cube-dom-v2             4e-1	4e-2	5e-3
+cube-H-6-dom-v2		3e-2	2e-3	5e-5
+cube-P-5-dom-v2		2e-1	6e-3	6e-4
+my_cube_TP-5-v2         3e-1	2e-2	3e-3	
+my_cube_PH-5-v2         7e-2	8e-3	3e-4
+my_cube_TPH-5-v2        2e-1	2e-2	5e-3
 "
-EXPECT="
-1e-6
-4e-4
-2e-3
-"
-
-for geo in $L; do
-  err=`echo $EXPECT | awk '{print $1}'`
-  EXPECT=`echo $EXPECT | awk '{for (i=2; i <= NF; i++) print $i}'`
-  loop_mpirun "./neumann $SRCDIR/$geo P1 2>/dev/null | ./neumann-check $err 2>/dev/null >/dev/null"
-  if test $? -ne 0; then status=1; fi
+while test "$L" != ""; do
+  geo=`echo $L | awk '{print $1}'`
+  L=`echo $L | awk '{for (i=2; i <= NF; i++) print $i}'`
+  for Pk in P1 P2 P3; do
+    err=`echo $L | awk '{print $1}'`
+    L=`echo $L | awk '{for (i=2; i <= NF; i++) print $i}'`
+    loop_mpirun "./neumann $DATADIR/$geo $Pk 2>/dev/null | RHEOPATH=$DATADIR \$RUN ./cosinusprod_error $err 2>/dev/null >/dev/null"
+    if test $? -ne 0; then status=1; fi
+  done
 done
 
 exit $status
