@@ -19,7 +19,7 @@ int main(int argc, char**argv) {
 
   field ph (Qh, 0.);
   form a (Vh, Vh, "2D_D");
-  form b (Vh, Qh, "div");
+  form b (Vh, Qh, "div"); b = -b;
   int   max_iter  = 50;
   Float tol       = 1e-12;
   Float r         = 1e+7;
