@@ -66,7 +66,7 @@ DESCRIPTION:
   @end example
   Such labeled file format could be easily generated
   using the @code{catchmark} stream manipulator
-  (@pxref{iorheo class}).
+  (@pxref{iorheo ialgorithm}).
   @example
       cout << catchmark("u")   << uh
            << catchmark("p")   << ph
