@@ -546,7 +546,8 @@ typedef enum {
 	input_grummp, 
 	input_gmsh, 
 	input_qmg, 
-	input_cemagref
+	input_cemagref,
+	input_peschetola
 } input_type;
 
 typedef enum { 
@@ -568,6 +569,7 @@ int main(int argc, char **argv)
     bool has_geo = false;
     bool do_dump = false;
     bool do_check = false;
+    bool use_tmp = false;
     string domain_names;
     string name = "output";
     string showdom = "";
@@ -594,6 +596,7 @@ int main(int argc, char **argv)
 
              if (strcmp (argv[i], "-input-geo") == 0)      input_format = input_geo;
         else if (strcmp (argv[i], "-input-bamg") == 0)     input_format = input_bamg;
+        else if (strcmp (argv[i], "-input-peschetola") == 0)     input_format = input_peschetola;
         else if (strcmp (argv[i], "-input-tetgen") == 0)   input_format = input_tetgen;
         else if (strcmp (argv[i], "-input-mmg3d") == 0)    input_format = input_mmg3d;
         else if (strcmp (argv[i], "-input-grummp") == 0)   input_format = input_grummp; 
@@ -635,6 +638,7 @@ int main(int argc, char **argv)
 
         else if (strcmp (argv[i], "-check") == 0)     do_check = true;
         else if (strcmp (argv[i], "-dump") == 0)      do_dump = true;
+	else if (strcmp ("-tmp",  argv [i]) == 0)     use_tmp=true; 
         
       // graphic options
 
@@ -691,6 +695,7 @@ int main(int argc, char **argv)
 
 	    // input geo on standard input
 	         if (input_format == input_bamg)     cin >> bamg;
+	    else if (input_format == input_peschetola)  {  cin >> peschetola; cerr << "PES";}
 	    else if (input_format == input_mmg3d)    cin >> mmg3d;
 	    else if (input_format == input_tetgen)   cin >> tetgen;
 	    else if (input_format == input_grummp)   cin >> grummp;
@@ -714,6 +719,7 @@ int main(int argc, char **argv)
 	    usage();
 	}
     }
+    if (use_tmp) { name="/tmp/" + name; cerr << "tmp "<< name; }
     cout << setprecision(digits10);
     if (!has_geo) {
 	cerr << "geo: no input specified" << endl;
@@ -744,12 +750,15 @@ int main(int argc, char **argv)
 	g.dump(cerr);
     }
     if (showdom!="") {
+	domain d;
+	if (showdom=="boundary") d=g.boundary();
+	else {
         // Skip delimiters at beginning.
 	showdom=showdom+"+";
     	string::size_type lastPos = showdom.find_first_not_of("+", 0);
     	// Find first "non-delimiter".
     	string::size_type pos     = showdom.find_first_of("+", lastPos);
-        domain d= g[showdom.substr(lastPos, pos - lastPos)];
+        d= g[showdom.substr(lastPos, pos - lastPos)];
 	while (string::npos != pos && string::npos != lastPos)
 	{
 	    lastPos = showdom.find_first_not_of("+", pos);
@@ -758,6 +767,7 @@ int main(int argc, char **argv)
 	    if (string::npos == pos) break;
             d=d+g[showdom.substr(lastPos, pos - lastPos)];
 	};
+	}
 	g=geo(g, d);
 	if (name=="output") cout << setbasename(showdom+"_from_"+name);
     }
