@@ -162,7 +162,7 @@ typedef union YYSTYPE
         size_type                     uint_value;
         double                        float_value;
         size_type                     string_value;
-        cad_element::enum_type        geo_value;
+        cad_element::variant_type        geo_value;
 
 
 
@@ -1645,7 +1645,7 @@ yyreduce:
 
 /* Line 1455 of yacc.c  */
 #line 227 "cad-qmg-yacc.y"
-    { (yyval.geo_value) = reference_element::max_size; ;}
+    { (yyval.geo_value) = reference_element::max_variant; ;}
     break;
 
   case 40:
@@ -1684,7 +1684,7 @@ yyreduce:
 
 /* Line 1455 of yacc.c  */
 #line 254 "cad-qmg-yacc.y"
-    { p_geoentity -> set_type ((yyvsp[(1) - (1)].geo_value)); ;}
+    { p_geoentity -> set_variant ((yyvsp[(1) - (1)].geo_value)); ;}
     break;
 
   case 43:
@@ -1698,21 +1698,21 @@ yyreduce:
 
 /* Line 1455 of yacc.c  */
 #line 256 "cad-qmg-yacc.y"
-    { p_geoentity -> set_type((yyvsp[(1) - (2)].geo_value),(yyvsp[(2) - (2)].int_value)); ;}
+    { p_geoentity -> set_variant((yyvsp[(1) - (2)].geo_value),(yyvsp[(2) - (2)].int_value)); ;}
     break;
 
   case 46:
 
 /* Line 1455 of yacc.c  */
 #line 258 "cad-qmg-yacc.y"
-    { p_geoentity -> set_type((yyvsp[(1) - (2)].geo_value),(yyvsp[(2) - (2)].int_value)); ;}
+    { p_geoentity -> set_variant((yyvsp[(1) - (2)].geo_value),(yyvsp[(2) - (2)].int_value)); ;}
     break;
 
   case 48:
 
 /* Line 1455 of yacc.c  */
 #line 260 "cad-qmg-yacc.y"
-    { p_geoentity -> set_type((yyvsp[(1) - (3)].geo_value),(yyvsp[(2) - (3)].int_value),(yyvsp[(3) - (3)].int_value)); ;}
+    { p_geoentity -> set_variant((yyvsp[(1) - (3)].geo_value),(yyvsp[(2) - (3)].int_value),(yyvsp[(3) - (3)].int_value)); ;}
     break;
 
   case 52:
