@@ -28,6 +28,7 @@
 # include "rheolef/form.h"
 # include "rheolef/point.h"
 # include "rheolef/asr.h"
+# include "rheolef/banded_level_set.h"
 # include "form_assembly.h"
 # include "field.h"
 # include "geo-connectivity.h"
@@ -673,8 +674,8 @@ form_assembly_discontinuous_galerkin_bdr (
       X.set_global_dof (Kj, idn);
       //we look for the local number of the degrees of liberty in
       //the neighbour element
-      size_t local_n0 = j;
-      size_t local_n1 = (j+1)%K.size();
+      size_t local_n1 = j;
+      size_t local_n0 = (j+1)%K.size();
       //loop over the vertices of K
       for (size_t i = 0 ; i < K.size() ; i++) {
 
@@ -716,4 +717,116 @@ form_assembly_discontinuous_galerkin_bdr (
   a.bu = csr<Float>(abu);
   a.bb = csr<Float>(abb);
 }
+// ======================================================================================
+// assembly on a banded level set: form(Wh,Bh,"mass_s",bh);
+//  where Wh is defined on Gamma_h = {x / phi_h(x) = }
+//  and   Bh is defined on beta_h  = the band containing Gamma_h
+// ======================================================================================
+void
+form_band_assembly (class form& a, const form_element& form_e, const class band& bh)
+{
+    typedef size_t size_type;
+    const space& X = a.get_first_space();
+    const space& Y = a.get_second_space();
+    form_e.set_weight (bh.get_bgd_phi());
+    form_e.set_use_coordinate_system_weight(true);
+    form_e.set_skip_geo_compatibility_check();
+    form_e.initialize (X, Y);
+    //
+    // use temporary dynamic matrix structures
+    //
+    asr<Float> auu(Y.n_unknown(), X.n_unknown());
+    asr<Float> aub(Y.n_unknown(), X.n_blocked());
+    asr<Float> abu(Y.n_blocked(), X.n_unknown());
+    asr<Float> abb(Y.n_blocked(), X.n_blocked());
+    ublas::matrix<Float> ak;
+    tiny_vector<size_type> idy, jdx;
+
+    typedef enum {Wh_Wh, Wh_Bh, Bh_Wh, Bh_Bh} XY_type;
+    XY_type XY_switch;
+    if (a.get_first_space().dimension()  == a.get_first_space().get_geo().map_dimension() +1 &&
+        a.get_second_space().dimension() == a.get_second_space().get_geo().map_dimension()) {
+      XY_switch = Wh_Bh; // extension
+    } else if (a.get_first_space().dimension()  == a.get_first_space().get_geo().map_dimension() &&
+               a.get_second_space().dimension() == a.get_second_space().get_geo().map_dimension()+1) {
+      XY_switch = Bh_Wh; // prolongement
+    } else if (a.get_first_space().dimension()  == a.get_first_space().get_geo().map_dimension() &&
+               a.get_second_space().dimension() == a.get_second_space().get_geo().map_dimension()) {
+      XY_switch = Bh_Bh; // classic banded level set
+    } else {
+      error_macro ("form on a banded level set: at least one of the two spaces may be a band");
+    }
+    space Bh;
+    if (XY_switch == Wh_Bh || XY_switch == Bh_Bh) {
+      Bh = Y;
+    } else { // Bh_Wh
+      Bh = X;
+    }
+    geo::const_iterator iter_element, last_element;
+    if (Bh.is_on_boundary_domain()) {
+      iter_element = Bh.get_boundary_domain().begin();
+      last_element = Bh.get_boundary_domain().end();
+    } else {
+      iter_element = Bh.get_geo().begin();
+      last_element = Bh.get_geo().end();
+    }
+    size_t idx = 0;
+    for (; iter_element != last_element; iter_element++, idx++) {
+      const geo_element& K = *iter_element;
+      // --------------------------------
+      // compute elementary matrix ak
+      // --------------------------------
+      form_e (K, ak);
+      // ------------------------------
+      // assembly ak in sparse matrix a
+      // ------------------------------Z
+      size_t Kx_idx, Ky_idx;
+      if (XY_switch == Wh_Bh) {
+        Ky_idx = K.index();
+	Kx_idx = bh.element2face (idx);
+      } else if (XY_switch == Bh_Wh) {
+        Kx_idx = K.index();
+	Ky_idx = bh.element2face (idx);
+      } else { // Bh_Bh
+        Kx_idx = Ky_idx = K.index();
+      }
+      check_macro (Kx_idx < X.get_global_geo().size(),	
+	"X element index " << Kx_idx << " out of range [0:"<< X.get_global_geo().size() << "[");
+      check_macro (Ky_idx < Y.get_global_geo().size(),
+	"Y element index " << Ky_idx << " out of range [0:"<< Y.get_global_geo().size() << "[");
+      const geo_element& Kx = X.get_global_geo().element(Kx_idx);
+      const geo_element& Ky = Y.get_global_geo().element(Ky_idx);
+      X.set_global_dof (Kx, jdx);
+      Y.set_global_dof (Ky, idy);
+      size_type ny = ak.size1(); // nrow
+      size_type nx = ak.size2(); // ncol
+      for (size_type i = 0; i < ny; i++) {
+        for (size_type j = 0; j < nx; j++) {
+
+	    if (ak(i,j) == Float(0)) continue;
+
+	    size_type i_dof = Y.domain_dof(idy(i));
+	    size_type j_dof = X.domain_dof(jdx(j));
+
+	    size_type ii    = Y.index(i_dof);
+	    size_type jj    = X.index(j_dof);
+   
+	    if   (Y.is_blocked(i_dof))
+	      if (X.is_blocked(j_dof))    abb.entry(ii, jj) += ak(i,j);
+              else                        abu.entry(ii, jj) += ak(i,j);
+	    else 
+	      if (X.is_blocked(j_dof))    aub.entry(ii, jj) += ak(i,j);
+              else                        auu.entry(ii, jj) += ak(i,j);
+        }
+      }
+    }
+    //
+    // convert dynamic matrix to fixed-size one
+    //
+    a.uu = csr<Float>(auu);
+    a.ub = csr<Float>(aub);
+    a.bu = csr<Float>(abu);
+    a.bb = csr<Float>(abb);
+}
+
 }// namespace rheolef
