@@ -304,7 +304,6 @@ hazel::grow
             }
         e++; e_index++;
      }
-
     grown =true;
  }
  else error_macro ( "Trying to overgrow hazel tree.\n" );
@@ -329,7 +328,7 @@ hazel::~hazel    ()
 bool
 hazel::in_bbox_element (const geo_element& e, const point& x) const
  {
-    switch(e.type())
+    switch(e.variant())
      {
         case geo_element::p:
          {
@@ -399,7 +398,7 @@ bool
 hazel::in_interior_element (const geo_element& e, const point& x, bool& not_too_far) const
 {
   not_too_far = false;
-  switch(e.type()) {
+  switch(e.variant()) {
     case geo_element::p: {
       error_macro("Wrong element dimension");
     }
@@ -441,7 +440,7 @@ hazel::hit_boundary_element (const geo_element& e,
                             point& x, Float& t, bool& inward) const
  {
     point x1 = x0 +v;
-    switch(e.type())
+    switch(e.variant())
      {
         case geo_element::p:
          {
@@ -819,7 +818,7 @@ hazel::localize    (const point& x, geo_element::size_type& element) const
 bool
 hazel::trace (const point& x0, const point& v, point& x, Float& t, size_type& element) const
  {
-     if (!grown) error_macro("Hazel tree must be grown before it is used.");
+    if (!grown) error_macro("Hazel tree must be grown before it is used.");
     int    side, dir;
     bool  got_one =false;
     bool  inward;
