@@ -23,91 +23,37 @@
 
 #include "rheolef/array.h"
 #include "rheolef/geo_element.h"
-
-namespace rheolef {
-
-// =====================================================================
-// 1) domain pair = pair (sign orientation ; size_t element_index)
-// =====================================================================
-// TODO: compact the sign bit and size_t in a long int
-struct domain_pair_type {
-
-// typedefs:
-
-    typedef array<size_t>::size_type      size_type;
-    typedef geo_element::orientation_type orientation_type;
-
-// allocators:
-
-    domain_pair_type () : _pos(true), _ige (std::numeric_limits<size_type>::max()) {}
-    domain_pair_type (orientation_type orient, size_type ige) : _pos(orient > 0), _ige(ige) {}
-
-// accessors:
-
-    orientation_type orientation() const { return (_pos ?  1 : - 1); }
-    size_type        index()       const { return _ige; }
-
-// modifiers:
-
-    void set_orientation (orientation_type orient) { _pos = (orient > 0); }
-    void set_index       (size_type ige)           { _ige = ige; }
-    void set (orientation_type orient, size_type ige) {
-        set_orientation (orient);
-        set_index       (ige);
-    }
-
-// i/o:
-
-    friend std::istream&  operator>> (std::istream& is, domain_pair_type& x);
-    friend std::ostream&  operator<< (std::ostream& os, const domain_pair_type& x);
-    template<class Archive>
-    void serialize (Archive& ar, const unsigned int version) { ar & _pos; ar & _ige; }
-
-// data:
-protected:
-    bool      _pos;
-    size_type _ige;
-};
-
-} // namespace rheolef
-
-#ifdef _RHEOLEF_HAVE_MPI
-// =====================================================================
-// Some serializable types, have a fixed amount of data stored at fixed field positions.
-// When this is the case, boost::mpi can optimize their serialization and transmission to avoid extraneous 
-// copy operations.
-// To enable this optimization, we specialize the type trait is_mpi_datatype, e.g.:
-namespace boost {
- namespace mpi {
-  template <> struct is_mpi_datatype<rheolef::domain_pair_type> : mpl::true_ { };
- } // namespace mpi
-} // namespace boost
-#endif // _RHEOLEF_HAVE_MPI
+#include "rheolef/geo_element_indirect.h"
+#include "rheolef/index_set.h"
 
 namespace rheolef {
 
 // foward declarations:
 template <class U, class M> class geo_rep;
+template <class U, class M> class geo_basic;
 
 // ==================================================================================
-// 2) domain_indirect
-// 2.1) representation: domain_indirect_base_rep<M> and domain_indirect_rep<M>
+// 1) domain_indirect
+//    representation: domain_indirect_base_rep<M> and domain_indirect_rep<M>
 // ==================================================================================
 template <class M>
-class domain_indirect_base_rep : public array<domain_pair_type,M> {
+class domain_indirect_base_rep : public array<geo_element_indirect,M> {
 public:
 
 // typedefs:
 
-    typedef array<domain_pair_type,M> base;
-    typedef typename domain_pair_type::size_type        size_type;
-    typedef typename domain_pair_type::orientation_type orientation_type;
+    typedef array<geo_element_indirect,M> base;
+    typedef typename geo_element_indirect::size_type        size_type;
+    typedef typename geo_element_indirect::orientation_type orientation_type;
     typedef typename base::iterator                     iterator_ioige;
     typedef typename base::const_iterator               const_iterator_ioige;
 
 // allocators:
 
     domain_indirect_base_rep();
+    domain_indirect_base_rep (const std::string& name, size_type map_dim,
+	const communicator&           comm,
+	const std::vector<size_type>& ie_list);
     void resize (size_type n) { base::resize (n); }
 
 // accessors & modifiers:
@@ -120,7 +66,7 @@ public:
           iterator_ioige ioige_begin()       { return base::begin(); }
           iterator_ioige ioige_end()         { return base::end(); }
 
-    const domain_pair_type& oige (size_type ioige) const { return base::operator[] (ioige); }
+    const geo_element_indirect& oige (size_type ioige) const { return base::operator[] (ioige); }
 
     std::string name ()    const { return _name; }
     size_type map_dimension () const { return _map_dim; }
@@ -135,7 +81,7 @@ protected:
 template <class M>
 inline
 domain_indirect_base_rep<M>::domain_indirect_base_rep()
- : array<domain_pair_type,M>(),
+ : array<geo_element_indirect,M>(),
    _name(),
    _map_dim(0)
 {
@@ -158,6 +104,14 @@ public:
 // allocators:
 
     domain_indirect_rep () : domain_indirect_base_rep<sequential>() {}
+
+    domain_indirect_rep (
+        const std::string&            name,
+        size_type                     map_dim,
+	const communicator&           comm,
+	const std::vector<size_type>& ie_list)
+     : domain_indirect_base_rep<sequential>(name, map_dim, comm, ie_list) {}
+
     void resize (size_type n) { base::resize (n); }
 
 // accessors:
@@ -170,7 +124,7 @@ public:
           iterator_ioige ioige_begin()       { return base::ioige_begin(); }
           iterator_ioige ioige_end()         { return base::ioige_end(); }
 
-    const domain_pair_type& oige (size_type ioige) const {
+    const geo_element_indirect& oige (size_type ioige) const {
 	return base::oige (ioige); }
 
     void set_name (std::string name)   { base::set_name(name); }
@@ -180,9 +134,18 @@ public:
 
 // i/o:
 
+    template<class U>
+    void build_from_data (
+        const geo_rep<U,sequential>&  omega,
+        array<geo_element_auto<heap_allocator<size_type> >,sequential, heap_allocator<size_type> >&
+                                      d_tmp,
+        std::vector<index_set>*       ball);
+
     template <class U>
-    idiststream& get (idiststream& ips, const geo_rep<U,sequential>& omega,
-    	std::vector<std::set<size_type> > ball[4]);
+    idiststream& get (
+        idiststream&                  ips,
+        const geo_rep<U,sequential>&  omega,
+        std::vector<index_set>*       ball);
 
     odiststream& put (odiststream& ops) const;
 };
@@ -203,13 +166,20 @@ public:
 // allocators:
 
     domain_indirect_rep();
+    template<class T>
+    domain_indirect_rep (
+        const geo_basic<T,distributed>& omega,
+        const std::string&            name,
+        size_type                     map_dim,
+	const communicator&           comm,
+	const std::vector<size_type>& ie_list);
 
 // accessors & modifiers:
 
     size_type size()       const { return base::size(); }
     size_type dis_size()   const { return base::dis_size(); }
 
-    const domain_pair_type& oige (size_type ioige) const {
+    const geo_element_indirect& oige (size_type ioige) const {
 	return base::oige (ioige); }
 
     void set_name (std::string name)   { base::set_name(name); }
@@ -235,19 +205,17 @@ protected:
 // data:
     array<size_type,distributed>        _ioige2ini_dis_ioige;
     array<size_type,distributed>        _ini_ioige2dis_ioige;
-    size_type		                _ini_size_by_variant [reference_element::max_size];
 };
 inline
 domain_indirect_rep<distributed>::domain_indirect_rep()
   : domain_indirect_base_rep<distributed>(),
     _ioige2ini_dis_ioige(),
-    _ini_ioige2dis_ioige(),
-    _ini_size_by_variant()
+    _ini_ioige2dis_ioige()
 {
 }
 #endif // _RHEOLEF_HAVE_MPI
 // ====================================================================
-// 2.2) wrapper: domain_indirect_basic<M>
+// 2) wrapper: domain_indirect_basic<M>
 // ====================================================================
 /*Class:domain_indirect
 NAME: @code{domain_indirect} - a named part of a finite element mesh
@@ -301,6 +269,22 @@ public:
 // allocators:
 
     domain_indirect_basic ();
+ 
+    template <class T>
+    domain_indirect_basic (
+        const geo_basic<T,sequential>&  omega,
+        const std::string&              name, 
+        size_type                       map_dim,
+	const communicator&             comm,
+	const std::vector<size_type>&   ie_list);
+
+    template <class U>
+    domain_indirect_basic (
+        array<geo_element_auto<heap_allocator<size_type> >,sequential, heap_allocator<size_type> >&
+                                        d_tmp,
+	const geo_basic<U, sequential>& omega,
+	std::vector<index_set>*         ball);
+
     void resize (size_type n);
 
 // accessors:
@@ -314,7 +298,7 @@ public:
           iterator_ioige ioige_begin();
           iterator_ioige ioige_end();
 
-    const domain_pair_type& oige (size_type ioige) const;
+    const geo_element_indirect& oige (size_type ioige) const;
 
     void set_name (std::string name);
     void set_map_dimension (size_type map_dim);
@@ -325,13 +309,23 @@ public:
 
     odiststream& put (odiststream&) const;
     template <class T>
-    idiststream& get (idiststream& ips, const geo_rep<T,sequential>& omega,
-    	std::vector<std::set<size_type> > ball[4]);
+    idiststream& get (idiststream& ips, const geo_rep<T,sequential>& omega, std::vector<index_set> *ball);
 };
 //>verbatim:
 inline
 domain_indirect_basic<sequential>::domain_indirect_basic ()
- : smart_pointer<domain_indirect_rep<sequential> > (new_macro(domain_indirect_rep<sequential>))
+ : smart_pointer<rep> (new_macro(rep))
+{
+}
+template<class T>
+inline
+domain_indirect_basic<sequential>::domain_indirect_basic (
+    const geo_basic<T,sequential>& omega,
+    const std::string&            name,
+    size_type                     map_dim,
+    const communicator&           comm,
+    const std::vector<size_type>& ie_list)
+ : smart_pointer<rep> (new_macro(rep(name, map_dim, comm, ie_list)))
 {
 }
 inline
@@ -359,7 +353,7 @@ domain_indirect_basic<sequential>::ownership() const
   return base::data().ownership();
 }
 inline
-const domain_pair_type&
+const geo_element_indirect&
 domain_indirect_basic<sequential>::oige (size_type ioige) const
 {
   return base::data().oige (ioige);
@@ -422,9 +416,9 @@ template<class T>
 inline
 idiststream&
 domain_indirect_basic<sequential>::get (
-    idiststream&                       ips,
+    idiststream&                      ips,
     const geo_rep<T,sequential>&      omega,
-    std::vector<std::set<size_type> > ball[4])
+    std::vector<index_set>           *ball)
 {
   return base::data().template get (ips, omega, ball);
 }
@@ -444,6 +438,13 @@ public:
 // allocators:
 
     domain_indirect_basic ();
+    template<class T>
+    domain_indirect_basic (
+        const geo_basic<T,distributed>& omega,
+        const std::string&            name,
+        size_type                     map_dim,
+	const communicator&           comm,
+	const std::vector<size_type>& ie_list);
 
 // accessors/modifiers:
 
@@ -451,7 +452,7 @@ public:
     size_type dis_size() const;
     const distributor& ownership() const;
 
-    const domain_pair_type& oige (size_type ioige) const;
+    const geo_element_indirect& oige (size_type ioige) const;
 
     void set_name (std::string name);
     void set_map_dimension (size_type map_dim);
@@ -478,6 +479,17 @@ domain_indirect_basic<distributed>::domain_indirect_basic ()
  : smart_pointer<rep> (new_macro(rep))
 {
 }
+template<class T>
+inline
+domain_indirect_basic<distributed>::domain_indirect_basic (
+    const geo_basic<T,distributed>& omega,
+    const std::string&            name,
+    size_type                     map_dim,
+    const communicator&           comm,
+    const std::vector<size_type>& ie_list)
+ : smart_pointer<rep> (new_macro(rep(omega, name, map_dim, comm, ie_list)))
+{
+}
 inline
 domain_indirect_basic<distributed>::size_type
 domain_indirect_basic<distributed>::size() const
@@ -539,7 +551,7 @@ domain_indirect_basic<distributed>::set_map_dimension (size_type map_dim)
   return base::data().set_map_dimension (map_dim);
 }
 inline
-const domain_pair_type&
+const geo_element_indirect&
 domain_indirect_basic<distributed>::oige (size_type ioige) const
 {
   return base::data().oige (ioige);
