@@ -29,6 +29,7 @@
 // date: 23 february 2011
 //
 #include "rheolef/field.h"
+#include "rheolef/field_component.h"
 #include "rheolef/promote.h"
 #include "rheolef/dis_accumulate.h"
 #include "rheolef/dis_inner_product.h"
@@ -53,140 +54,14 @@ using proto::_;
 template <class Expr> struct field_expr;
 
 // -----------------------------------------------------------
-// Here is an evaluation context that indexes into a field
-// expression and combines the result:
-// 	expr.dof(i)
-// -----------------------------------------------------------
-struct field_subscript_context {
-    typedef std::size_t size_type;
-    field_subscript_context (size_type i) : _i(i) {}
-    // Unless this is a field terminal, use the default evaluation context
-    template<class Expr, class EnableIf = void>
-    struct eval : proto::default_eval<Expr, const field_subscript_context> {};
-    // Index field terminals with our subscript.
-    template<class Expr>
-    struct eval<Expr, typename boost::enable_if<
-        mpl::or_<
-            proto::matches<Expr, proto::terminal<field_basic<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
-        >
-      >::type>
-    {
-             typedef typename proto::result_of::value<Expr>::type::value_type result_type;
-             result_type operator() (Expr& expr, const field_subscript_context& ctx) const
-             {
-                 return proto::value(expr).dof(ctx._i);
-             }
-    };
-    size_type _i;
-};
-// -----------------------------------------------------------
-// get size
-// 	expr.size()
-// -----------------------------------------------------------
-struct field_get_size_context {
-    typedef std::size_t size_type;
-// allocator:
-    field_get_size_context () {}
-
-// eval:
-    template<class Expr, class EnableIf = void>
-    struct eval : proto::null_eval<Expr, const field_get_size_context> {};
-
-    template<class Expr>
-    struct eval<Expr, typename boost::enable_if<
-        mpl::or_<
-            proto::matches<Expr, proto::terminal<field_basic<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
-        >
-      >::type>
-    {
-        typedef void result_type;
-        result_type operator() (Expr& expr, const field_get_size_context& ctx) const
-        {
-            ctx._ownership = proto::value(expr).ownership();
-        }
-    };
-// accessors:
-    size_type        size() const { return _ownership.size(); };
-    distributor ownership() const { return _ownership; };
-// data:
-    mutable distributor _ownership;
-};
-// -----------------------------------------------------------
-// get space
-// 	expr.get_space()
-// -----------------------------------------------------------
-template<class T, class M>
-struct field_get_space_context {
-    typedef typename field_basic<T,M>::space_type space_type;
-// allocator:
-    field_get_space_context () {}
-
-// eval:
-    template<class Expr, class EnableIf = void>
-    struct eval : proto::null_eval<Expr, const field_get_space_context<T,M> > {};
-
-    template<class Expr>
-    struct eval<Expr, typename boost::enable_if<
-        mpl::or_<
-            proto::matches<Expr, proto::terminal<field_basic<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
-        >
-      >::type>
-    {
-        typedef void result_type;
-        result_type operator() (Expr &expr, const field_get_space_context<T,M>& ctx) const
-        {
-            ctx._V = proto::value(expr).get_space();
-        }
-    };
-// accessors:
-    space_type get_space() const { return _V; };
-// data:
-    mutable space_type _V;
-};
-// -----------------------------------------------------------
-// check that stamp match
-// 	expr.get_stamp()
-// -----------------------------------------------------------
-struct field_check_stamp_context {
-    field_check_stamp_context (std::string stamp) : _stamp(stamp) {}
-
-    template<class Expr, class EnableIf = void>
-    struct eval : proto::null_eval<Expr, const field_check_stamp_context> {};
-
-    template<class Expr>
-    struct eval<Expr, typename boost::enable_if<
-        mpl::or_<
-            proto::matches<Expr, proto::terminal<field_basic<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect<_, _> > >,
-            proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
-        >
-      >::type>
-    {
-        typedef void result_type;
-        result_type operator() (Expr &expr, const field_check_stamp_context& ctx) const
-        {
-            check_macro (ctx._stamp == proto::value(expr).get_stamp(),
-                "incompatible spaces " << ctx._stamp << " and " 
-		  << proto::value(expr).get_stamp() << " in field expression ");
-        }
-    };
-    std::string _stamp;
-};
-// -----------------------------------------------------------
-// iterator begin
-// 	expr.begin()
+// iterator:
+// 	expr.begin_dof()
 // -----------------------------------------------------------
 template<class Iterator>
 struct field_iterator_wrapper {
     typedef Iterator iterator;
     explicit field_iterator_wrapper (Iterator iter1) : iter(iter1) {} 
-    Iterator iter;
+    mutable Iterator iter;
 };
 struct field_begin : proto::callable {
     template<class Sig>
@@ -202,7 +77,7 @@ struct field_begin : proto::callable {
     template<class Container>
     typename result<field_begin(const Container&)>::type
     operator ()(const Container& cont) const {
-        field_iterator_wrapper<typename Container::const_iterator> iter (cont.begin());
+        field_iterator_wrapper<typename Container::const_iterator> iter (cont.begin_dof());
         return proto::as_expr (iter);
     }
 };
@@ -211,6 +86,8 @@ struct field_grammar_begin : proto::or_<
         proto::when <proto::terminal<field_basic<_, _> >,                field_begin(proto::_value)>
       , proto::when <proto::terminal<field_indirect<_, _> >,             field_begin(proto::_value)>
       , proto::when <proto::terminal<field_indirect_const<_, _> >,       field_begin(proto::_value)>
+      , proto::when <proto::terminal<field_component<_, _> >,            field_begin(proto::_value)>
+      , proto::when <proto::terminal<field_component_const<_, _> >,      field_begin(proto::_value)>
       , proto::when <proto::terminal<_> >
       , proto::when <proto::nary_expr<_, proto::vararg<field_grammar_begin> > >
     >
@@ -259,6 +136,151 @@ struct field_increment_context {
     };
 };
 // -----------------------------------------------------------
+// Here is an evaluation context that indexes into a field
+// expression and combines the result:
+// 	expr.dof(i)
+// -----------------------------------------------------------
+struct field_subscript_context {
+    typedef std::size_t size_type;
+    field_subscript_context (size_type i) : _i(i) {}
+    // Unless this is a field terminal, use the default evaluation context
+    template<class Expr, class EnableIf = void>
+    struct eval : proto::default_eval<Expr, const field_subscript_context> {};
+    // Index field terminals with our subscript.
+    template<class Expr>
+    struct eval<Expr, typename boost::enable_if<
+        mpl::or_<
+            proto::matches<Expr, proto::terminal<field_basic<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_component<_, _> > >
+	   ,proto::matches<Expr, proto::terminal<field_component_const<_, _> > >
+        >
+      >::type>
+    {
+             typedef typename proto::result_of::value<Expr>::type::value_type result_type;
+             result_type operator() (Expr& expr, const field_subscript_context& ctx) const
+             {
+                 return proto::value(expr).dof(ctx._i);
+             }
+    };
+    // Index iterator on field terminals
+    template<class Expr>
+    struct eval<Expr, typename boost::enable_if<
+         proto::matches<Expr, proto::terminal<field_iterator_wrapper<_> > > >::type > {
+        typedef typename proto::result_of::value<Expr>::type IteratorWrapper;
+        typedef typename IteratorWrapper::iterator iterator;
+        typedef typename std::iterator_traits<iterator>::reference result_type;
+        result_type operator() (Expr &expr, const field_subscript_context& ctx) const {
+            return proto::value(expr).iter [ctx._i];
+        }
+    };
+    mutable size_type _i;
+};
+// -----------------------------------------------------------
+// get size
+// 	expr.ndof()
+// -----------------------------------------------------------
+struct field_get_size_context {
+    typedef std::size_t size_type;
+// allocator:
+    field_get_size_context () {}
+
+// eval:
+    template<class Expr, class EnableIf = void>
+    struct eval : proto::null_eval<Expr, const field_get_size_context> {};
+
+    template<class Expr>
+    struct eval<Expr, typename boost::enable_if<
+        mpl::or_<
+            proto::matches<Expr, proto::terminal<field_basic<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_component<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_component_const<_, _> > >
+        >
+      >::type>
+    {
+        typedef void result_type;
+        result_type operator() (Expr& expr, const field_get_size_context& ctx) const
+        {
+            ctx._ownership = proto::value(expr).ownership();
+        }
+    };
+// accessors:
+    size_type        ndof() const { return _ownership.size(); };
+    distributor ownership() const { return _ownership; };
+// data:
+    mutable distributor _ownership;
+};
+// -----------------------------------------------------------
+// get space
+// 	expr.get_space()
+// -----------------------------------------------------------
+template<class T, class M>
+struct field_get_space_context {
+    typedef typename field_basic<T,M>::space_type space_type;
+// allocator:
+    field_get_space_context () {}
+
+// eval:
+    template<class Expr, class EnableIf = void>
+    struct eval : proto::null_eval<Expr, const field_get_space_context<T,M> > {};
+
+    template<class Expr>
+    struct eval<Expr, typename boost::enable_if<
+        mpl::or_<
+            proto::matches<Expr, proto::terminal<field_basic<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_component<_, _> > >
+	   ,proto::matches<Expr, proto::terminal<field_component_const<_, _> > >
+        >
+      >::type>
+    {
+        typedef void result_type;
+        result_type operator() (Expr &expr, const field_get_space_context<T,M>& ctx) const
+        {
+            ctx._V = proto::value(expr).get_space();
+        }
+    };
+// accessors:
+    space_type get_space() const { return _V; };
+// data:
+    mutable space_type _V;
+};
+// -----------------------------------------------------------
+// check that stamp match
+// 	expr.stamp()
+// -----------------------------------------------------------
+struct field_check_stamp_context {
+    field_check_stamp_context (std::string stamp) : _stamp(stamp) {}
+
+    template<class Expr, class EnableIf = void>
+    struct eval : proto::null_eval<Expr, const field_check_stamp_context> {};
+
+    template<class Expr>
+    struct eval<Expr, typename boost::enable_if<
+        mpl::or_<
+            proto::matches<Expr, proto::terminal<field_basic<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_indirect_const<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_component<_, _> > >
+           ,proto::matches<Expr, proto::terminal<field_component_const<_, _> > >
+        >
+      >::type>
+    {
+        typedef void result_type;
+        result_type operator() (Expr &expr, const field_check_stamp_context& ctx) const
+        {
+            check_macro (ctx._stamp == proto::value(expr).stamp(),
+                "incompatible spaces " << ctx._stamp << " and " 
+		  << proto::value(expr).stamp() << " in field expression ");
+        }
+    };
+    mutable std::string _stamp;
+};
+// -----------------------------------------------------------
 // A grammar which matches all the assignment operators,
 // so we can easily disable them.
 // -----------------------------------------------------------
@@ -298,7 +320,6 @@ struct field_expr : proto::extends<Expr, field_expr<Expr>, field_domain> {
 // typedefs :
 
     typedef field_get_size_context::size_type size_type;
-    typedef typename proto::result_of::eval<const Expr, const field_subscript_context>::type element_type;
     typedef typename boost::result_of<field_grammar_begin(const Expr&)>::type raw_iterator;
 
 // allocators:
@@ -316,16 +337,12 @@ struct field_expr : proto::extends<Expr, field_expr<Expr>, field_domain> {
         proto::eval (*this, get_size);
         return get_size.ownership();
     }
-    size_type size() const {
+    size_type ndof() const {
         return ownership().size();
     }
-    size_type dis_size() const {
+    size_type dis_ndof() const {
         return ownership().dis_size();
     }
-    element_type dof (size_type i) const {
-        const field_subscript_context get_subscript(i);
-        return proto::eval(*this, get_subscript);
-    }
     struct const_iterator_begin : raw_iterator {
 
        typedef std::input_iterator_tag        iterator_category;
@@ -334,9 +351,16 @@ struct field_expr : proto::extends<Expr, field_expr<Expr>, field_domain> {
        typedef value_type* pointer;
        typedef std::ptrdiff_t difference_type;
 
-       const_iterator_begin (const field_expr<Expr>& expr) 
-	 : raw_iterator(field_grammar_begin() (expr)) {
-       }
+#if BOOST_VERSION < 104601
+       const_iterator_begin (const field_expr<Expr>& expr)
+         : raw_iterator(field_grammar_begin() (expr))
+       {}
+#else // BOOST_VERSION < 104601
+       const_iterator_begin (const field_expr<Expr>& expr)
+         : raw_iterator(field_grammar_begin() (expr.proto_expr_))
+           // ICI: il faut recuperer base_type(expr) de type Expr au lieu field_expr<Expr>
+       {}
+#endif // BOOST_VERSION < 104601
        const_iterator_begin& operator++ () { 	
          const field_increment_context inc = {};
          proto::eval (*this, inc);
@@ -355,17 +379,35 @@ struct field_expr : proto::extends<Expr, field_expr<Expr>, field_domain> {
        }
     };
     typedef const_iterator_begin const_iterator;
-    const_iterator begin() const { return const_iterator(*this); }
+    const_iterator begin_dof() const { return const_iterator(*this); }
 
+// random access:
+    typedef typename proto::result_of::eval<const Expr, const field_subscript_context>::type value_type;
+    value_type dof (size_type i) const {
+        const field_subscript_context get_subscript(i);
+        return proto::eval(*this, get_subscript);
+    }
+
+#ifdef TODO
+    value_type max () const {
+      value_type val = std::numeric_limits<value_type>::min();
+      for (const_iterator iter = begin_dof(), last = end_dof(); iter != last; iter++) {
+        val = std::max(val, *iter);
+      }
+      return val;
+    }
+    value_type max_abs () const { return abs(*this).max(); }
+    value_type min_abs () const { return abs(*this).min(); }
+#endif // TODO
 private:
     // hide this:
     using proto::extends<Expr, field_expr<Expr>, field_domain>::operator[];
 };
 template <class Expr>
 inline
-odiststream& operator << (odiststream& ops, const field_expr<Expr>& uh)
+odiststream& operator<< (odiststream& ops, const field_expr<Expr>& uh)
 {
-    typedef typename field_expr<Expr>::element_type T;
+    typedef typename field_expr<Expr>::value_type T;
     field_basic<T> tmp = uh; // TODO: add M as <T,M>
     return tmp.put (ops);
 }
@@ -383,6 +425,12 @@ struct is_field<field_indirect<T, M> > : mpl::true_ {};
 template<class T, class M>
 struct is_field<field_indirect_const<T, M> > : mpl::true_ {};
 
+template<class T, class M>
+struct is_field<field_component<T, M> > : mpl::true_ {};
+
+template<class T, class M>
+struct is_field<field_component_const<T, M> > : mpl::true_ {};
+
 // -------------------------------------------
 // x = expr
 // -------------------------------------------
@@ -408,21 +456,36 @@ template<class Expr>
 inline
 field_basic<T,M>& 
 field_basic<T,M>::operator= (const field_expr<Expr>& expr) {
+  dis_dof_update_needed();
+  // get and check sizes:
+  const field_get_size_context get_size;
+  proto::eval (proto::as_expr<field_domain>(expr), get_size);
+  size_type   expr_size      = get_size.ndof();
+  if (stamp() == "") {
+    const field_get_space_context<T,M> get_space;
+    proto::eval (proto::as_expr<field_domain>(expr), get_space);
+    space_type V = get_space.get_space();
+    resize (V);
+  } else {
+    const field_check_stamp_context check_stamp (stamp());
+    proto::eval (proto::as_expr<field_domain>(expr), check_stamp); // error if the spaces arent compatibles
+  }
+  // perform all computations here:
+  field_detail::evaluate (begin_dof(), end_dof(), proto::as_expr<field_domain>(expr), field_detail::assign_op());
+  return *this;
+}
+template<class T, class M>
+template<class Expr>
+inline
+field_component<T,M>& 
+field_component<T,M>::operator= (const field_expr<Expr>& expr) {
     // get and check sizes:
     const field_get_size_context get_size;
     proto::eval (proto::as_expr<field_domain>(expr), get_size);
-    size_type   expr_size      = get_size.size();
-    if (get_stamp() == "") {
-        const field_get_space_context<T,M> get_space;
-        proto::eval (proto::as_expr<field_domain>(expr), get_space);
-        space_type V = get_space.get_space();
-	resize (V);
-    } else {
-        const field_check_stamp_context check_stamp (get_stamp());
-        proto::eval (proto::as_expr<field_domain>(expr), check_stamp); // error if the spaces arent compatibles
-    }
-    // perform all computations here:
-    field_detail::evaluate (begin(), end(), proto::as_expr<field_domain>(expr), field_detail::assign_op());
+    size_type   expr_size      = get_size.ndof();
+    const field_check_stamp_context check_stamp (stamp());
+    proto::eval (proto::as_expr<field_domain>(expr), check_stamp); // error if the spaces arent compatibles
+    field_detail::evaluate (begin_dof(), end_dof(), proto::as_expr<field_domain>(expr), field_detail::assign_op());
     return *this;
 }
 template<class T, class M>
@@ -430,8 +493,9 @@ template<class Expr>
 inline
 field_basic<T,M>::field_basic (const field_expr<Expr>& expr)
  : _V (),
-   u (),
-   b ()
+   _u (),
+   _b (),
+   _dis_dof_update_needed(true)
 {
     operator= (expr);
 }
