@@ -25,8 +25,16 @@
 #include "rheolef/field.h"
 #include "rheolef/form_element.h"
 #include "rheolef/scalar_traits.h"
+#include "rheolef/quadrature.h"
 namespace rheolef { 
 
+// these classes are used for allocator from the std::initializer_list
+template <class T, class M> class form_concat_value;
+template <class T, class M> class form_concat_line;
+
+// forward declaration:
+template <class T, class M> class band_basic;
+
 /*Class:form
 NAME:  @code{form} - representation of a finite element bilinear form
 DESCRIPTION:       
@@ -42,10 +50,11 @@ DESCRIPTION:
        A: U  ----> V'
           u  |---> A(u)
   @end example
-  where @code{A(u)} is such that @code{a(u,v)=<A(u),v>}
+  where @code{u} and @code{v} are fields (@pxref{field class}),
+  and @code{A(u)} is such that @code{a(u,v)=<A(u),v>}
   for all u in U and v in V and where @code{<.,.>} denotes
-  the duality product between V and V'. Since V is a
-  finite dimensional spaces, the duality product is
+  the duality product between V and V'.
+  Since V is a finite dimensional spaces, the duality product is
   the euclidian product in IR^dim(V).
 
   Since both U and V are finite dimensional spaces,
@@ -56,7 +65,7 @@ DESCRIPTION:
   of origin and destination spaces (@pxref{space class}).
 EXAMPLE:
   
-The operator A associated to a bilinear form a(.,.) by
+  The operator A associated to a bilinear form a(.,.) by
   the relation (Au,v) = a(u,v) could be applied by using
   a sample matrix notation A*u, as shown by the following code:
   @example
@@ -68,12 +77,18 @@ The operator A associated to a bilinear form a(.,.) by
       cout << v;
   @end example
 
-  The block-matrix @code{v=a*u} operation is equivalent to:
+  The form-field @code{vh=a*uh} operation is equivalent to
+  the following matrix-vector operations:
   @example
-     vh.u = a.uu*uh.u + a.ub*uh.b;
-     vh.b = a.bu*uh.u + a.bb*uh.b;
+     vh.set_u() = a.uu()*uh.u() + a.ub()*uh.b();
+     vh.set_b() = a.bu()*uh.u() + a.bb()*uh.b();
   @end example
-
+ALGABRA:
+ Forms, as matrices (see @ref{csr class}), support linear algebra:
+ Adding or substracting two forms writes @code{a+b} and @code{a-b}, respectively,
+ and multiplying a form by a field @code{uh} writes @code{a*uh}.
+ Thus, any linear combination of forms is available.
+ 
 SEE ALSO: "space"(3), "field"(3), "csr"(3)
 AUTHOR: Pierre.Saramito@imag.fr
 DATE:   2 july 1997
@@ -96,8 +111,33 @@ public :
 // allocator/deallocator:
 
     form_basic ();
-    form_basic (const space_type& X, const space_type& Y, const std::string& name);
-    form_basic (const space_type& X, const space_type& Y, const std::string& name, const geo_basic<T,M>& gamma);
+    form_basic (const form_basic<T,M>&);
+
+    form_basic (const space_type& X, const space_type& Y,
+	const std::string& name = "",
+	quadrature_option_type  qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+
+    form_basic (const space_type& X, const space_type& Y,
+	const std::string& name,
+	const geo_basic<T,M>& gamma,
+	quadrature_option_type  qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+
+    form_basic (const space_type& X, const space_type& Y,
+	const std::string& name,
+	const field_basic<T,M>& weight, 
+	quadrature_option_type  qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+
+    form_basic (const space_type& X, const space_type& Y,
+	const std::string& name,
+	const band_basic<T,M>& bh, 
+	quadrature_option_type  qopt = quadrature_option_type(quadrature_option_type::max_family,0));
+
+// allocators from initializer list (c++ 2011):
+
+#ifdef _RHEOLEF_HAVE_STD_INITIALIZER_LIST
+    form_basic (const std::initializer_list<form_concat_value<T,M> >& init_list);
+    form_basic (const std::initializer_list<form_concat_line <T,M> >& init_list);
+#endif // _RHEOLEF_HAVE_STD_INITIALIZER_LIST
 
 // accessors:
 
@@ -109,9 +149,15 @@ public :
 
 // linear algebra:
 
-    form_basic<T,M>  operator+ (const form_basic<T,M>& b) const;
-    form_basic<T,M>  operator- (const form_basic<T,M>& b) const;
-    field_basic<T,M> operator* (const field_basic<T,M>& xh) const;
+    form_basic<T,M>  operator+  (const form_basic<T,M>& b) const;
+    form_basic<T,M>  operator-  (const form_basic<T,M>& b) const;
+    form_basic<T,M>& operator*= (const T& lambda);
+    field_basic<T,M> operator*  (const field_basic<T,M>& xh) const;
+#ifdef TO_CLEAN
+    template <class Expr>
+    field_basic<T,M> operator*  (const field_expr<Expr>& xh) const;
+#endif // TO_CLEAN
+    field_basic<T,M> trans_mult (const field_basic<T,M>& yh) const;
 
     float_type operator () (const field_basic<T,M>& uh, const field_basic<T,M>& vh) const;
 
@@ -120,23 +166,39 @@ public :
     odiststream& put (odiststream& ops, bool show_partition = true) const;
     void dump (std::string name) const;
 
+// accessors & modifiers to unknown & blocked parts:
+
+    const csr<T,M>&     uu() const { return _uu; }
+    const csr<T,M>&     ub() const { return _ub; }
+    const csr<T,M>&     bu() const { return _bu; }
+    const csr<T,M>&     bb() const { return _bb; }
+          csr<T,M>& set_uu()       { return _uu; }
+          csr<T,M>& set_ub()       { return _ub; }
+          csr<T,M>& set_bu()       { return _bu; }
+          csr<T,M>& set_bb()       { return _bb; }
+
 // data
 protected:
     space_type  _X;
     space_type  _Y;
-public:
-    csr<T,M>      uu;
-    csr<T,M>      ub;
-    csr<T,M>      bu;
-    csr<T,M>      bb;
+    csr<T,M>    _uu;
+    csr<T,M>    _ub;
+    csr<T,M>    _bu;
+    csr<T,M>    _bb;
 
 // internals:
-protected:
     void assembly (const form_element<T,M>&   form_e, 
     		   const geo_basic<T,M>&      X_geo,
     		   const geo_basic<T,M>&      Y_geo,
     		   bool  X_geo_is_background = true);
+    void form_init (
+    		   const std::string&      name,
+    		   bool                    has_weight,
+    		   const field_basic<T,M>& weight,
+    		   quadrature_option_type  qopt);
+
 };
+template<class T, class M> form_basic<T,M> trans (const form_basic<T,M>& a);
 typedef form_basic<Float,rheo_default_memory_model> form;
 //>form:
 
@@ -145,7 +207,13 @@ typedef form_basic<Float,rheo_default_memory_model> form;
 template<class T, class M>
 inline
 form_basic<T,M>::form_basic ()
-: _X(), _Y(), uu(), ub(), bu(), bb()
+: _X(), _Y(), _uu(), _ub(), _bu(), _bb()
+{
+}
+template<class T, class M>
+inline
+form_basic<T,M>::form_basic (const form_basic<T,M>& a)
+: _X(a._X), _Y(a._Y), _uu(a._uu), _ub(a._ub), _bu(a._bu), _bb(a._bb)
 {
 }
 template<class T, class M>
@@ -176,16 +244,19 @@ form_basic<T,M>::comm() const
 {
   return get_geo().comm();
 }
+// ----------------
+// linear albebra
+// ----------------
 template<class T, class M>
 inline
 form_basic<T,M>
 form_basic<T,M>::operator+ (const form_basic<T,M>& b) const
 {
   form_basic<T,M> c;
-  c.uu = uu + b.uu;
-  c.ub = ub + b.ub;
-  c.bu = bu + b.bu;
-  c.bb = bb + b.bb;
+  c._uu = _uu + b._uu;
+  c._ub = _ub + b._ub;
+  c._bu = _bu + b._bu;
+  c._bb = _bb + b._bb;
   return c;
 }
 template<class T, class M>
@@ -194,12 +265,47 @@ form_basic<T,M>
 form_basic<T,M>::operator- (const form_basic<T,M>& b) const
 {
   form_basic<T,M> c;
-  c.uu = uu - b.uu;
-  c.ub = ub - b.ub;
-  c.bu = bu - b.bu;
-  c.bb = bb - b.bb;
+  c._uu = _uu - b._uu;
+  c._ub = _ub - b._ub;
+  c._bu = _bu - b._bu;
+  c._bb = _bb - b._bb;
   return c;
 }
+template<class T, class M>
+inline
+form_basic<T,M>&
+form_basic<T,M>::operator*= (const T& lambda)
+{
+  _uu *= lambda;
+  _ub *= lambda;
+  _bu *= lambda;
+  _bb *= lambda;
+  return *this;
+}
+template<class T, class M>
+inline
+form_basic<T,M>
+operator* (const T& lambda, const form_basic<T,M>& a)
+{
+  form_basic<T,M> b = a;
+  b *= lambda;
+  return b;
+}
+template<class T, class M>
+inline
+form_basic<T,M>
+operator- (const form_basic<T,M>& a)
+{
+  return T(-1)*a;
+}
+template <class T, class M, class Expr>
+inline
+field_basic<T,M>
+operator* (const form_basic<T,M>& a, const field_expr<Expr>& expr)
+{
+  field_basic<T,M> xh = expr;
+  return a.operator* (xh);
+}
 
 }// namespace rheolef
 # endif /* _RHEOLEF_FORM_H */
