@@ -64,6 +64,7 @@ End:
 */
 
 // for simplicity: include alls
+# include "rheolef/rheostream.h"
 # include "rheolef/iorheo.h"
 # include "rheolef/skit.h"
 # include "rheolef/pcg.h"
@@ -72,9 +73,23 @@ End:
 # include "rheolef/geo_domain_indirect.h"
 # include "rheolef/geo_domain.h"
 # include "rheolef/field.h"
+# include "rheolef/field_indirect.h"
 # include "rheolef/field_expr.h"
 # include "rheolef/field_expr_ops.h"
 # include "rheolef/form.h"
 # include "rheolef/interpolate.h"
+# include "rheolef/exact_compose.h"
+# include "rheolef/riesz.h"
+# include "rheolef/integrate.h"
+# include "rheolef/catchmark.h"
+# include "rheolef/space_mult.h"
+# include "rheolef/space_component.h"
+# include "rheolef/field_concat.h"
+# include "rheolef/form_concat.h"
+# include "rheolef/branch.h"
+# include "rheolef/characteristic.h"
+# include "rheolef/adapt.h"
+# include "rheolef/level_set.h"
+# include "rheolef/band.h"
 
 #endif // _RHEOLEF_H
