@@ -23,136 +23,17 @@
 
 #include "rheolef/geo.h"
 #include "rheolef/geo_domain.h"
-#include "rheolef/element.h"
+#include "rheolef/numbering.h"
+#include "rheolef/space_constitution.h"
 
 namespace rheolef {
 
-// =====================================================================
-// space_act = a domain + an act (block, unblock)
-// =====================================================================
+// forward declarations:
+template <class T, class M> class field_basic;
+template <class T, class M> class space_mult_list;
+template <class T, class M> class space_component;
+template <class T, class M> class space_component_const;
 
-template <class M>
-class space_act {
-public:
-
-// typedefs:
-
-    enum act_type {
-        block = 0,
-        unblock = 1
-    };
-
-// allocators:
-
-    space_act() : _dom(), _act() {}
-    space_act(const domain_indirect_basic<M>& dom, act_type act)
-	: _dom(dom), _act(act) {}
-    space_act(const space_act<M>& sp)
-	: _dom(sp._dom), _act(sp._act) {}
-
-// accessors:
-
-    const domain_indirect_basic<M>& get_domain_indirect() const { return _dom; }
-    act_type                    get_act()         const { return _act; }
-    
-// data:
-protected:
-    domain_indirect_basic<M> _dom;
-    act_type                 _act;
-};
-// =====================================================================
-// space_constitution = a table of acts
-// =====================================================================
-
-template <class T, class M>
-class space_constitution_rep : public std::vector<space_act<M> > {
-public:
-    typedef std::vector<space_act<M> >     base;
-    typedef typename  base::size_type      size_type;
-    typedef typename  base::const_iterator const_iterator;
-
-// allocators:
-
-    space_constitution_rep()
-      : base(), _omega(), _element() {}
-    space_constitution_rep(const geo_basic<T,M>& omega, std::string approx)
-      : base(), _omega(compact(omega)), _element(approx) {}
-    space_constitution_rep (const space_constitution_rep<T,M>& scr)
-      : base(scr), _omega(scr._omega), _element(scr._element)
-    {
-	error_macro ("physical copy of space_constitution_rep: size="<< base::size());
-    }
-
-// accessors:
-
-    const geo_basic<T,M>& get_geo() const { return _omega; }
-    const element<T>& get_element() const { return _element; }
-
-    size_type      size()  const { return base::size(); }
-    const_iterator begin() const { return base::begin(); }
-    const_iterator end()   const { return base::end(); }
-
-// modifiers
-
-    void set_geo (const geo_basic<T,M>& omega)  { _omega = omega; }
-    void set_element (const element<T>& elt)    { _element = elt; }
-
-    void block  (const domain_indirect_basic<M>& dom) {
-	base::push_back (space_act<M> (dom, space_act<M>::block));
-    }
-    void unblock(const domain_indirect_basic<M>& dom) {
-	base::push_back (space_act<M> (dom, space_act<M>::unblock));
-    }
-
-// comparator:
-
-    bool operator== (const space_constitution_rep<T,M>& V2) const { return _omega == V2._omega && _element == V2._element; }
-
-// data:
-protected:
-    geo_basic<T,M>                   _omega;
-    element<T>                       _element;
-};
-template <class T, class M>
-class space_constitution : public smart_pointer<space_constitution_rep<T,M> > {
-public:
-
-    typedef space_constitution_rep<T,M>   rep;
-    typedef smart_pointer<rep>            base;
-    typedef typename  rep::size_type      size_type;
-    typedef typename  rep::const_iterator const_iterator;
-
-// allocators:
-
-    space_constitution()
-     : smart_pointer<rep>(new_macro(rep)) {}
-
-    space_constitution(const geo_basic<T,M>& omega, std::string approx)
-     : smart_pointer<rep>(new_macro(rep (omega,approx))) {}
-
-// accessors:
-
-    const geo_basic<T,M>& get_geo() const { return base::data().get_geo(); }
-    const element<T>& get_element() const { return base::data().get_element(); }
-
-    size_type      size()  const { return base::data().size(); }
-    const_iterator begin() const { return base::data().begin(); }
-    const_iterator end()   const { return base::data().end(); }
-
-// comparator:
-
-    bool operator== (const space_constitution<T,M>& V2) const { return base::data().operator==(V2.data()); }
-
-// modifiers
-
-    void set_geo (const geo_basic<T,M>& omega) { base::data().set_geo (omega); }
-    void set_element (const element<T>& elt)   { base::data().set_element (elt); }
-
-    void block  (const domain_indirect_basic<M>& dom) {
-	base::data().block(dom);
-    }
-    void unblock(const domain_indirect_basic<M>& dom) { base::data().unblock(dom); }
-};
 // =====================================================================
 // a dof = a degree-of-freedom 
 //       = space_pair
@@ -193,9 +74,6 @@ namespace boost {
 
 namespace rheolef {
 
-// forward declaration:
-template <class T, class M> class field_basic;
-
 // =====================================================================
 // 1) representation: space_base_rep and space_rep
 // =====================================================================
@@ -205,41 +83,59 @@ public:
 
 // typedefs:
 
-    typedef typename space_pair_type::size_type size_type;
+    typedef typename space_pair_type::size_type  size_type;
+    typedef typename space_constant::valued_type valued_type;
 
 // allocators:
 
     space_base_rep ();
-    space_base_rep (const geo_basic<T,M>& omega, std::string approx);
+    space_base_rep (const space_constitution<T,M>& constit);
+    space_base_rep (const geo_basic<T,M>& omega, std::string approx, std::string valued);
+    space_base_rep (const space_mult_list<T,M>&);
+    virtual ~space_base_rep () {}
 
 // accessors:
 
     const distributor&  ownership() const { return _idof2blk_iub.ownership(); }
-    size_type                size() const { return ownership().size(); }
-    size_type            dis_size() const { return ownership().dis_size(); }
+    size_type                ndof() const { return ownership().size(); }
+    size_type            dis_ndof() const { return ownership().dis_size(); }
     const communicator&      comm() const { return ownership().comm(); }
 
-    const geo_basic<T,M>& get_geo() const { return _constit.get_geo(); }
-    const element<T>& get_element() const { return _constit.get_element(); }
+    const space_constitution<T,M>& get_constitution() const { return _constit; }
+    const geo_basic<T,M>& get_geo()       const { return _constit.get_geo(); }
+    const numbering<T,M>& get_numbering() const { return _constit.get_numbering(); }
+    valued_type           valued_tag()   const { return _constit.valued_tag(); }
+    const std::string&    valued()       const { return _constit.valued(); }
+    size_type             size()   const { return _constit.size(); }
+    space_component<T,M>       operator[] (size_type i_comp);
+    space_component_const<T,M> operator[] (size_type i_comp) const;
 
-    std::string get_stamp() const; /// e.g. "P1(square)", for field_expr<Expr> checks
+    std::string stamp() const; /// e.g. "P1(square)", for field_expr<Expr> checks
 
     void block  (const domain_indirect_basic<M>& dom) { no_freeze_guard(); _constit.block  (dom); }
-    void unblock(const domain_indirect_basic<M>& dom) {no_freeze_guard(); _constit.unblock(dom);}
+    void unblock(const domain_indirect_basic<M>& dom) { no_freeze_guard(); _constit.unblock(dom);}
 
     bool                 is_blocked (size_type     idof) const { freeze_guard(); return _idof2blk_iub [idof].is_blocked(); }
     size_type                   iub (size_type     idof) const { freeze_guard(); return _idof2blk_iub [idof].iub(); }
+    const point_basic<T>& xdof (size_type idof) const { return _xdof [idof]; }
+    const array<point_basic<T>,M >& get_xdofs() const { return _xdof; }
 
     const distributor& iu_ownership() const { freeze_guard(); return _iu_ownership; }
     const distributor& ib_ownership() const { freeze_guard(); return _ib_ownership; }
 
-    void get_dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const;
+    void dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const;
 
     template <class Function>
-    T momentum (Function f, size_type idof) const;
+    T momentum (Function f, size_type idof) const { return f (xdof(idof)); }
+
+    template <class Function>
+    point_basic<T> vector_momentum (Function f, size_type idof) const { return f (xdof(idof)); }
 
     array<size_type, M> build_indirect_array (
-    	const domain_indirect_basic<M>& indirect) const;
+    	const space_base_rep<T,M>& Wh, const std::string& dom_name) const;
+
+    array<size_type, M> build_indirect_array (
+        const space_base_rep<T,M>& Wh, const geo_basic<T,M>& bgd_gamma) const;
 
 // comparator:
 
@@ -252,44 +148,26 @@ public:
 protected:
     template <class T1, class M1> friend class field_basic;
 // internal:
+    void init_xdof();
     void freeze_guard() const {
-    	if (_is_freezed) return;
-    	_is_freezed = true;
+    	if (_have_freezed) return;
+    	_have_freezed = true;
     	freeze_body();
     }
     void no_freeze_guard() const {
-    	check_macro (!_is_freezed, "freezed space cannot accept new (un)blocked domains");
+    	check_macro (!_have_freezed, "freezed space cannot accept new (un)blocked domains");
     }
     void base_freeze_body() const;
     virtual void freeze_body() const { return base_freeze_body(); }
-// data:
-    space_constitution<T,M>          _constit;
-    mutable bool                     _is_freezed;
-    mutable array<space_pair_type,M> _idof2blk_iub;  // pair (is_blocked ; iu_or_ib); use ownership
-    mutable distributor              _iu_ownership;  // unknown values distribution
-    mutable distributor              _ib_ownership;  // blocked values distribution
+// data: lazy initialization (on demand only), thus most are mutable
+    space_constitution<T,M>   _constit;
+    array<point_basic<T>,M >               _xdof;          // nodal approx only
+    mutable bool                           _have_freezed;
+    mutable array<space_pair_type,M>       _idof2blk_iub;  // pair (is_blocked ; iu_or_ib); use ownership
+    mutable distributor                    _iu_ownership;  // unknown values distribution
+    mutable distributor                    _ib_ownership;  // blocked values distribution
 };
 // ---------------------------------------------------------------------
-// momentum/interpolation implementation
-// ---------------------------------------------------------------------
-template <class T, class M>
-template <class Function>
-/* TODO: typename function_traits<Function>::result_type */
-inline
-T
-space_base_rep<T,M>::momentum (
-    Function  f,
-    size_type idof) const
-{
-#ifdef TO_CLEAN
-    // P1 Lagrange only ; TODO: extend
-    const basic_point<T>& x = get_geo().vertex (idof);
-    return f(x);
-#endif // TO_CLEAN
-    basic_point<T> x = get_element().x_dof (idof, get_geo());
-    return f(x);
-}
-// ---------------------------------------------------------------------
 template <class T, class M> class space_rep {};
 
 template <class T>
@@ -298,21 +176,28 @@ public:
 
 // typedefs:
 
-    typedef space_base_rep<T,sequential> base;
-    typedef typename base::size_type size_type;
+    typedef space_base_rep<T,sequential>    base;
+    typedef typename base::size_type        size_type;
 
 // allocators:
 
-    space_rep (const geo_basic<T,sequential>& omega = (geo_basic<T,sequential>()), std::string approx = "");
+    space_rep (const space_constitution<T,sequential>& constit);
+    space_rep (const geo_basic<T,sequential>& omega, std::string approx, std::string valued);
+    space_rep (const space_mult_list<T,sequential>&);
+    ~space_rep () {}
 
 // compatibility with the distributed interface:
 
-    bool             dis_is_blocked (size_type dis_idof) const { return is_blocked(dis_idof); }
-    size_type               dis_iub (size_type dis_idof) const { return iub(dis_idof); }
+    bool             dis_is_blocked (size_type dis_idof) const { return base::is_blocked(dis_idof); }
+    size_type               dis_iub (size_type dis_idof) const { return base::iub(dis_idof); }
 
     const distributor& ios_ownership() const { return base::ownership(); }
     size_type     idof2ios_dis_idof (size_type     idof) const { return idof; }
     size_type     ios_idof2dis_idof (size_type ios_idof) const { return ios_idof; }
+
+    // for compatibility with the distributed case:
+    const std::set<size_type>& ext_iu_set() const;
+    const std::set<size_type>& ext_ib_set() const;
 };
 // ---------------------------------------------------------------------
 #ifdef _RHEOLEF_HAVE_MPI
@@ -327,7 +212,10 @@ public:
 
 // allocators:
 
-    space_rep (const geo_basic<T,distributed>& omega = (geo_basic<T,distributed>()), std::string approx = "");
+    space_rep (const space_constitution<T,distributed>& constit);
+    space_rep (const geo_basic<T,distributed>& omega, std::string approx, std::string valued);
+    space_rep (const space_mult_list<T,distributed>&);
+    ~space_rep () {}
 
 // accessors:
 
@@ -342,6 +230,8 @@ public:
 #ifdef TO_CLEAN
     void get_ios_dis_idof (const geo_element& K, std::vector<size_type>& ios_dis_idof) const;
 #endif // TO_CLEAN
+    const std::set<size_type>& ext_iu_set() const { return _ext_iu_set; }
+    const std::set<size_type>& ext_ib_set() const { return _ext_ib_set; }
 
 protected:
     template <class T1, class M1> friend class field_basic;
@@ -353,10 +243,11 @@ protected:
     void freeze_body() const;
     void append_external_dof (const geo_basic<T,distributed>& dom, std::set<size_type>& ext_dof_set) const;
 // data:
-    array<size_type>               _idof2ios_dis_idof;   // permut to/from ios dof numbering (before geo part), for i/o
-    array<size_type>               _ios_idof2dis_idof;
+    array<size_type,distributed>   _idof2ios_dis_idof;   // permut to/from ios dof numbering (before geo part), for i/o
+    array<size_type,distributed>   _ios_idof2dis_idof;
 // mutable data, affected by freeze_*()const:
-    mutable map_pair_type          _ext_dis_idof2blk_dis_iub; // same for partition bdry dofs
+    mutable std::set<size_type>    _ext_iu_set;    // external dofs used by field::dis_dof
+    mutable std::set<size_type>    _ext_ib_set;
 };
 #endif // _RHEOLEF_HAVE_MPI
 // ====================================================================
@@ -370,6 +261,17 @@ DESCRIPTION:
 The @code{space} class contains some numbering 
 for unknowns and blocked degrees of freedoms
 related to a given mesh and polynomial approximation.
+SYNOPSIS:
+@example
+    space Q (omega, "P1");
+    space V (omega, "P2", "vector");
+    space T (omega, "P1d", "tensor");
+@end example
+PRODUCT:
+@example
+    space X = T*V*Q;
+    space Q2 = pow(Q,2);
+@end example
 SEE ALSO: "geo"(3)
 AUTHORS: Pierre.Saramito@imag.fr
 DATE:   14 december 2010
@@ -389,13 +291,17 @@ public:
 
 // typedefs:
 
-    typedef space_rep<T,sequential> rep;
-    typedef smart_pointer<rep>      base;
-    typedef typename rep::size_type size_type;
+    typedef space_rep<T,sequential>   rep;
+    typedef smart_pointer<rep>        base;
+    typedef typename rep::size_type   size_type;
+    typedef typename rep::valued_type valued_type;
 
 // allocators:
 
-    space_basic (const geo_basic<T,sequential>& omega = (geo_basic<T,sequential>()), std::string approx = "");
+    space_basic (const geo_basic<T,sequential>& omega = (geo_basic<T,sequential>()),
+		  std::string approx = "", std::string valued = "scalar");
+    space_basic (const space_mult_list<T,sequential>& expr);
+    space_basic (const space_constitution<T,sequential>& constit);
 
 // accessors:
 
@@ -406,14 +312,22 @@ public:
 
     const distributor&  ownership() const;
     const communicator& comm() const;
-    size_type           size() const;
-    size_type           dis_size() const;
+    size_type           ndof() const;
+    size_type           dis_ndof() const;
 
     const geo_basic<T,sequential>& get_geo() const;
-    const element<T>&   get_element() const;
-    std::string get_stamp() const;
+    const numbering<T,sequential>& get_numbering() const;
+    size_type size() const;
+    valued_type           valued_tag()   const;
+    const std::string&    valued()       const;
+    space_component<T,sequential>       operator[] (size_type i_comp);
+    space_component_const<T,sequential> operator[] (size_type i_comp) const;
+    const space_constitution<T,sequential>& get_constitution() const;
+    size_type degree() const;
+    std::string get_approx() const;
+    std::string stamp() const;
 
-    void get_dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const;
+    void dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const;
 
     const distributor& iu_ownership() const;
     const distributor& ib_ownership() const;
@@ -427,11 +341,23 @@ public:
     size_type idof2ios_dis_idof (size_type idof) const;
     size_type ios_idof2dis_idof (size_type ios_idof) const;
 
+    const point_basic<T>& xdof (size_type idof) const;
+    const array<point_basic<T>,sequential>& get_xdofs() const;
+
     template <class Function>
     T momentum (Function f, size_type idof) const;
 
-    array<size_type, sequential>
-    build_indirect_array (const geo_basic<T,sequential>& dom) const;
+    template <class Function>
+    point_basic<T> vector_momentum (Function f, size_type idof) const;
+
+    array<size_type, sequential> build_indirect_array (
+    	const space_basic<T,sequential>& Wh, const std::string& dom_name) const;
+
+    array<size_type, sequential> build_indirect_array (
+        const space_basic<T,sequential>& Wh, const geo_basic<T,sequential>& bgd_gamma) const;
+
+    const std::set<size_type>& ext_iu_set() const { return base::data().ext_iu_set(); }
+    const std::set<size_type>& ext_ib_set() const { return base::data().ext_ib_set(); }
 
 // comparator:
 
@@ -443,8 +369,24 @@ public:
 //>verbatim:
 template<class T>
 inline
-space_basic<T,sequential>::space_basic (const geo_basic<T,sequential>& omega, std::string approx)
- : base (new_macro(rep(omega, approx)))
+space_basic<T,sequential>::space_basic (
+    const space_constitution<T,sequential>& constit)
+ : base (new_macro(rep(constit)))
+{
+}
+template<class T>
+inline
+space_basic<T,sequential>::space_basic (
+    const geo_basic<T,sequential>& omega,
+    std::string approx,
+    std::string valued)
+ : base (new_macro(rep(omega, approx, valued)))
+{
+}
+template<class T>
+inline
+space_basic<T,sequential>::space_basic (const space_mult_list<T,sequential>& expr)
+ : base (new_macro(rep(expr)))
 {
 }
 template<class T>
@@ -471,16 +413,16 @@ space_basic<T,sequential>::comm() const
 template<class T>
 inline
 typename space_basic<T,sequential>::size_type
-space_basic<T,sequential>::size() const
+space_basic<T,sequential>::ndof() const
 {
-    return base::data().size();
+    return base::data().ndof();
 }
 template<class T>
 inline
 typename space_basic<T,sequential>::size_type
-space_basic<T,sequential>::dis_size() const
+space_basic<T,sequential>::dis_ndof() const
 {
-    return base::data().dis_size();
+    return base::data().dis_ndof();
 }
 template<class T>
 inline
@@ -491,24 +433,80 @@ space_basic<T,sequential>::get_geo() const
 }
 template<class T>
 inline
-const element<T>&
-space_basic<T,sequential>::get_element() const
+const numbering<T,sequential>&
+space_basic<T,sequential>::get_numbering() const
 {
-    return base::data().get_element();
+    return base::data().get_numbering();
+}
+template<class T>
+inline
+const space_constitution<T,sequential>&
+space_basic<T,sequential>::get_constitution() const
+{
+    return base::data().get_constitution();
+}
+template<class T>
+inline
+typename space_basic<T,sequential>::size_type
+space_basic<T,sequential>::size() const
+{
+    return base::data().size();
+}
+template<class T>
+inline
+const std::string&
+space_basic<T,sequential>::valued() const
+{
+    return base::data().valued();
+}
+template<class T>
+inline
+typename space_basic<T,sequential>::valued_type
+space_basic<T,sequential>::valued_tag() const
+{
+    return base::data().valued_tag();
+}
+template<class T>
+inline
+space_component<T,sequential>
+space_basic<T,sequential>::operator[] (size_type i_comp)
+{
+    return base::data().operator[] (i_comp);
+}
+template<class T>
+inline
+space_component_const<T,sequential>
+space_basic<T,sequential>::operator[] (size_type i_comp) const
+{
+    return base::data().operator[] (i_comp);
+}
+template<class T>
+inline
+typename space_basic<T,sequential>::size_type
+space_basic<T,sequential>::degree() const
+{
+    return get_numbering().degree();
 }
 template<class T>
 inline
 std::string
-space_basic<T,sequential>::get_stamp() const
+space_basic<T,sequential>::get_approx() const
 {
-    return base::data().get_stamp();
+    return get_numbering().name();
+}
+template<class T>
+inline
+std::string
+space_basic<T,sequential>::stamp() const
+{
+    return base::data().stamp();
 }
 template<class T>
 inline
 void
-space_basic<T,sequential>::get_dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const
+space_basic<T,sequential>::dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const
 {
-    return base::data().get_dis_idof (K, dis_idof);
+    return base::data().dis_idof (K, dis_idof);
 }
 template<class T>
 inline
@@ -595,6 +593,20 @@ space_basic<T,sequential>::unblock (const domain_indirect_basic<sequential>& dom
     return base::data().unblock (dom);
 }
 template<class T>
+inline
+const point_basic<T>&
+space_basic<T,sequential>::xdof (size_type idof) const
+{
+    return base::data().xdof (idof);
+}
+template<class T>
+inline
+const array<point_basic<T>,sequential>&
+space_basic<T,sequential>::get_xdofs() const
+{
+    return base::data().get_xdofs();
+}
+template<class T>
 template <class Function>
 inline
 T
@@ -602,6 +614,14 @@ space_basic<T,sequential>::momentum (Function f, size_type idof) const
 {
     return base::data().momentum (f, idof);
 }
+template<class T>
+template <class Function>
+inline
+point_basic<T>
+space_basic<T,sequential>::vector_momentum (Function f, size_type idof) const
+{
+    return base::data().vector_momentum (f, idof);
+}
 // ---------------------------------------------------------------------
 #ifdef _RHEOLEF_HAVE_MPI
 //<verbatim:
@@ -611,13 +631,17 @@ public:
 
 // typedefs:
 
-    typedef space_rep<T,distributed> rep;
-    typedef smart_pointer<rep>       base;
-    typedef typename rep::size_type  size_type;
+    typedef space_rep<T,distributed>  rep;
+    typedef smart_pointer<rep>        base;
+    typedef typename rep::size_type   size_type;
+    typedef typename rep::valued_type valued_type;
 
 // allocators:
 
-    space_basic (const geo_basic<T,distributed>& omega = (geo_basic<T,distributed>()), std::string approx = "");
+    space_basic (const geo_basic<T,distributed>& omega = (geo_basic<T,distributed>()),
+		  std::string approx = "", std::string valued = "scalar");
+    space_basic (const space_mult_list<T,distributed>&);
+    space_basic (const space_constitution<T,distributed>& constit);
 
 // accessors:
 
@@ -628,14 +652,22 @@ public:
 
     const distributor&  ownership() const;
     const communicator& comm() const;
-    size_type           size() const;
-    size_type           dis_size() const;
+    size_type           ndof() const;
+    size_type           dis_ndof() const;
 
     const geo_basic<T,distributed>& get_geo() const;
-    const element<T>&   get_element() const;
-    std::string get_stamp() const;
+    const numbering<T,distributed>& get_numbering() const;
+    size_type size() const;
+    valued_type           valued_tag()   const;
+    const std::string&    valued()       const;
+    space_component<T,distributed>       operator[] (size_type i_comp);
+    space_component_const<T,distributed> operator[] (size_type i_comp) const;
+    const space_constitution<T,distributed>& get_constitution() const;
+    size_type degree() const;
+    std::string get_approx() const;
+    std::string stamp() const;
 
-    void get_dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const;
+    void dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const;
 
     const distributor& iu_ownership() const;
     const distributor& ib_ownership() const;
@@ -650,11 +682,23 @@ public:
     size_type idof2ios_dis_idof (size_type idof) const;
     size_type ios_idof2dis_idof (size_type ios_idof) const;
 
+    const point_basic<T>& xdof (size_type idof) const;
+    const array<point_basic<T>,distributed>& get_xdofs() const;
+
     template <class Function>
     T momentum (Function f, size_type idof) const;
 
-    array<size_type, distributed>
-    build_indirect_array (const geo_basic<T,distributed>& dom) const;
+    template <class Function>
+    point_basic<T> vector_momentum (Function f, size_type idof) const;
+
+    array<size_type, distributed> build_indirect_array (
+    	const space_basic<T,distributed>& Wh, const std::string& dom_name) const;
+
+    array<size_type, distributed> build_indirect_array (
+        const space_basic<T,distributed>& Wh, const geo_basic<T,distributed>& bgd_gamma) const;
+
+    const std::set<size_type>& ext_iu_set() const { return base::data().ext_iu_set(); }
+    const std::set<size_type>& ext_ib_set() const { return base::data().ext_ib_set(); }
 
 // comparator:
 
@@ -667,8 +711,24 @@ public:
 
 template<class T>
 inline
-space_basic<T,distributed>::space_basic (const geo_basic<T,distributed>& omega, std::string approx)
- : base (new_macro(rep(omega, approx)))
+space_basic<T,distributed>::space_basic (
+    const space_constitution<T,distributed>& constit)
+ : base (new_macro(rep(constit)))
+{
+}
+template<class T>
+inline
+space_basic<T,distributed>::space_basic (
+    const geo_basic<T,distributed>& omega,
+    std::string approx,
+    std::string valued)
+ : base (new_macro(rep(omega, approx, valued)))
+{
+}
+template<class T>
+inline
+space_basic<T,distributed>::space_basic (const space_mult_list<T,distributed>& expr)
+ : base (new_macro(rep(expr)))
 {
 }
 template<class T>
@@ -695,16 +755,16 @@ space_basic<T,distributed>::comm() const
 template<class T>
 inline
 typename space_basic<T,distributed>::size_type
-space_basic<T,distributed>::size() const
+space_basic<T,distributed>::ndof() const
 {
-    return base::data().size();
+    return base::data().ndof();
 }
 template<class T>
 inline
 typename space_basic<T,distributed>::size_type
-space_basic<T,distributed>::dis_size() const
+space_basic<T,distributed>::dis_ndof() const
 {
-    return base::data().dis_size();
+    return base::data().dis_ndof();
 }
 template<class T>
 inline
@@ -715,24 +775,80 @@ space_basic<T,distributed>::get_geo() const
 }
 template<class T>
 inline
-const element<T>&
-space_basic<T,distributed>::get_element() const
+const numbering<T,distributed>&
+space_basic<T,distributed>::get_numbering() const
 {
-    return base::data().get_element();
+    return base::data().get_numbering();
+}
+template<class T>
+inline
+const space_constitution<T,distributed>&
+space_basic<T,distributed>::get_constitution() const
+{
+    return base::data().get_constitution();
+}
+template<class T>
+inline
+typename space_basic<T,distributed>::size_type
+space_basic<T,distributed>::size() const
+{
+    return base::data().size();
+}
+template<class T>
+inline
+const std::string&
+space_basic<T,distributed>::valued() const
+{
+    return base::data().valued();
+}
+template<class T>
+inline
+typename space_basic<T,distributed>::valued_type
+space_basic<T,distributed>::valued_tag() const
+{
+    return base::data().valued_tag();
+}
+template<class T>
+inline
+space_component<T,distributed>
+space_basic<T,distributed>::operator[] (size_type i_comp)
+{
+    return base::data().operator[] (i_comp);
+}
+template<class T>
+inline
+space_component_const<T,distributed>
+space_basic<T,distributed>::operator[] (size_type i_comp) const
+{
+    return base::data().operator[] (i_comp);
+}
+template<class T>
+inline
+typename space_basic<T,distributed>::size_type
+space_basic<T,distributed>::degree() const
+{
+    return get_numbering().degree();
 }
 template<class T>
 inline
 std::string
-space_basic<T,distributed>::get_stamp() const
+space_basic<T,distributed>::get_approx() const
 {
-    return base::data().get_stamp();
+    return get_numbering().name();
+}
+template<class T>
+inline
+std::string
+space_basic<T,distributed>::stamp() const
+{
+    return base::data().stamp();
 }
 template<class T>
 inline
 void
-space_basic<T,distributed>::get_dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const
+space_basic<T,distributed>::dis_idof (const geo_element& K, std::vector<size_type>& dis_idof) const
 {
-    return base::data().get_dis_idof (K, dis_idof);
+    return base::data().dis_idof (K, dis_idof);
 }
 template<class T>
 inline
@@ -819,6 +935,20 @@ space_basic<T,distributed>::unblock (const domain_indirect_basic<distributed>& d
     base::data().unblock (dom);
 }
 template<class T>
+inline
+const point_basic<T>&
+space_basic<T,distributed>::xdof (size_type idof) const
+{
+    return base::data().xdof (idof);
+}
+template<class T>
+inline
+const array<point_basic<T>,distributed>&
+space_basic<T,distributed>::get_xdofs() const
+{
+    return base::data().get_xdofs();
+}
+template<class T>
 template <class Function>
 inline
 T
@@ -826,6 +956,14 @@ space_basic<T,distributed>::momentum (Function f, size_type idof) const
 {
     return base::data().momentum (f, idof);
 }
+template<class T>
+template <class Function>
+inline
+point_basic<T>
+space_basic<T,distributed>::vector_momentum (Function f, size_type idof) const
+{
+    return base::data().vector_momentum (f, idof);
+}
 #endif // _RHEOLEF_HAVE_MPI
 
 // only valid when M=sequential or M=distributed => use a macro
@@ -833,15 +971,20 @@ space_basic<T,distributed>::momentum (Function f, size_type idof) const
 template<class T>									\
 inline											\
 array<typename space_basic<T,M>::size_type, M>						\
-space_basic<T,M>::build_indirect_array (const geo_basic<T,M>& dom) const		\
+space_basic<T,M>::build_indirect_array (						\
+	const space_basic<T,M>& Wh,							\
+	const std::string& dom_name) const						\
 {											\
-    if (dom.data().variant() == geo_abstract_base_rep<T>::geo_domain_indirect) {	\
-      const geo_domain_indirect_rep<T,M>& dom_rep					\
-       = dynamic_cast<const geo_domain_indirect_rep<T,M>&>(dom.data());			\
-      return base::data().build_indirect_array (dom_rep.get_indirect());		\
-    }											\
-    error_macro ("build_indirect_array: unsupported non domain_indirect geo variant");	\
-    return array<size_type, M>(); /* not  reached */					\
+    return base::data().build_indirect_array (Wh.data(), dom_name);			\
+}											\
+template<class T>									\
+inline											\
+array<typename space_basic<T,M>::size_type, M>						\
+space_basic<T,M>::build_indirect_array (						\
+	const space_basic<T,M>& Wh,							\
+	const geo_basic<T,M>&   bgd_gamma) const					\
+{											\
+    return base::data().build_indirect_array (Wh.data(), bgd_gamma);			\
 }
 _RHEOLEF_space_build_indirect_array(sequential)
 
