@@ -20,19 +20,19 @@
 /// =========================================================================
 // check field_indirect_const
 #include "rheolef.h"
-#include "rounder.h"
+#include "rheolef/rounder.h"
 using namespace rheolef;
 using namespace std;
 Float g (const point& x) { return sin(x[0]+x[1]+0.5); }
 int main(int argc, char**argv) {
-  environment distributed(argc, argv);
+  environment rheolef(argc, argv);
   geo omega (argv[1]);
-  string dom_name = (argc > 2) ? argv[2] : "right";
-  Float prec = (argc > 3) ? atof(argv[3]) : 1e-10;
-  space Vh (omega, "P1");
-  space Wh (omega[dom_name], "P1");
+  space Vh (omega, argv[2]);
+  string dom_name = (argc > 3) ? argv[3] : "right";
+  Float prec = (argc > 4) ? atof(argv[4]) : 1e-10;
+  space Wh (omega[dom_name], argv[2]);
   field fh = interpolate(Vh, g);
   field gh (Wh);
   gh = fh[dom_name];
-  dcout << setprecision(15) << compose (rounder(prec),gh);
+  dout << field(compose (rounder(prec),gh));
 }
