@@ -20,18 +20,19 @@
 /// =========================================================================
 // check field_indirect
 #include "rheolef.h"
-#include "rounder.h"
+#include "rheolef/rounder.h"
 using namespace rheolef;
 using namespace std;
 size_t N;
 Float f (const point& x) { return 1+(0.5/N)*(x[0]*(1-x[0])+x[1]*(1-x[1])+x[2]*(1-x[2])); }
 int main(int argc, char**argv) {
-  environment distributed(argc, argv);
+  environment rheolef(argc, argv);
   geo omega (argv[1]);
-  Float prec = (argc > 2) ? atof(argv[2]) : 1e-10;
-  space Vh (omega, "P1");
+  space Vh (omega, argv[2]);
+  string dom_name = (argc > 3) ? argv[3] : "left";
+  Float prec = (argc > 4) ? atof(argv[4]) : 1e-10;
   N = omega.dimension();
   field fh = interpolate(Vh, f);
-  fh["left"] = 0;
-  dcout << setprecision(15) << compose(rounder(prec),fh);
+  fh[dom_name] = 0;
+  dout << setprecision(15) << field(compose(rounder(prec),fh));
 }
