@@ -19,13 +19,18 @@
 ///
 /// =========================================================================
 #include "rheolef.h"
-#include "rounder.h"
+#include "rheolef/rounder.h"
 using namespace rheolef;
 using namespace std;
 int main(int argc, char**argv) {
-  environment distributed(argc, argv);
+  environment rheolef(argc, argv);
   Float prec = (argc > 1) ? atof(argv[1]) : 1e-10;
   field uh;
-  dcin  >> uh;
-  dcout << setprecision(15) << compose (rounder(prec),uh);
+  din  >> uh;
+#undef BUG
+#ifdef BUG
+  dout << setprecision(15) << compose (rounder(prec),uh);
+#else // BUG
+  dout << setprecision(15) << field(compose (rounder(prec),uh));
+#endif // BUG
 }
