@@ -19,29 +19,34 @@
 ///
 /// =========================================================================
 #include "rheolef.h"
-#include "rounder.h"
+#include "rheolef/rounder.h"
 using namespace rheolef;
 using namespace std;
 size_t N;
 Float f (const point& x) { return 1+(0.5/N)*(x[0]*(1-x[0])+x[1]*(1-x[1])+x[2]*(1-x[2])); }
 int main(int argc, char**argv) {
-  environment distributed(argc, argv);
+  environment rheolef(argc, argv);
   geo omega (argv[1]);
+  space Xh (omega, argv[2]);
   Float prec = (argc > 3) ? atof(argv[3]) : 1e-10;
   N = omega.dimension();
-  space Xh (omega, "P1");
-  if (omega.dimension() >= 3) {
+  if (argc > 4) {
+    warning_macro ("block " << argv[4]);
+    Xh.block(argv[4]);
+  } else {
+    if (omega.dimension() >= 3) {
       Xh.block("back");
       Xh.block("front");
-  }
-  if (omega.dimension() >= 2) {
+    }
+    if (omega.dimension() >= 2) {
       Xh.block("top");
       Xh.block("bottom");
+    }
+    Xh.block("left");
+    Xh.block("right");
   }
-  Xh.block("left");
-  Xh.block("right");
   field fh = interpolate(Xh, f);
   form m (Xh, Xh, "mass");
   field gh = m*fh;
-  dcout << setprecision(15) << compose(rounder(prec),gh);
+  dout << setprecision(15) << field(compose(rounder(prec),gh));
 }
