@@ -22,32 +22,86 @@
 SRCDIR=${SRCDIR-"."}
 TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 DATA_DIR="${TOP_SRCDIR}/nfem/data"
-NPROC_MAX=${NPROC_MAX-"8"}
+NPROC_MAX=${NPROC_MAX-"6"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
-ZDIFF="$TOP_SRCDIR/config/zdiff"
-
 status=0
+
 L="
-carre-v1.geo
-carre-v2.geo
-carre-3-dom-v2.geo
-bielle-v2.geo
+zero-5.geo
+line-1-bdry.geo
 line-dom-v2.geo
+edge_s_3d_p1-v2.geo
+line-2-bdry.geo
+line-3-bdry.geo
+line-20-bdry.geo
+carre-v2.geo
+cube-P-5-dom-v2.geo
+cube-H-6-dom-v2.geo
+
+circle_s_p1-v2.geo
+sphere_s_p1-v2.geo
+sphere_s_q_p1-v2.geo
+carre-3-dom-v2.geo
 carre-dom-v2.geo
 cube-dom-v2.geo
 carre-100-dom-v2.geo
+line-dom-v2.right.geo
+carre-100-dom-v2.right.geo
+carre-bamg-v2.right.geo
+cube-10-dom-v2.right.geo
+cube-gmsh-v2.right.geo
+circle_p1-v2.geo
+circle_p2-v2.geo
+circle_p3-v2.geo
+circle_p5-v2.geo
+circle_q_p1-v2.geo
+sphere_p1-v2.geo
+sphere_p2-v2.geo
+sphere_p3-v2.geo
+sphere_H_p1-v2.geo
+
+carre-tq-10-v2.geo
 carre-tq-10-dom-v2.geo
+my_cube_PH-5-v2.geo
+my_cube_TP-5-v2.geo
+my_cube_TPH-5-v2.geo
 extrude-TP-gmsh-dom-v2.geo
-cube-10-dom-v2.geo
+
+circle_s_p2-v2.geo
+circle_p2-v2.geo
+circle_q_p2-v2.geo
+sphere_s_p2-v2.geo
+sphere_s_q_p2-v2.geo
+sphere_p2-v2.geo
+sphere_H_p2-v2.geo
+
+triangle_p5-v2.geo
+triangle2_p5-v2.geo
+tetra_p5-v2.geo
+tetra2_p5-v2.geo
+
+circle_s_p10-v2.geo
+circle_p5-v2.geo
+circle_q_p10-v2.geo
+circle_tq_p10-v2.geo
+sphere_s_p10-v2.geo
+sphere_s_q_p10-v2.geo
+sphere_p9-v2.geo
 "
+
 for geo in $L; do
   loop_mpirun "./geo_io_tst ${SRCDIR}/$geo 2>/dev/null | diff -B -w ${SRCDIR}/$geo - >/dev/null"
   if test $? -ne 0; then status=1; fi
 done
 
+#export TMPDIR="."
+ZDIFF="$TOP_SRCDIR/config/zdiff"
 
 L="
+"
+
+L_TODO="
 tetra2.geo.gz
 dodecaedre00.geo.gz
 dodecaedre01.geo.gz
