@@ -19,17 +19,20 @@
 ///
 /// =========================================================================
 #include "rheolef.h"
-#include "rounder.h"
+#include "rheolef/rounder.h"
 using namespace rheolef;
 using namespace std;
 Float g (const point& x) { return sin(x[0]+x[1]+0.5); }
 int main(int argc, char**argv) {
-  environment distributed(argc, argv);
+  environment rheolef(argc, argv);
   geo omega (argv[1]);
   string approx = argv[2];
   string dom_name = (argc > 3) ? argv[3] : "right";
+dis_warning_macro ("space...");
   space Wh (omega[dom_name], approx);
   Float prec = (argc > 4) ? atof(argv[4]) : 1e-10;
+dis_warning_macro ("interpolate...");
   field gh = interpolate(Wh, g);
-  dcout << setprecision(15) << compose (rounder(prec),gh);
+dis_warning_macro ("interpolate done");
+  dout << setprecision(15) << field(compose (rounder(prec),gh));
 }
