@@ -19,14 +19,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 # -------------------------------------------------------------------------
+TOP_SRCDIR=${TOP_SRCDIR-"../.."}
+. "${TOP_SRCDIR}/config/loop_mpirun.sh"
+
 status=0
-echo "    ./gauss_legendre_tst ..."
 for R in 1 2 3 4 5 6 7 8; do
-    #echo "    ./gauss_legendre_tst $R"
-    ./gauss_legendre_tst $R >/dev/null 2>/dev/null
+    run "./gauss_legendre_tst $R >/dev/null 2>/dev/null"
     if test $? -ne 0; then status=1; fi
 done
-if test $status -ne 0; then echo "      *NO*"; fi
 exit $status
-
-
