@@ -85,44 +85,57 @@
 # --------------------------------------------------------------------------
 # includes
 # --------------------------------------------------------------------------
-#from enthought.mayavi.scripts import mayavi2
-#mayavi2.standalone(globals())
-
 from os.path import join, abspath
-import enthought.mayavi
-
 import numpy	# sqrt, ect
-
-from enthought.mayavi.sources.vtk_file_reader import VTKFileReader
-from enthought.mayavi.modules.outline import Outline
-from enthought.mayavi.modules.axes import Axes
-from enthought.mayavi.modules.text import Text
-from enthought.mayavi.modules.surface import Surface
-from enthought.mayavi.modules.iso_surface import IsoSurface
-from enthought.mayavi.modules.scalar_cut_plane import ScalarCutPlane
-from enthought.mayavi.modules.volume import Volume
-from enthought.mayavi.modules.vectors import Vectors
-from enthought.mayavi.modules.tensor_glyph import TensorGlyph
-
-from enthought.mayavi.filters.extract_edges import ExtractEdges
-from enthought.mayavi.filters.user_defined import UserDefined
-from enthought.mayavi.filters.image_data_probe import ImageDataProbe
-from enthought.mayavi.filters.warp_scalar import WarpScalar
-from enthought.mayavi.filters.warp_vector import WarpVector
-from enthought.mayavi.filters.cut_plane import CutPlane
-
-from enthought.mayavi.components.implicit_plane import ImplicitPlane
-
-from enthought.tvtk.api import tvtk
-
-from enthought.traits.api import Instance, Property, Str, TraitError
+from sys import version_info # mayavi2-4.0 has changed his interface from python-2.7.2...
+if version_info[0] >= 2 and version_info[1] >= 7 and version_info[2] >= 2:
+    from mayavi.sources.vtk_file_reader import VTKFileReader
+    from mayavi.modules.outline import Outline
+    from mayavi.modules.axes import Axes
+    from mayavi.modules.text import Text
+    from mayavi.modules.surface import Surface
+    from mayavi.modules.iso_surface import IsoSurface
+    from mayavi.modules.scalar_cut_plane import ScalarCutPlane
+    from mayavi.modules.volume import Volume
+    from mayavi.modules.vectors import Vectors
+    from mayavi.modules.tensor_glyph import TensorGlyph
+    from mayavi.filters.extract_edges import ExtractEdges
+    from mayavi.filters.user_defined import UserDefined
+    from mayavi.filters.image_data_probe import ImageDataProbe
+    from mayavi.filters.warp_scalar import WarpScalar
+    from mayavi.filters.warp_vector import WarpVector
+    from mayavi.filters.cut_plane import CutPlane
+    from mayavi.components.implicit_plane import ImplicitPlane
+    from tvtk.api import tvtk
+    from traits.api import Instance, Property, Str, TraitError
+else:
+    # the same with the old "enthought" prefix (I dont known how to merge these cases)
+    from enthought.mayavi.sources.vtk_file_reader import VTKFileReader
+    from enthought.mayavi.modules.outline import Outline
+    from enthought.mayavi.modules.axes import Axes
+    from enthought.mayavi.modules.text import Text
+    from enthought.mayavi.modules.surface import Surface
+    from enthought.mayavi.modules.iso_surface import IsoSurface
+    from enthought.mayavi.modules.scalar_cut_plane import ScalarCutPlane
+    from enthought.mayavi.modules.volume import Volume
+    from enthought.mayavi.modules.vectors import Vectors
+    from enthought.mayavi.modules.tensor_glyph import TensorGlyph
+    from enthought.mayavi.filters.extract_edges import ExtractEdges
+    from enthought.mayavi.filters.user_defined import UserDefined
+    from enthought.mayavi.filters.image_data_probe import ImageDataProbe
+    from enthought.mayavi.filters.warp_scalar import WarpScalar
+    from enthought.mayavi.filters.warp_vector import WarpVector
+    from enthought.mayavi.filters.cut_plane import CutPlane
+    from enthought.mayavi.components.implicit_plane import ImplicitPlane
+    from enthought.tvtk.api import tvtk
+    from enthought.traits.api import Instance, Property, Str, TraitError
 
 # --------------------------------------------------------------------------
 # utilities
 # --------------------------------------------------------------------------
 def start_new_scene (view, option):
 
-    view.new_scene()
+    view.engine.new_scene()
     scene = view.engine.current_scene.scene
     scene.disable_render = True
 
@@ -139,7 +152,7 @@ def show_scene (scene):
 
 def mayavi2_axis (view, option):
 
-    if option['view_2d']:
+    if option['view_2d'] or option['view_1d']:
         option['stereo'] = 0
         scene = view.engine.current_scene.scene
         #print "scene.interactor.interactor_style = ", scene.interactor.interactor_style
@@ -148,7 +161,13 @@ def mayavi2_axis (view, option):
     # 2D or 3D and not stereo
     # axes are not rendered as stereo and disturbs the outline
     axes = Axes()
-    #print "axes.axes = ", axes.axes
+    if option['elevation']:
+      # scale in Y or Z direction: use ranges
+      axes.axes.use_ranges = True
+      axes.axes.ranges = option['ranges']
+      scene = view.engine.current_scene.scene
+      scene.parallel_projection = True
+
     if option['stereo']:
         axes.axes.visibility = False
     view.add_module(axes)
@@ -157,6 +176,12 @@ def mayavi2_axis (view, option):
         #   axes.axes.z_axis_visibility = False
         # thus, only clean z label
         axes.axes.z_label = ''
+        axes.axes.z_axis_visibility = 0
+    if option['view_1d']:
+        axes.axes.y_label = ''
+        #axes.axes.z_label = ''
+        axes.axes.y_axis_visibility = 0
+        #axes.axes.z_axis_visibility = 0
 
     if not option['view_2d']:
 	# 3D and stereo : put small axes at bottom
@@ -200,7 +225,7 @@ def check_stereo_axis (view, option):
               except BadWindow:
                 pass
 
-    if option['view_2d']:
+    if option['view_2d'] or option['view_1d']:
 
         # select initial view:
         scene.z_plus_view()
@@ -232,8 +257,6 @@ def mayavi2_geo_domain (view, option, filename, dom_name, idx):
     if idx == 0:
         output = data.reader.get_output()
         x_min, x_max, y_min, y_max, z_min, z_max = output.bounds
-        option['view_2d'] = (z_max - z_min == 0)
-	check_stereo_axis (view,option)
 
 	# title:
         #print "output = ", output
@@ -287,10 +310,13 @@ def mayavi2_geo_domain (view, option, filename, dom_name, idx):
         view.add_module(polydata_filter)
         clip_filter = UserDefined(filter=tvtk.ClipPolyData(), name='ClipPolyData')
         clip_filter.filter.clip_function = cut_plane
+        clip_filter.filter.inside_out = 1
         view.add_module(clip_filter)
 
     surface = Surface()
     view.add_module(surface)
+    #ICI
+    surface.actor.mapper.scalar_visibility = False;
     surface.actor.property.color = color
     if option['fill']:
         surface.actor.property.representation = 'surface'
@@ -347,8 +373,63 @@ def mayavi2_geo (view, option, d):
     for i in range(1,len(d)):
         mayavi2_geo_domain (view, option, d[0]+"."+d[i]+".vtk", d[i], i)
 
+    check_stereo_axis (view,option)
     show_scene (scene)
 
+# --------------------------------------------------------------------------
+# non-interactive cut: from file to file
+# --------------------------------------------------------------------------
+def plane_cutter_on_file (input_file_name, output_file_name, origin, normal, tolerance):
+    # --------------------
+    # 1. reader
+    # --------------------
+    data = VTKFileReader()
+    data.initialize(input_file_name)
+    # --------------------
+    # 2. plane
+    # --------------------
+    plane2 = tvtk.Plane()
+    if origin[0] < 1e+10:
+      plane2.origin = origin
+    else:
+      plane2.origin = data.reader.get_output().center
+    plane2.normal = normal
+    #print "plane2=",plane2
+    # --------------------
+    # 3. cut filter
+    # --------------------
+    cutter = UserDefined(filter=tvtk.Cutter(), name='Cutter')
+    cutter.filter.cut_function = plane2
+    cutter.filter.input  = data.reader.get_output()
+    # --------------------
+    # 4. merge points
+    # --------------------
+    cleaner = UserDefined(filter=tvtk.CleanPolyData(), name='Cleaner')
+    cleaner.filter.input     = cutter.filter.output
+    cleaner.filter.tolerance = tolerance
+    # --------------------
+    # 5. force triangle
+    # --------------------
+    caster = UserDefined(filter=tvtk.CastToConcrete(), name='Caster')
+    caster.filter.input     = cleaner.filter.output
+    triangler = UserDefined(filter=tvtk.TriangleFilter(), name='Triangler')
+    triangler.filter.input  = caster.filter.output
+    # --------------------
+    # F. writer
+    # --------------------
+    writer = tvtk.DataSetWriter()
+    writer.input = triangler.filter.output
+    writer.file_name = output_file_name
+    writer.write()
+
+# call example:
+#input_file_name = "contraction-zr-P2.vtk"
+#output_file_name = "cut.vtk"
+#origin = (0.0, 1e-5, 0.0)
+#normal = (0,1,0)
+#tolerance = 1e-14
+#plane_cutter_on_file(input_file_name, output_file_name, origin, normal, tolerance)
+
 # --------------------------------------------------------------------------
 # scalar field visualization
 # --------------------------------------------------------------------------
@@ -369,7 +450,7 @@ def build_isovalue_table (fmin, fmax, option):
     values = values + [fmax]
     return values
 
-def mayavi2_field_scalar_P1 (view, basename, option):
+def mayavi2_field_scalar (view, basename, option):
 
     scene = start_new_scene (view, option)
 
@@ -378,11 +459,15 @@ def mayavi2_field_scalar_P1 (view, basename, option):
     view.add_source(data)
     output = data.reader.get_output()
     #print "output = ", output
+    # P1 = point_data data:
     #print "output.point_data = ", output.point_data
-    #print "output.cell_data = ", output.cell_data
     #print "output.point_data.number_of_arrays = ", output.point_data.number_of_arrays
-    #print "output.cell_data.number_of_arrays = ", output.cell_data.number_of_arrays
+    #print "output.point_data.scalars = ", output.point_data.scalars
+    #print "output.point_data.scalars.name = ", output.point_data.scalars.name
+    # P0 = cell data:
+    #print "output.cell_data = ", output.cell_data
     #print "output.cell_data.scalars = ", output.cell_data.scalars
+    #print "output.cell_data.number_of_arrays = ", output.cell_data.number_of_arrays
     #print "output.cell_data.scalars.name = ", output.cell_data.scalars.name
     # determine whether continuous or discontinuous data:
     if output.point_data.number_of_arrays > 0:
@@ -391,14 +476,52 @@ def mayavi2_field_scalar_P1 (view, basename, option):
     else:
         option['approx'] = "P0"
         option['name'] = output.cell_data.scalars.name
+
     x_min, x_max, y_min, y_max, z_min, z_max = output.bounds
-    if (z_max - z_min == 0) and option['elevation'] and option['approx'] == "P0":
+    if (z_max - z_min == 0) and (y_max - y_min != 0) and option['elevation'] and option['approx'] == "P0":
         print "mayavi2_rheolef.py: elevation for P0 approximation not yet supported"
         option['elevation'] = 0
-    option['view_2d'] = (z_max - z_min == 0) and not option['elevation']
-    check_stereo_axis (view,option)
+    if (y_max - y_min == 0):
+        option['elevation'] = 1;
+    if ((z_max - z_min == 0) and option['cut']):
+      option['elevation'] = 1
+    option['view_2d'] = ((z_max - z_min == 0) and not option['elevation'])
+    option['view_1d'] = (z_max - z_min == 0 and y_max - y_min == 0)
 
-    if (z_max - z_min == 0):
+    if (z_max - z_min == 0 and y_max - y_min == 0):
+	# 1D data:
+        print "1D data..."
+
+        elevation = WarpScalar()
+        view.add_module(elevation)
+        
+        surface = Surface()
+        view.add_module(surface)
+        surface.enable_contours = True;
+        surface.contour.filled_contours = True;
+	surface.actor.mapper.scalar_visibility = False;
+
+	mm = surface.module_manager
+	ms = mm.scalar_lut_manager
+        print "ms=",ms
+        ms.show_legend = False
+        ms.render()
+	range = tuple(ms.data_range)
+        f_min = range[0]
+        f_max = range[1]
+	if (f_max-f_min > 0):
+            scale = 0.5*(x_max-x_min)/(f_max - f_min)
+	else:
+            scale = 1.0
+ 	scale = option['scale']*scale
+        elevation.filter.scale_factor = scale
+        elevation.filter.normal = 0, 1, 0; # 2d-elevation: y=f(x)
+        elevation.filter.use_normal = True
+ 	option['ranges'] = (x_min, x_max, f_min, f_max, 0, 0)
+        elevation.render()
+        scene.z_plus_view()
+
+    elif (z_max - z_min == 0):
 	# 2D data
         if option['elevation']:
             elevation = WarpScalar()
@@ -434,12 +557,25 @@ def mayavi2_field_scalar_P1 (view, basename, option):
             mm.scalar_lut_manager.number_of_labels = option['n_isovalue']+1
         if option['elevation']:
 	    if (f_max-f_min > 0):
-                scale = 0.5*numpy.sqrt((x_max-x_min)**2+(y_max-y_min))/(f_max - f_min)
+                scale = 0.5*numpy.sqrt((x_max-x_min)**2+(y_max-y_min)**2)/(f_max - f_min)
 	    else:
                 scale = 1.0
  	    scale = option['scale']*scale
             elevation.filter.scale_factor = scale
+ 	    option['ranges'] = (x_min, x_max, y_min, y_max, f_min, f_max)
             elevation.render()
+        if option['cut']:
+            scalar_cut_plane = ScalarCutPlane()
+            scalar_cut_plane.implicit_plane.widget.outline_translation = False
+            view.add_module(scalar_cut_plane)
+	    scalar_cut_plane.implicit_plane._set_normal(option['normal'])
+            if option['origin'][0] < 1e+10:
+	      scalar_cut_plane.implicit_plane._set_origin(option['origin'])
+ 	    scalar_cut_plane.actor.mapper.scalar_visibility = False
+ 	    prop = scalar_cut_plane.actor.property
+	    #print "prop=",prop
+ 	    prop.line_width = 4
+
     else:
 	# 3D
         view.engine.current_scene.scene.isometric_view()
@@ -455,15 +591,18 @@ def mayavi2_field_scalar_P1 (view, basename, option):
 	    mm = None
             if option['view_map']:
                 surface = Surface()
+	        # avoid smoothed iso-surface contours on 3D surface:
+ 		surface.actor.mapper.interpolate_scalars_before_mapping = True
                 view.add_module(surface)
 	        mm = surface.module_manager
                 option['cut'] = 0
-            if option['iso'] and option['approx'] == "P1":
+            if (option['iso'] or option['view_map']) and option['approx'] == "P1":
                 iso_surface = IsoSurface()
                 if option['view_map']:
 		    iso_surface.contour.auto_contours = True
-		    iso_surface.contour.number_of_contours = option['n_isovalue']
+		    iso_surface.contour.number_of_contours = option['n_isovalue']+2
  		    iso_surface.actor.mapper.scalar_visibility = False
+ 		    iso_surface.actor.property.line_width = 3
 
                 view.add_module(iso_surface)
                 if not option['fill']:
@@ -489,6 +628,8 @@ def mayavi2_field_scalar_P1 (view, basename, option):
 		    scalar_cut_plane.contour.filled_contours = True
 		    scalar_cut_plane.contour.auto_contours = True
 		    scalar_cut_plane.contour.number_of_contours = option['n_isovalue']
+	        if mm == None:
+                    mm = scalar_cut_plane.module_manager
             if option['approx'] == "P0":
                 if option['cut']:
                     polydata_filter = UserDefined(filter=tvtk.GeometryFilter(), name='GeometryFilter')
@@ -496,30 +637,29 @@ def mayavi2_field_scalar_P1 (view, basename, option):
                     clip_filter = UserDefined(filter=tvtk.ClipPolyData(), name='ClipPolyData')
                     cut_plane = scalar_cut_plane.implicit_plane.plane
                     clip_filter.filter.clip_function = cut_plane
+                    clip_filter.filter.inside_out = 1
                     view.add_module(clip_filter)
                 surface = Surface()
                 view.add_module(surface)
-	        mm = surface.module_manager
+	        if mm == None:
+	            mm = surface.module_manager
 
     #print "mm.scalar_lut_manager = ", mm.scalar_lut_manager
-    if option['color'] != 'black_and_white' and mm != None:
+    if not option['view_1d'] and option['color'] != 'black_and_white' and mm != None:
         mm.scalar_lut_manager.show_scalar_bar = True
-        if option['label'] != "output" and option['name'] != "scalars":
+        if option['label'] != "output" and (option['name'] != "scalars" and option['name'] != "unamed"):
             title = option['name'] + ": " + option['label']
         elif option['label'] != "output":
             title = option['label']
-        elif option['name'] != "scalars":
+        elif (option['name'] != "scalars" and option['name'] != "unamed"):
             title = option['name']
         else:
             title = ""
         mm.scalar_lut_manager.data_name = title
 
+    check_stereo_axis (view,option)
     show_scene (scene)
 
-def mayavi2_field_scalar (view, basename, option):
-
-    mayavi2_field_scalar_P1 (view, basename, option)
-
 # --------------------------------------------------------------------------
 # vector field visualization
 # --------------------------------------------------------------------------
