@@ -95,51 +95,62 @@ namespace rheolef {
 // -------------------------------------------------------------
 namespace rheolef {
 
-template <class T>
-class array_seq_rep : public std::vector<T> {
+template <class T, class A>
+class array_seq_rep : public std::vector<T,A> {
 public:
-    typedef T                                       value_type;
-    typedef typename std::vector<T>::size_type      size_type;
-    typedef typename std::vector<T>::iterator       iterator;
-    typedef typename std::vector<T>::const_iterator const_iterator;
-    typedef typename std::vector<T>::const_reference const_reference;
-    typedef typename std::vector<T>::reference      reference;
-    typedef reference	                            dis_reference;
-    typedef distributor::communicator_type          communicator_type;
-    typedef sequential                              memory_type;
+    typedef T                                         value_type;
+    typedef A                                         allocator_type;
+    typedef typename A::difference_type               difference_type;
+    typedef std::vector<T,A>                          base;
+    typedef typename base::size_type                  size_type;
+    typedef typename base::iterator                   iterator;
+    typedef typename base::const_iterator             const_iterator;
+    typedef typename base::const_reference            const_reference;
+    typedef typename base::reference                  reference;
+    typedef reference	                              dis_reference;
+    typedef distributor::communicator_type            communicator_type;
+    typedef sequential                                memory_type;
 
-    array_seq_rep (const distributor& ownership, const T& init_val = T());
+    explicit array_seq_rep (const A& alloc = A());
+    array_seq_rep (const distributor& ownership, const T& init_val = T(), const A& alloc = A());
     void resize   (const distributor& ownership, const T& init_val = T());
-    array_seq_rep (size_type loc_size = 0,       const T& init_val = T());
+    array_seq_rep (size_type loc_size = 0,       const T& init_val = T(), const A& alloc = A());
     void resize   (size_type loc_size = 0,       const T& init_val = T());
-    array_seq_rep (const array_seq_rep<T>& x);
+    array_seq_rep (const array_seq_rep<T,A>& x);
 
-    size_type size() const { return std::vector<T>::size(); }
-    iterator begin() { return std::vector<T>::begin(); }
-    const_iterator begin() const { return std::vector<T>::begin(); }
-    iterator end() { return std::vector<T>::end(); }
-    const_iterator end() const { return std::vector<T>::end(); }
+    A get_allocator() const { return base::get_allocator(); }
+    size_type size() const { return base::size(); }
+    iterator begin() { return base::begin(); }
+    const_iterator begin() const { return base::begin(); }
+    iterator end() { return base::end(); }
+    const_iterator end() const { return base::end(); }
     const distributor& ownership() const { return _ownership; }
 
-    reference       operator[] (size_type i)       { return std::vector<T>::operator[] (i); }
-    const_reference operator[] (size_type i) const { return std::vector<T>::operator[] (i); }
+    reference       operator[] (size_type i)       { return base::operator[] (i); }
+    const_reference operator[] (size_type i) const { return base::operator[] (i); }
+    const_reference dis_at (size_type dis_i) const { return operator[] (dis_i); }
    
-    size_type dis_size () const { return std::vector<T>::size(); }
+    size_type dis_size () const { return base::size(); }
     size_type first_index () const { return 0; }
-    size_type last_index () const { return std::vector<T>::size(); }
+    size_type last_index () const { return base::size(); }
     reference dis_entry (size_type dis_i) { return operator[](dis_i); }
+    void reset_dis_indexes() const {}
     template<class SetOp> void dis_entry_assembly_begin (SetOp) {}
     template<class SetOp> void dis_entry_assembly_end (SetOp) {}
     void dis_entry_assembly_begin () {}
     void dis_entry_assembly_end () {}
     void repartition (					      // old_numbering for *this
-        const array_seq_rep<size_type>& partition,	      // old_ownership
-        array_seq_rep<T>&               new_array,	      // new_ownership (created)
-        array_seq_rep<size_type>&       old_numbering,	      // new_ownership
-        array_seq_rep<size_type>&       new_numbering) const  // old_ownership
+        const array_seq_rep<size_type,A>& partition,	      // old_ownership
+        array_seq_rep<T,A>&               new_array,	      // new_ownership (created)
+        array_seq_rep<size_type,A>&       old_numbering,      // new_ownership
+        array_seq_rep<size_type,A>&       new_numbering) const // old_ownership
     {
 	error_macro ("not yet");
     }
+    template<class A2>
+    void reverse_permutation (                                // old_ownership for *this=iold2dis_inew
+        array_seq_rep<size_type,A2>& inew2dis_iold) const;    // new_ownership
+
     idiststream& get_values (idiststream& s);
     odiststream& put_values (odiststream& s) const;
     template <class GetFunction> idiststream& get_values (idiststream& ips, GetFunction get_element);
@@ -153,22 +164,27 @@ protected:
 // the distributed representation
 // -------------------------------------------------------------
 #ifdef _RHEOLEF_HAVE_MPI
-template <class T>
-class array_mpi_rep : public array_seq_rep<T> {
+template <class T, class A>
+class array_mpi_rep : public array_seq_rep<T,A> {
 public:
 
 // typedefs:
 
-    typedef typename array_seq_rep<T>::value_type     value_type;
-    typedef typename array_seq_rep<T>::size_type      size_type;
-    typedef typename array_seq_rep<T>::reference      reference;
-    typedef typename array_seq_rep<T>::iterator       iterator;
-    typedef typename array_seq_rep<T>::const_iterator const_iterator;
-    typedef distributor::communicator_type            communicator_type;
-    typedef distributed                               memory_type;
+    typedef array_seq_rep<T,A>         		base;
+    typedef typename base::value_type     	value_type;
+    typedef typename base::size_type      	size_type;
+    typedef typename base::difference_type      difference_type;
+    typedef typename base::reference      	reference;
+    typedef typename base::const_reference      const_reference;
+    typedef typename base::iterator       	iterator;
+    typedef typename base::const_iterator 	const_iterator;
+    typedef distributor::communicator_type      communicator_type;
+    typedef distributed                         memory_type;
+    typedef std::map <size_type, T, std::less<size_type>, heap_allocator<std::pair<size_type,T> > > 
+						scatter_map_type;
 
     struct dis_reference {
-      dis_reference (array_mpi_rep<T>& x, size_type dis_i)
+      dis_reference (array_mpi_rep<T,A>& x, size_type dis_i)
        : _x(x), _dis_i(dis_i) {}
 
       dis_reference& operator= (const T& value) {
@@ -181,33 +197,24 @@ public:
       }
     // data:
     protected:
-      array_mpi_rep<T>& _x;
-      size_type         _dis_i;
+      array_mpi_rep<T,A>& _x;
+      size_type           _dis_i;
     };
 
 // allocators:
 
-#ifdef TO_CLEAN
-    array_mpi_rep (
-    	size_type dis_size = 0,
-	const T&  init_val = T(),
-    	size_type loc_size = distributor::decide); 
-    void resize (
-    	size_type dis_size = 0,
-	const T&  init_val = T(),
-    	size_type loc_size = distributor::decide);
-#endif // TO_CLEAN
-    array_mpi_rep (const distributor& ownership, const T&  init_val = T());
+    array_mpi_rep (const distributor& ownership, const T&  init_val = T(), const A& alloc = A());
     void resize   (const distributor& ownership, const T&  init_val = T());
-    array_mpi_rep (const array_mpi_rep<T>& x);
+    array_mpi_rep (const array_mpi_rep<T,A>& x);
 
-    size_type size() const       { return array_seq_rep<T>::size(); }
-    const_iterator begin() const { return array_seq_rep<T>::begin(); }
-    const_iterator end() const   { return array_seq_rep<T>::end(); }
-    iterator begin()             { return array_seq_rep<T>::begin(); }
-    iterator end()               { return array_seq_rep<T>::end(); }
+    A get_allocator() const        { return base::get_allocator(); }
+    size_type size() const         { return base::size(); }
+    const_iterator begin() const   { return base::begin(); }
+    const_iterator end() const     { return base::end(); }
+    iterator begin()               { return base::begin(); }
+    iterator end()                 { return base::end(); }
 
-    const distributor& ownership() const  { return array_seq_rep<T>::_ownership; }
+    const distributor& ownership() const  { return base::_ownership; }
     const mpi::communicator& comm() const { return ownership().comm(); }
     size_type first_index () const        { return ownership().first_index(); }
     size_type last_index () const         { return ownership().last_index(); }
@@ -225,28 +232,47 @@ public:
     void dis_entry_assembly ()       { dis_entry_assembly_begin (); dis_entry_assembly_end (); }
 
     template<class Set, class Map>
-    void get_dis_entry (const Set& ext_idx_set, Map& ext_idx_map) const;
+    void append_dis_entry (const Set& ext_idx_set, Map& ext_idx_map) const;
+
+    template<class Set, class Map>
+    void get_dis_entry (const Set& ext_idx_set, Map& ext_idx_map) const {
+    	    ext_idx_map.clear();
+    	    append_dis_entry (ext_idx_set, ext_idx_map);
+	}
 
     template<class Set>
-    void set_dis_indexes (const Set& ext_idx_set) { get_dis_entry (ext_idx_set, _ext_x); }
+    void append_dis_indexes (const Set& ext_idx_set) const { append_dis_entry (ext_idx_set, _ext_x); }
 
-    const T& dis_at (size_type dis_i) const;
+    template<class Set>
+    void set_dis_indexes    (const Set& ext_idx_set) const { get_dis_entry    (ext_idx_set, _ext_x); }
+    void reset_dis_indexes() const;
 
+    const_reference dis_at (size_type dis_i) const;
+
+    // get all external pairs (dis_i, values):
+    const scatter_map_type& get_dis_map_entries() const { return _ext_x; }
+
+    template<class A2>
     void repartition (					      // old_numbering for *this
-        const array_mpi_rep<size_type>& partition,	      // old_ownership
-        array_mpi_rep<T>&               new_array,	      // new_ownership (created)
-        array_mpi_rep<size_type>&       old_numbering,	      // new_ownership
-        array_mpi_rep<size_type>&       new_numbering) const; // old_ownership
+        const array_mpi_rep<size_type,A2>& partition,	      // old_ownership
+        array_mpi_rep<T,A>&                new_array,	      // new_ownership (created)
+        array_mpi_rep<size_type,A2>&       old_numbering,     // new_ownership
+        array_mpi_rep<size_type,A2>&       new_numbering) const; // old_ownership
 
-    void permutation_apply (				      // old_numbering for *this
-        const array_mpi_rep<size_type>& new_numbering,        // old_ownership
-        array_mpi_rep<T>&               new_array) const;     // new_ownership (already allocated)
+    template<class A2>
+    void permutation_apply (		 		       // old_numbering for *this
+        const array_mpi_rep<size_type,A2>& new_numbering,      // old_ownership
+        array_mpi_rep<T,A>&                new_array) const;   // new_ownership (already allocated)
+
+    template<class A2>
+    void reverse_permutation (                                // old_ownership for *this=iold2dis_inew
+        array_mpi_rep<size_type,A2>& inew2dis_iold) const;    // new_ownership
 
     idiststream& get_values (idiststream& s);
     odiststream& put_values (odiststream& s) const;
     template <class GetFunction> idiststream& get_values (idiststream& ips, GetFunction get_element);
     template <class PutFunction> odiststream& put_values (odiststream& ops, PutFunction put_element) const;
-    template <class PutFunction> odiststream& permuted_put_values (odiststream& ops, const array_mpi_rep<size_type>& perm,
+    template <class PutFunction, class A2> odiststream& permuted_put_values (odiststream& ops, const array_mpi_rep<size_type,A2>& perm,
 		PutFunction put_element) const;
     void dump (std::string name) const;
 protected:
@@ -269,31 +295,27 @@ protected:
     };
     typedef typename is_container_of_mpi_datatype<T>::type is_container;
     typedef typename stash_traits<T,is_container>::mapped_type stash_value;
-    typedef typename stash_traits<T,is_container>::map_type    map_type;
-#ifdef TODO
-    typedef std::map <size_type, T, std::less<size_type>, heap_allocator<std::pair<size_type,T> > >  scatter_map_type;
-#endif // TODO
-    typedef std::map <size_type, T>  scatter_map_type;
+    typedef typename stash_traits<T,is_container>::map_type    stash_map_type;
 
     /** 2) message: for communication during assembly_begin(), assembly_end()
       */
     struct message_type {
-        std::list<std::pair<size_t,mpi::request> >       waits;
-        std::vector<std::pair<size_type,stash_value> >   data;
+        std::list<std::pair<size_type,mpi::request>,A>    waits;
+        std::vector<std::pair<size_type,stash_value>,A>   data;
     };
     /** 3) scatter (get_entry): specialized versions for T=container and T=simple type
       */
     template<class Set, class Map>
-    void get_dis_entry (const Set& ext_idx_set, Map& ext_idx_map, boost::mpl::true_) const;
+    void append_dis_entry (const Set& ext_idx_set, Map& ext_idx_map, boost::mpl::true_) const;
     template<class Set, class Map>
-    void get_dis_entry (const Set& ext_idx_set, Map& ext_idx_map, boost::mpl::false_) const;
+    void append_dis_entry (const Set& ext_idx_set, Map& ext_idx_map, boost::mpl::false_) const;
 
 // data:
-    map_type         _stash;		// for assembly msgs:
+    stash_map_type   _stash;		// for assembly msgs:
     message_type     _send;
     message_type     _receive;
     size_type        _receive_max_size;
-    scatter_map_type _ext_x;		// for ext values (scatter)
+    mutable scatter_map_type _ext_x;		// for ext values (scatter)
 };
 #endif // _RHEOLEF_HAVE_MPI
 // -------------------------------------------------------------
@@ -301,16 +323,16 @@ protected:
 // the user-level class with memory-model parameter
 // -------------------------------------------------------------
 /*Class:array
-NAME:  array - container in distributed environment (@PACKAGE@-@VERSION@)
+NAME:  @code{array} - container in distributed environment (@PACKAGE@-@VERSION@)
 SYNOPSYS:       
- STL-like vector container for a distributed memory machine model.
+  STL-like vector container for a distributed memory machine model.
 EXAMPLE:
    A sample usage of the class is:
    @example
      int main(int argc, char**argv) @{
         environment distributed(argc, argv);
         array<double> x(distributor(100), 3.14);
-        dcout << x << endl;
+        dout << x << endl;
      @}
    @end example
    The array<T> interface is similar to those of the std::vector<T> with the 
@@ -378,26 +400,27 @@ IMPLEMENTATION NOTE:
 AUTHOR: Pierre.Saramito@imag.fr
 End:
 */
-template <class T, class M = rheo_default_memory_model>
+template <class T, class M = rheo_default_memory_model, class A = std::allocator<T> >
 class array {
 public:
     typedef M memory_type;
-    typedef typename std::vector<T>::size_type      size_type;
-    typedef typename std::vector<T>::iterator       iterator;
-    typedef typename std::vector<T>::const_iterator const_iterator;
+    typedef typename std::vector<T,A>::size_type      size_type;
+    typedef typename std::vector<T,A>::iterator       iterator;
+    typedef typename std::vector<T,A>::const_iterator const_iterator;
 };
 //<verbatim:
-template <class T>
-class array<T,sequential> : public smart_pointer<array_seq_rep<T> > {
+template <class T, class A>
+class array<T,sequential,A> : public smart_pointer<array_seq_rep<T,A> > {
 public:
 
 // typedefs:
 
-    typedef array_seq_rep<T>  		  rep;
+    typedef array_seq_rep<T,A>  	  rep;
     typedef smart_pointer<rep> 		  base;
 
     typedef sequential 			  memory_type;
     typedef typename rep::size_type 	  size_type;
+    typedef typename rep::difference_type difference_type;
     typedef typename rep::value_type 	  value_type;
     typedef typename rep::reference 	  reference;
     typedef typename rep::dis_reference   dis_reference;
@@ -408,13 +431,14 @@ public:
 // allocators:
 
 
-    array       (size_type loc_size = 0,       const T& init_val = T());
+    array       (size_type loc_size = 0,       const T& init_val = T(), const A& alloc = A());
     void resize (size_type loc_size = 0,       const T& init_val = T());
-    array       (const distributor& ownership, const T& init_val = T());
+    array       (const distributor& ownership, const T& init_val = T(), const A& alloc = A());
     void resize (const distributor& ownership, const T& init_val = T());
 
 // local accessors & modifiers:
 
+    A get_allocator() const              { return base::data().get_allocator(); }
     size_type     size () const          { return base::data().size(); }
     size_type dis_size () const          { return base::data().dis_size(); }
     const distributor& ownership() const { return base::data().ownership(); }
@@ -422,6 +446,9 @@ public:
 
     reference       operator[] (size_type i)       { return base::data().operator[] (i); }
     const_reference operator[] (size_type i) const { return base::data().operator[] (i); }
+    reference       operator() (size_type i)       { return base::data().operator[] (i); }
+    const_reference operator() (size_type i) const { return base::data().operator[] (i); }
+    const_reference dis_at (size_type dis_i) const { return operator[] (dis_i); }
 
           iterator begin()       { return base::data().begin(); }
     const_iterator begin() const { return base::data().begin(); }
@@ -432,29 +459,36 @@ public:
 
     dis_reference dis_entry (size_type dis_i) { return base::data().dis_entry(dis_i); }
     void dis_entry_assembly()                 {}
-    template<class SetOp>
-    void dis_entry_assembly(SetOp my_set_op)        {}
-    template<class SetOp>
-    void dis_entry_assembly_begin (SetOp my_set_op) {}
-    template<class SetOp>
-    void dis_entry_assembly_end (SetOp my_set_op)   {}
+    template<class SetOp> void dis_entry_assembly(SetOp my_set_op)        {}
+    template<class SetOp> void dis_entry_assembly_begin (SetOp my_set_op) {}
+    template<class SetOp> void dis_entry_assembly_end (SetOp my_set_op)   {}
+
+    void reset_dis_indexes() const {}
+    template<class Set> void set_dis_indexes    (const Set& ext_idx_set) const {}
+    template<class Set> void append_dis_indexes (const Set& ext_idx_set) const {}
+    template<class Set, class Map> void append_dis_entry (const Set& ext_idx_set, Map& ext_idx_map) const {}
+    template<class Set, class Map> void get_dis_entry    (const Set& ext_idx_set, Map& ext_idx_map) const {}
 
 // apply a partition:
  
     template<class RepSize>
-    void repartition (			           // old_numbering for *this
-        const RepSize&       partition,	           // old_ownership
-        array<T,sequential>& new_array,	           // new_ownership (created)
-        RepSize&             old_numbering,	   // new_ownership
-        RepSize&             new_numbering) const  // old_ownership
+    void repartition (			             // old_numbering for *this
+        const RepSize&         partition,	     // old_ownership
+        array<T,sequential,A>& new_array,	     // new_ownership (created)
+        RepSize&               old_numbering,	     // new_ownership
+        RepSize&               new_numbering) const  // old_ownership
         { return base::data().repartition (partition, new_array, old_numbering, new_numbering); }
 
     template<class RepSize>
-    void permutation_apply (                     // old_numbering for *this
-        const RepSize&        new_numbering,     // old_ownership
-        array<T,sequential>&  new_array) const   // new_ownership (already allocated)
+    void permutation_apply (                       // old_numbering for *this
+        const RepSize&          new_numbering,     // old_ownership
+        array<T,sequential,A>&  new_array) const   // new_ownership (already allocated)
         { return base::data().permutation_apply (new_numbering, new_array); }
 
+    void reverse_permutation (                                 // old_ownership for *this=iold2dis_inew
+        array<size_type,sequential,A>& inew2dis_iold) const   // new_ownership
+        { base::data().reverse_permutation (inew2dis_iold.data()); }
+
 // i/o:
 
     odiststream& put_values (odiststream& ops) const { return base::data().put_values(ops); }
@@ -466,35 +500,37 @@ public:
     void dump (std::string name) const { return base::data().dump(name); }
 };
 //>verbatim:
-template <class T>
+template <class T, class A>
 inline
-array<T,sequential>::array (
+array<T,sequential,A>::array (
     	size_type loc_size,
-	const T&  init_val)
- : base(new_macro(rep(loc_size,init_val)))
+	const T&  init_val,
+        const A&  alloc)
+ : base(new_macro(rep(loc_size,init_val,alloc)))
 {
 }
-template <class T>
+template <class T, class A>
 inline
-array<T,sequential>::array (
+array<T,sequential,A>::array (
     	const distributor& ownership,
-	const T&           init_val)
- : base(new_macro(rep(ownership,init_val)))
+	const T&           init_val,
+        const A&           alloc)
+ : base(new_macro(rep(ownership,init_val,alloc)))
 {
 }
-template <class T>
+template <class T, class A>
 inline
 void
-array<T,sequential>::resize (
+array<T,sequential,A>::resize (
     	size_type loc_size,
 	const T&  init_val)
 {
   base::data().resize (loc_size,init_val);
 }
-template <class T>
+template <class T, class A>
 inline
 void
-array<T,sequential>::resize (
+array<T,sequential,A>::resize (
     	const distributor& ownership,
 	const T&           init_val)
 {
@@ -502,31 +538,34 @@ array<T,sequential>::resize (
 }
 #ifdef _RHEOLEF_HAVE_MPI
 //<verbatim:
-template <class T>
-class array<T,distributed> : public smart_pointer<array_mpi_rep<T> > {
+template <class T, class A>
+class array<T,distributed,A> : public smart_pointer<array_mpi_rep<T,A> > {
 public:
 
 // typedefs:
 
-    typedef array_mpi_rep<T>  		  rep;
+    typedef array_mpi_rep<T,A>  	  rep;
     typedef smart_pointer<rep> 		  base;
 
     typedef distributed 		  memory_type;
     typedef typename rep::size_type 	  size_type;
+    typedef typename rep::difference_type difference_type;
     typedef typename rep::value_type 	  value_type;
     typedef typename rep::reference 	  reference;
     typedef typename rep::dis_reference   dis_reference;
     typedef typename rep::iterator 	  iterator;
     typedef typename rep::const_reference const_reference;
     typedef typename rep::const_iterator  const_iterator;
+    typedef typename rep::scatter_map_type scatter_map_type;
 
 // allocators:
 
-    array       (const distributor& ownership = distributor(), const T& init_val = T());
+    array       (const distributor& ownership = distributor(), const T& init_val = T(), const A& alloc = A());
     void resize (const distributor& ownership = distributor(), const T& init_val = T());
 
 // local accessors & modifiers:
 
+    A get_allocator() const              { return base::data().get_allocator(); }
     size_type     size () const          { return base::data().size(); }
     size_type dis_size () const          { return base::data().dis_size(); }
     const distributor& ownership() const { return base::data().ownership(); }
@@ -534,6 +573,8 @@ public:
 
     reference       operator[] (size_type i)       { return base::data().operator[] (i); }
     const_reference operator[] (size_type i) const { return base::data().operator[] (i); }
+    reference       operator() (size_type i)       { return base::data().operator[] (i); }
+    const_reference operator() (size_type i) const { return base::data().operator[] (i); }
 
           iterator begin()       { return base::data().begin(); }
     const_iterator begin() const { return base::data().begin(); }
@@ -543,13 +584,23 @@ public:
 // global accessor:
 
     template<class Set, class Map>
-    void get_dis_entry (const Set& ext_idx_set, Map& ext_idx_map) const { base::data().get_dis_entry (ext_idx_set, ext_idx_map); }
+    void append_dis_entry (const Set& ext_idx_set, Map& ext_idx_map) const { base::data().append_dis_entry (ext_idx_set, ext_idx_map); }
+
+    template<class Set, class Map>
+    void get_dis_entry    (const Set& ext_idx_set, Map& ext_idx_map) const { base::data().get_dis_entry (ext_idx_set, ext_idx_map); }
 
     template<class Set>
-    void set_dis_indexes (const Set& ext_idx_set)  { base::data().set_dis_indexes (ext_idx_set); }
+    void append_dis_indexes (const Set& ext_idx_set) const { base::data().append_dis_indexes (ext_idx_set); }
+    void reset_dis_indexes() const { base::data().reset_dis_indexes(); }
+
+    template<class Set>
+    void set_dis_indexes    (const Set& ext_idx_set) const { base::data().set_dis_indexes (ext_idx_set); }
 
     const T& dis_at (size_type dis_i) const { return base::data().dis_at (dis_i); }
 
+    // get all external pairs (dis_i, values):
+    const scatter_map_type& get_dis_map_entries() const { return base::data().get_dis_map_entries(); }
+
 // global modifiers (for compatibility with distributed interface):
 
     dis_reference dis_entry (size_type dis_i)       { return base::data().dis_entry(dis_i); }
@@ -574,11 +625,15 @@ public:
         { return base::data().repartition (partition.data(), new_array.data(), old_numbering.data(), new_numbering.data()); }
 
     template<class RepSize>
-    void permutation_apply (                     // old_numbering for *this
-        const RepSize&        new_numbering,     // old_ownership
-        array<T,distributed>& new_array) const   // new_ownership (already allocated)
+    void permutation_apply (                       // old_numbering for *this
+        const RepSize&          new_numbering,     // old_ownership
+        array<T,distributed,A>& new_array) const   // new_ownership (already allocated)
         { base::data().permutation_apply (new_numbering.data(), new_array.data()); }
 
+    void reverse_permutation (                                 // old_ownership for *this=iold2dis_inew
+        array<size_type,distributed,A>& inew2dis_iold) const   // new_ownership
+        { base::data().reverse_permutation (inew2dis_iold.data()); }
+
 // i/o:
 
     odiststream& put_values (odiststream& ops) const { return base::data().put_values(ops); }
@@ -589,23 +644,24 @@ public:
     idiststream& get_values (idiststream& ips, GetFunction get_element)       { return base::data().get_values(ips, get_element); }
     template <class PutFunction>
     odiststream& put_values (odiststream& ops, PutFunction put_element) const { return base::data().put_values(ops, put_element); }
-    template <class PutFunction> odiststream& permuted_put_values (
-		odiststream& ops, const array<size_type,distributed>& perm, PutFunction put_element) const 
+    template <class PutFunction, class A2> odiststream& permuted_put_values (
+		odiststream& ops, const array<size_type,distributed,A2>& perm, PutFunction put_element) const 
 								     { return base::data().permuted_put_values (ops, perm.data(), put_element); }
 };
 //>verbatim:
-template <class T>
+template <class T, class A>
 inline
-array<T,distributed>::array (
+array<T,distributed,A>::array (
     	const distributor& ownership,
-	const T&           init_val)
- : base(new_macro(rep(ownership,init_val)))
+	const T&           init_val,
+        const A&           alloc)
+ : base(new_macro(rep(ownership,init_val,alloc)))
 {
 }
-template <class T>
+template <class T, class A>
 inline
 void
-array<T,distributed>::resize (
+array<T,distributed,A>::resize (
     	const distributor& ownership,
 	const T         &  init_val)
 {
@@ -616,32 +672,32 @@ array<T,distributed>::resize (
 // -------------------------------------------------------------
 // i/o with operator<< & >>
 // -------------------------------------------------------------
-template <class T>
+template <class T, class A>
 inline
 idiststream&
-operator >> (idiststream& ips,  array<T,sequential>& x)
+operator >> (idiststream& ips,  array<T,sequential,A>& x)
 { 
     return x.get_values(ips); 
 }
-template <class T> 
+template <class T, class A>
 inline
 odiststream&
-operator << (odiststream& ops, const array<T,sequential>& x)
+operator << (odiststream& ops, const array<T,sequential,A>& x)
 {
     return x.put_values(ops);
 }
 #ifdef _RHEOLEF_HAVE_MPI
-template <class T>
+template <class T, class A>
 inline
 idiststream&
-operator >> (idiststream& ips,  array<T,distributed>& x)
+operator >> (idiststream& ips,  array<T,distributed,A>& x)
 { 
     return x.get_values(ips); 
 }
-template <class T> 
+template <class T, class A>
 inline
 odiststream&
-operator << (odiststream& ops, const array<T,distributed>& x)
+operator << (odiststream& ops, const array<T,distributed,A>& x)
 {
     return x.put_values(ops);
 }
