@@ -27,8 +27,8 @@
 // instanciation in library
 // ----------------------------------------------------------------------------
 namespace rheolef {
-template class array_mpi_rep<Float>;
-template class array_mpi_rep<int>;
-template class array_mpi_rep<size_t>;
+template class array_mpi_rep<Float,  std::allocator<Float> >;
+template class array_mpi_rep<int,    std::allocator<int> >;
+template class array_mpi_rep<size_t, std::allocator<size_t> >;
 } // namespace rheolef
 # endif // _RHEOLEF_HAVE_MPI
