@@ -84,6 +84,9 @@ public:
     typedef T                                             element_type;
     typedef sequential                                    memory_type;
     typedef distributor::communicator_type                communicator_type;
+    typedef std::vector<std::map<size_type, T,
+            std::less<size_type>,
+	    heap_allocator<std::pair<size_type,T> > > >   base;
     typedef std::map <size_type, T, std::less<size_type>,
             heap_allocator<std::pair<size_type,T> > >     map_type;     // (j, value) collection
     typedef std::map<std::pair<size_type, size_type>, T, 
@@ -112,15 +115,15 @@ public:
     size_type col_first_index () const { return 0; }
     size_type col_last_index () const { return ncol(); }
     T& dis_entry (size_type i, size_type j) {
-	row_type& row_i = operator[](i);
+	row_type& row_i = base::operator[](i);
         std::pair<typename row_type::iterator,bool> status
 	= row_i.insert (typename row_type::value_type(j,T(0)));
         if (status.second) _nnz++;
 	return (*(status.first)).second;
       }
     T operator() (size_type i, size_type j) const {
-        typename row_type::const_iterator pos_aij  = operator[](i).find(j);
-        if (pos_aij != operator[](i).end())
+        typename row_type::const_iterator pos_aij  = base::operator[](i).find(j);
+        if (pos_aij != base::operator[](i).end())
 	    return (*pos_aij).second;
 	return T(0);
       }
@@ -145,7 +148,8 @@ public:
     typedef typename asr_seq_rep<T>::size_type    size_type;
     typedef typename asr_seq_rep<T>::element_type element_type;
     typedef typename asr_seq_rep<T>::row_type     row_type;
-    typedef distributed memory_type;
+    typedef distributed                           memory_type;
+    typedef asr_seq_rep<T>                        base;
 
     asr_mpi_rep(const distributor& row_ownership, const distributor& col_ownership);
     void resize(const distributor& row_ownership, const distributor& col_ownership);
@@ -192,7 +196,7 @@ protected:
 // the user-level class with memory-model parameter
 // -------------------------------------------------------------
 /*Class:asr
-NAME:  asr - associative sparse matrix (@PACKAGE@-@VERSION@)
+NAME:  @code{asr} - associative sparse matrix (@PACKAGE@-@VERSION@)
 SYNOPSYS:       
  Associative sparse matrix container stored row by row
  using the STL map class.
