@@ -33,6 +33,10 @@
 // using namespace std;
 
 namespace rheolef {
+
+// extern:
+void read_mm_header(idiststream&);
+
 // ----------------------------------------------------------------------------
 // class member functions
 // ----------------------------------------------------------------------------
@@ -125,7 +129,7 @@ asr_mpi_rep<T>::dis_entry (size_type dis_i, size_type dis_j)
     if (dis_i >= istart && dis_i <  ilast) {
 
         // local entry
-	row_type& row_i = operator[](dis_i-istart);
+	row_type& row_i = base::operator[](dis_i-istart);
 	std::pair<typename row_type::iterator,bool> status
 	 = row_i.insert (std::pair<const size_type,T>(dis_j,T(0)));
 	if (status.second) {
@@ -191,8 +195,8 @@ asr_mpi_rep<T>::put (odiststream& ops) const
     asr<T> a (io_row_ownership, io_col_ownership);
     size_type istart = row_first_index();
     for (size_type i = 0; i < asr_seq_rep<T>::nrow(); i++) {
-        typename row_type::const_iterator iter = operator[](i).begin();
-        typename row_type::const_iterator last = operator[](i).end();
+        typename row_type::const_iterator iter = base::operator[](i).begin();
+        typename row_type::const_iterator last = base::operator[](i).end();
 	while (iter != last) {
   	    a.dis_entry (i+istart, (*iter).first) += (*iter).second;
 	    iter++;
@@ -211,7 +215,6 @@ template <class T>
 idiststream&
 asr_mpi_rep<T>::get (idiststream& ips) 
 {
-    extern void read_mm_header(idiststream&);
     read_mm_header(ips);
     size_type n_row, n_col;
     ips >> n_row >> n_col;
