@@ -22,6 +22,10 @@
 # include "rheolef/csr.h"
 using namespace std;
 namespace rheolef {
+
+// extern:
+void read_mm_header(idiststream&);
+
 // ----------------------------------------------------------------------------
 // class member functions
 // ----------------------------------------------------------------------------
@@ -82,7 +86,6 @@ template <class T>
 idiststream&
 asr_seq_rep<T>::get (idiststream& ips) 
 {
-    extern void read_mm_header(idiststream&);
     // matrix market format:
     //   %%mm_header
     //   nrow ncol nnz
@@ -112,12 +115,13 @@ odiststream&
 asr_seq_rep<T>::put (odiststream& ops, size_type istart) const
 {
     std::ostream& os = ops.os();
-    os << "%%MatrixMarket matrix coordinate real general" << std::endl
+    os << setprecision(std::numeric_limits<T>::digits10)
+       << "%%MatrixMarket matrix coordinate real general" << std::endl
        << nrow() << " " << ncol() << " " << nnz() << endl;
     if (nnz() == 0) return ops;
     for (size_type i = 0; i < nrow(); i++) {
-        typename row_type::const_iterator iter = operator[](i).begin();
-        typename row_type::const_iterator last = operator[](i).end();
+        typename row_type::const_iterator iter = base::operator[](i).begin();
+        typename row_type::const_iterator last = base::operator[](i).end();
 	while (iter != last) {
   	    os << i+istart+1 << " " 
 	       << (*iter).first+1 << " " 
