@@ -37,10 +37,11 @@ public:
     typedef T                                                             element_type;
     typedef sequential                                                    memory_type;
     typedef typename std::pair<size_type,T>                               pair_type;
-    typedef typename vector_of_iterator<pair_type>::value_type            data_iterator;
-    typedef typename vector_of_iterator<pair_type>::const_value_type      const_data_iterator;
     typedef typename vector_of_iterator<pair_type>::iterator              iterator;
     typedef typename vector_of_iterator<pair_type>::const_iterator        const_iterator;
+    typedef typename vector_of_iterator<pair_type>::difference_type       difference_type;
+    typedef typename vector_of_iterator<pair_type>::value_type            data_iterator;
+    typedef typename vector_of_iterator<pair_type>::const_value_type      const_data_iterator;
 
     csr_seq_rep (size_type loc_nrow1 = 0, size_type loc_ncol1 = 0, size_type loc_nnz1 = 0);
     void resize (size_type loc_nrow1 = 0, size_type loc_ncol1 = 0, size_type loc_nnz1 = 0);
@@ -74,9 +75,12 @@ public:
     idiststream& get (idiststream&);
     odiststream& put (odiststream&, size_type istart = 0) const;
     void dump (const std::string& name, size_type istart = 0) const;
-    void mult (const vec<T,sequential>& x, vec<T,sequential>& y) const;
+    void mult       (const vec<T,sequential>& x, vec<T,sequential>& y) const;
+    void trans_mult (const vec<T,sequential>& x, vec<T,sequential>& y) const;
+    csr_seq_rep<T>& operator*= (const T& lambda);
     template <class BinaryOp>
     void assign_add (const csr_seq_rep<T>& a, const csr_seq_rep<T>& b, BinaryOp binop);
+    void build_transpose (csr_seq_rep<T>& b) const;
 //protected:
     distributor          		                          _row_ownership;
     distributor          		                          _col_ownership;
@@ -91,12 +95,12 @@ public:
 template<class T>
 class csr_mpi_rep : public csr_seq_rep<T> {
 public:
-    typedef typename csr_seq_rep<T>::size_type size_type;
-    typedef typename csr_seq_rep<T>::element_type element_type;
-    typedef distributed memory_type;
-    typedef typename csr_seq_rep<T>::iterator iterator;
-    typedef typename csr_seq_rep<T>::const_iterator const_iterator;
-    typedef typename csr_seq_rep<T>::data_iterator data_iterator;
+    typedef typename csr_seq_rep<T>::size_type           size_type;
+    typedef typename csr_seq_rep<T>::element_type        element_type;
+    typedef distributed                                  memory_type;
+    typedef typename csr_seq_rep<T>::iterator            iterator;
+    typedef typename csr_seq_rep<T>::const_iterator      const_iterator;
+    typedef typename csr_seq_rep<T>::data_iterator       data_iterator;
     typedef typename csr_seq_rep<T>::const_data_iterator const_data_iterator;
 
     csr_mpi_rep ();
@@ -104,6 +108,7 @@ public:
     explicit csr_mpi_rep (const asr_mpi_rep<T>& a);
     void build_from_asr (const asr_mpi_rep<T>& a);
     void to_asr (asr_mpi_rep<T>& b) const;
+    void resize (const distributor& row_ownership, const distributor& col_ownership, size_type nnz1 = 0);
 
     const distributor& row_ownership() const { return csr_seq_rep<T>::_row_ownership; }
     const distributor& col_ownership() const { return csr_seq_rep<T>::_col_ownership; }
@@ -133,9 +138,12 @@ public:
     idiststream& get (idiststream&);
     odiststream& put (odiststream&) const;
     void dump (const std::string& name) const;
-    void mult(const vec<T,distributed>& x, vec<T,distributed>& y) const;
+    void mult       (const vec<T,distributed>& x, vec<T,distributed>& y) const;
+    void trans_mult (const vec<T,distributed>& x, vec<T,distributed>& y) const;
+    csr_mpi_rep<T>& operator*= (const T& lambda);
     template <class BinaryOp>
     void assign_add (const csr_mpi_rep<T>& a, const csr_mpi_rep<T>& b, BinaryOp binop);
+    void build_transpose (csr_mpi_rep<T>& b) const;
 protected:
     // diagonal part is the basic csr_seq_rep type
     // extra-diagonal blocs are sequential csr also:
@@ -157,19 +165,33 @@ csr_mpi_rep<T>::jext2dis_j (size_type jext)  const
   return _jext2dis_j [jext];
 }
 #endif // _RHEOLEF_HAVE_MPI
+
+// these classes are used for allocator from the std::initializer_list
+template <class T, class M> class csr_concat_value;
+template <class T, class M> class csr_concat_line;
+
 // -------------------------------------------------------------
 // the basic class with a smart pointer to representation
 // the user-level class with memory-model parameter
 // -------------------------------------------------------------
 /*Class:csr
-NAME:  csr - compressed sparse row matrix (@PACKAGE@-@VERSION@)
+NAME:  @code{csr} - compressed sparse row matrix (@PACKAGE@-@VERSION@)
 SYNOPSYS:       
- Distributed compressed sparse matrix container stored row by row.
-TO DO:
-   For efficiency purpose, the assembly phase may
-   access directly to the csr representation, without
-   crossing the reference counting and pointer handler.
-   Something like the iterator for dense vectors.
+  Distributed compressed sparse matrix container stored row by row.
+DESCRIPTION:
+  Sparse matrix are compressed by rows. In distributed environment, the
+  distribution follows the row distributor (see @ref{distributor class}).
+ALGEBRA:
+  Adding or substracting two matrices writes @code{a+b} and @code{a-b}, respectively,
+  and multiplying a matrix by a scalar writes @code{lambda*x}.
+  Thus, any linear combination of sparse matrices is available.
+ 
+  Matrix-vector product writes @code{a*x} where @code{x} is a vector (see @ref{vec class}).
+LIMITATIONS:
+  Some basic linear algebra is still under development:
+  @code{a.trans_mult(x)} matrix transpose vector product,
+  @code{trans(a)} matrix transpose,
+  @code{a*b} matrix product.
 AUTHORS: Pierre.Saramito@imag.fr
 DATE:   10 february 1999
 METHODS: @csr
@@ -194,12 +216,24 @@ public:
     typedef typename rep::element_type        element_type;
     typedef typename rep::iterator            iterator;
     typedef typename rep::const_iterator      const_iterator;
+    typedef typename rep::data_iterator       data_iterator;
     typedef typename rep::const_data_iterator const_data_iterator;
 
 // allocators/deallocators:
 
     csr() : base(new_macro(rep())) {}
     explicit csr(const asr<T,sequential>& a) : base(new_macro(rep(a.data()))) {}
+    void resize (size_type loc_nrow1 = 0, size_type loc_ncol1 = 0, size_type loc_nnz1 = 0)
+        { base::data().resize(loc_nrow1, loc_ncol1, loc_nnz1); }
+    void resize (const distributor& row_ownership, const distributor& col_ownership, size_type nnz1 = 0)
+        { base::data().resize(row_ownership, col_ownership, nnz1); }
+
+// allocators from initializer list (c++ 2011):
+
+#ifdef _RHEOLEF_HAVE_STD_INITIALIZER_LIST
+    csr (const std::initializer_list<csr_concat_value<T,sequential> >& init_list);
+    csr (const std::initializer_list<csr_concat_line<T,sequential> >&  init_list);
+#endif // _RHEOLEF_HAVE_STD_INITIALIZER_LIST
 
 // accessors:
 
@@ -227,32 +261,75 @@ public:
 
     const_iterator begin() const             { return base::data().begin(); }
     const_iterator end()   const             { return base::data().end(); }
+    iterator begin_nonconst()                { return base::data().begin(); }
+    iterator end_nonconst()                  { return base::data().end(); }
 
-#ifdef TO_CLEAN
-    // accessors, only for distributed
-    size_type ext_nnz() const                { return base::data().ext_nnz(); }
-    const_iterator ext_begin() const         { return base::data().ext_begin(); }
-    const_iterator ext_end()   const         { return base::data().ext_end(); }
-    size_type jext2dis_j (size_type jext) const { return base::data().jext2dis_j(); }
-#endif // TO_CLEAN
+    // accessors, only for distributed (for interface compatibility)
+    size_type ext_nnz() const                { return 0; }
+    const_iterator ext_begin() const         { return const_iterator(); }
+    const_iterator ext_end()   const         { return const_iterator(); }
+    size_type jext2dis_j (size_type jext) const { return 0; }
 
 // algebra:
 
+    // y := a*x
+    void mult (const vec<element_type,sequential>& x, vec<element_type,sequential>& y) const {
+      base::data().mult (x,y);
+    }
+    vec<element_type,sequential> operator* (const vec<element_type,sequential>& x) const {
+      vec<element_type,sequential> y (row_ownership(), element_type());
+      mult (x, y);
+      return y;
+    }
+    void trans_mult (const vec<element_type,sequential>& x, vec<element_type,sequential>& y) const {
+      base::data().trans_mult (x,y);
+    }
+    vec<element_type,sequential> trans_mult (const vec<element_type,sequential>& x) const {
+      vec<element_type,sequential> y (col_ownership(), element_type());
+      trans_mult (x, y);
+      return y;
+    }
+    // a+b, a-b
     csr<T,sequential> operator+ (const csr<T,sequential>& b) const;
     csr<T,sequential> operator- (const csr<T,sequential>& b) const;
 
-    void mult (const vec<element_type,sequential>& x, vec<element_type,sequential>& y) const
-	{ base::data().mult (x,y); }
-    vec<element_type,sequential> operator* (const vec<element_type,sequential>& x) const {
-          vec<element_type,sequential> y (row_ownership(), element_type());
-          mult (x, y);
-          return y;
+    // lambda*a
+    csr<T,sequential>& operator*= (const T& lambda) {
+      base::data().operator*= (lambda);
+      return *this;
     }
-
 // output:
 
     void dump (const std::string& name) const { base::data().dump(name); }
 };
+// lambda*a
+template<class T>
+inline
+csr<T,sequential>
+operator* (const T& lambda, const csr<T,sequential>& a)
+{
+  csr<T,sequential> b = a;
+  b.operator*= (lambda);
+  return b;
+}
+// -a
+template<class T>
+inline
+csr<T,sequential>
+operator- (const csr<T,sequential>& a)
+{
+  return T(-1)*a;
+}
+// trans(a)
+template<class T>
+inline
+csr<T,sequential>
+trans (const csr<T,sequential>& a)
+{
+  csr<T,sequential> b;
+  a.data().build_transpose (b.data());
+  return b;
+}
 //>verbatim:
 
 #ifdef _RHEOLEF_HAVE_MPI
@@ -270,12 +347,22 @@ public:
     typedef typename rep::element_type        element_type;
     typedef typename rep::iterator            iterator;
     typedef typename rep::const_iterator      const_iterator;
+    typedef typename rep::data_iterator       data_iterator;
     typedef typename rep::const_data_iterator const_data_iterator;
 
 // allocators/deallocators:
 
     csr() : base(new_macro(rep())) {}
     explicit csr(const asr<T,memory_type>& a) : base(new_macro(rep(a.data()))) {}
+    void resize (const distributor& row_ownership, const distributor& col_ownership, size_type nnz1 = 0)
+        { base::data().resize(row_ownership, col_ownership, nnz1); }
+
+// allocators from initializer list (c++ 2011):
+
+#ifdef _RHEOLEF_HAVE_STD_INITIALIZER_LIST
+    csr (const std::initializer_list<csr_concat_value<T,distributed> >& init_list);
+    csr (const std::initializer_list<csr_concat_line<T,distributed> >&  init_list);
+#endif // _RHEOLEF_HAVE_STD_INITIALIZER_LIST
 
 // accessors:
 
@@ -303,6 +390,8 @@ public:
 
     const_iterator begin() const             { return base::data().begin(); }
     const_iterator end()   const             { return base::data().end(); }
+    iterator begin_nonconst()                { return base::data().begin(); }
+    iterator end_nonconst()                  { return base::data().end(); }
 
     // accessors, only for distributed
     size_type ext_nnz() const                { return base::data().ext_nnz(); }
@@ -312,21 +401,65 @@ public:
 
 // algebra:
 
+
+    // y := a*x
+    void mult (const vec<element_type,distributed>& x, vec<element_type,distributed>& y) const {
+      base::data().mult (x,y);
+    }
+    vec<element_type,distributed> operator* (const vec<element_type,distributed>& x) const {
+      vec<element_type,distributed> y (row_ownership(), element_type());
+      mult (x, y);
+      return y;
+    }
+    void trans_mult (const vec<element_type,distributed>& x, vec<element_type,distributed>& y) const {
+      base::data().trans_mult (x,y);
+    }
+    vec<element_type,distributed> trans_mult (const vec<element_type,distributed>& x) const {
+      vec<element_type,distributed> y (col_ownership(), element_type());
+      trans_mult (x, y);
+      return y;
+    }
+    // a+b, a-b
     csr<T,distributed> operator+ (const csr<T,distributed>& b) const;
     csr<T,distributed> operator- (const csr<T,distributed>& b) const;
 
-    void mult (const vec<element_type,distributed>& x, vec<element_type,distributed>& y) const
-	{ base::data().mult (x,y); }
-    vec<element_type,distributed> operator* (const vec<element_type,distributed>& x) const {
-          vec<element_type,distributed> y (row_ownership(), element_type());
-          mult (x, y);
-          return y;
+    // lambda*a
+    csr<T,distributed>& operator*= (const T& lambda) {
+      base::data().operator*= (lambda);
+      return *this;
     }
-
 // output:
 
     void dump (const std::string& name) const { base::data().dump(name); }
 };
+// lambda*a
+template<class T>
+inline
+csr<T,distributed>
+operator* (const T& lambda, const csr<T,distributed>& a)
+{
+  csr<T,distributed> b = a;
+  b.operator*= (lambda);
+  return b;
+}
+// -a
+template<class T>
+inline
+csr<T,distributed>
+operator- (const csr<T,distributed>& a)
+{
+  return T(-1)*a;
+}
+// trans(a)
+template<class T>
+inline
+csr<T,distributed>
+trans (const csr<T,distributed>& a)
+{
+  csr<T,distributed> b;
+  a.data().build_transpose (b.data());
+  return b;
+}
 //>verbatim:
 #endif // _RHEOLEF_HAVE_MPI
 
