@@ -28,6 +28,7 @@
 #include "rheolef/csr_to_asr.h"
 #include "rheolef/asr_to_csr.h"
 #include "rheolef/csr_amux.h"
+#include "rheolef/csr_cumul_trans_mult.h"
 #include "rheolef/mpi_scatter_init.h"
 #include "rheolef/mpi_scatter_begin.h"
 #include "rheolef/mpi_scatter_end.h"
@@ -79,15 +80,21 @@ csr_mpi_rep<T>::csr_mpi_rep()
 template<class T>
 csr_mpi_rep<T>::csr_mpi_rep(const csr_mpi_rep<T>& a)
   : csr_seq_rep<T>(a),
-    _ext(a._ext.nrow(), a._ext.ncol(), a._ext.nnz()),
-    _jext2dis_j(_jext2dis_j.size()),
-    _dis_nnz(a.dis_nnz()),
-    _from(),
-    _to(),
-    _buffer()
+    _ext(a._ext),
+    _jext2dis_j(a._jext2dis_j),
+    _dis_nnz(a._dis_nnz),
+    _from(a._from),
+    _to(a._to),
+    _buffer(a._buffer)
 {
-    // TODO: performs copy of arrays
-    fatal_macro ("physical copy of csr");  
+    // "physical copy of csr"
+}
+template<class T>
+void
+csr_mpi_rep<T>::resize (const distributor& row_ownership, const distributor& col_ownership, size_type nnz1)
+{
+  csr_seq_rep<T>::resize (row_ownership, col_ownership, nnz1);
+  _ext.resize            (row_ownership, col_ownership, 0); // note: the _ext part will be resized elsewhere
 }
 template<class T>
 csr_mpi_rep<T>::csr_mpi_rep(const asr_mpi_rep<T>& a)
@@ -269,7 +276,7 @@ csr_mpi_rep<T>::mult(
     vec<T,distributed>&       y)
     const
 {
-    assert_macro (x.size() == ncol(), "incompatible partition of A and X");
+    check_macro (x.size() == ncol(), "csr*vec: incompatible csr("<<nrow()<<","<<ncol()<<") and vec("<<x.size()<<")");
     y.resize (row_ownership());
 
     distributor::tag_type tag = distributor::get_new_tag();
@@ -305,6 +312,63 @@ csr_mpi_rep<T>::mult(
     // y += ext*tmp
     csr_amux (_ext.begin(), _ext.end(), _buffer.begin(), set_add_op<T,T>(), y.begin());
 }
+template<class T>
+void
+csr_mpi_rep<T>::trans_mult(
+    const vec<T,distributed>& x,
+    vec<T,distributed>&       y)
+    const
+{
+    check_macro (x.size() == nrow(), "csr.trans_mult(vec): incompatible csr("<<nrow()<<","<<ncol()<<") and vec("<<x.size()<<")");
+    y.resize (col_ownership());
+
+    // y = dia*x
+    std::fill (y.begin(), y.end(), T(0));
+    csr_cumul_trans_mult (
+        csr_seq_rep<T>::begin(), 
+        csr_seq_rep<T>::end(), 
+        x.begin(), 
+        set_add_op<T,T>(),
+        y.begin());
+
+    // buffer = ext*x
+    std::fill (_buffer.begin(), _buffer.end(), T(0));
+    csr_cumul_trans_mult (
+        _ext.begin(),
+        _ext.end(),
+        x.begin(), 
+        set_add_op<T,T>(),
+        _buffer.begin());
+
+    // send buffer to others parts of y (+=)
+    distributor::tag_type tag = distributor::get_new_tag();
+    mpi_scatter_begin (
+        _buffer.begin().operator->(),
+	y.begin().operator->(),
+        _to,  // reverse mode
+	_from,
+	set_add_op<T,T>(), // += 
+        tag, 
+	col_ownership().comm());
+
+    // receive buffer from others
+    mpi_scatter_end (
+        _buffer.begin(), 
+	y.begin(),
+        _to, // reverse mode
+	_from,
+	set_add_op<T,T>(), // +=
+	tag, 
+	col_ownership().comm());
+}
+template<class T>
+csr_mpi_rep<T>&
+csr_mpi_rep<T>::operator*= (const T& lambda)
+{
+  csr_seq_rep<T>::operator*= (lambda);
+  _ext.operator*= (lambda);
+  return *this;
+}
 // ----------------------------------------------------------------------------
 // expression c=a+b and c=a-b with a temporary c=*this
 // ----------------------------------------------------------------------------
@@ -434,6 +498,39 @@ csr_mpi_rep<T>::assign_add (
         _to);
 }
 // ----------------------------------------------------------------------------
+// trans(a)
+// ----------------------------------------------------------------------------
+template<class T>
+void
+csr_mpi_rep<T>::build_transpose (csr_mpi_rep<T>& b) const
+{
+  //
+  // first: assembly all _ext parts of the a matrix in b=trans(a)
+  //
+  asr_mpi_rep<T> b_ext (col_ownership(), row_ownership());
+  size_type first_i = row_ownership().first_index();
+  const_iterator ext_ia = ext_begin();
+  for (size_type i = 0, n = nrow(); i < n; i++) {
+    size_type dis_i = first_i + i;
+    for (const_data_iterator p = ext_ia[i], last_p = ext_ia[i+1]; p < last_p; p++) {
+      size_type dis_j = jext2dis_j ((*p).first);
+      const T& val    = (*p).second;
+      b_ext.dis_entry (dis_j, dis_i) = val;
+    }
+  }
+  b_ext.dis_entry_assembly_begin();
+  b_ext.dis_entry_assembly_end();
+  b.build_from_asr (b_ext);
+  //
+  // second: add all _diag parts
+  //
+  csr_seq_rep<T>::build_transpose (b);
+  //
+  // third: update dis_nnz by adding all diag nnz
+  //
+  b._dis_nnz += mpi::all_reduce (comm(), csr_seq_rep<T>::nnz(), std::plus<size_type>());
+}
+// ----------------------------------------------------------------------------
 // instanciation in library
 // ----------------------------------------------------------------------------
 template class csr_mpi_rep<Float>;
