@@ -26,6 +26,7 @@
 #include "rheolef/asr_to_csr.h"
 #include "rheolef/csr_to_asr.h"
 #include "rheolef/csr_amux.h"
+#include "rheolef/csr_cumul_trans_mult.h"
 using namespace std;
 namespace rheolef {
 // ----------------------------------------------------------------------------
@@ -50,6 +51,8 @@ csr_seq_rep<T>::resize (const distributor& row_ownership, const distributor& col
    _row_ownership = row_ownership;
    _col_ownership = col_ownership;
    _data.resize (nnz1);
+   // first pointer points to the beginning of the data:
+   vector_of_iterator<pair_type>::operator[](0) = _data.begin().operator->();
 }
 template<class T>
 csr_seq_rep<T>::csr_seq_rep(size_type loc_nrow1, size_type loc_ncol1, size_type loc_nnz1) 
@@ -69,19 +72,27 @@ csr_seq_rep<T>::resize (size_type loc_nrow1, size_type loc_ncol1, size_type loc_
    _row_ownership = distributor (distributor::decide, communicator(), loc_nrow1);
    _col_ownership = distributor (distributor::decide, communicator(), loc_ncol1);
    _data.resize (loc_nnz1);
+   // first pointer points to the beginning of the data:
+   vector_of_iterator<pair_type>::operator[](0) = _data.begin().operator->();
 }
 template<class T>
-csr_seq_rep<T>::csr_seq_rep(const csr_seq_rep<T>& a)
- : vector_of_iterator<pair_type>(a.nrow()+1),
-   _row_ownership (a.row_ownership()),
-   _col_ownership (a.col_ownership()),
-   _data(a._data),
-   _is_symmetric (a._is_symmetric),
-   _pattern_dimension (a._pattern_dimension)
+csr_seq_rep<T>::csr_seq_rep(const csr_seq_rep<T>& b)
+ : vector_of_iterator<pair_type>(b.nrow()+1),
+   _row_ownership (b.row_ownership()),
+   _col_ownership (b.col_ownership()),
+   _data(b._data),
+   _is_symmetric (b._is_symmetric),
+   _pattern_dimension (b._pattern_dimension)
 {
-   // TODO: performs copy of arrays
-   fatal_macro ("physical copy of csr");
-}     
+  // physical copy of csr
+  typedef typename csr_seq_rep<T>::size_type size_type;
+  typename csr_seq_rep<T>::const_iterator ib = b.begin();
+  typename csr_seq_rep<T>::iterator       ia = begin();
+  ia[0] = _data.begin().operator->();
+  for (size_type i = 0, n = b.nrow(); i < n; i++) {
+    ia [i+1] = ia[0] + (ib[i+1] - ib[0]);
+  }
+}
 template<class T>
 csr_seq_rep<T>::csr_seq_rep(const asr_seq_rep<T>& a)
   : vector_of_iterator<pair_type>(a.nrow()+1),
@@ -137,7 +148,8 @@ odiststream&
 csr_seq_rep<T>::put (odiststream& ops, size_type istart) const
 {
     std::ostream& os = ops.os();
-    os << "%%MatrixMarket matrix coordinate real general" << std::endl
+    os << setprecision (std::numeric_limits<T>::digits10)
+       << "%%MatrixMarket matrix coordinate real general" << std::endl
        << nrow() << " " << ncol() << " " << nnz() << endl;
     const_iterator ia = begin();
     const size_type base = 1;
@@ -179,6 +191,32 @@ csr_seq_rep<T>::mult(
         set_op<T,T>(),
         y.begin());
 }
+template<class T>
+void
+csr_seq_rep<T>::trans_mult(
+    const vec<T,sequential>& x,
+    vec<T,sequential>&       y)
+    const
+{
+    // reset y and then cumul
+    std::fill (y.begin(), y.end(), T(0));
+    csr_cumul_trans_mult (
+        vector_of_iterator<pair_type>::begin(), 
+        vector_of_iterator<pair_type>::end(), 
+        x.begin(), 
+        set_add_op<T,T>(),
+        y.begin());
+}
+template<class T>
+csr_seq_rep<T>&
+csr_seq_rep<T>::operator*= (const T& lambda)
+{
+  iterator ia = begin();
+  for (data_iterator p = ia[0], last_p = ia[nrow()]; p != last_p; p++) {
+    (*p).second *= lambda;
+  }
+  return *this;
+}
 // ----------------------------------------------------------------------------
 // expression c=a+b and c=a-b with a temporary c=*this
 // ----------------------------------------------------------------------------
@@ -248,6 +286,100 @@ csr_seq_rep<T>::assign_add (
     set_pattern_dimension (std::max(a.pattern_dimension(), b.pattern_dimension()));
 }
 // ----------------------------------------------------------------------------
+// trans(a)
+// ----------------------------------------------------------------------------
+/*@! 
+ \vfill \pagebreak \mbox{} \vfill \begin{algorithm}[h]
+  \caption{{\tt sort}: sort rows by increasing column order}
+  \begin{algorithmic}
+    \INPUT {the matrix in CSR format}
+      ia(0:nrow), ja(0:nnz-1), a(0:nnz-1)
+    \ENDINPUT
+    \OUTPUT {the sorted CSR matrix}
+      iao(0:nrow), jao(0:nnzl-1), ao(0:nnzl-1)
+    \ENDOUTPUT
+    \BEGIN 
+      {\em first: reset iao} \\
+      \FORTO {i := 0} {nrow}
+	  iao(i) := 0;
+      \ENDFOR
+	
+      {\em second: compute lengths from pointers} \\
+      \FORTO {i := 0} {nrow-1}
+        \FORTO {p := ia(i)} {ia(i+1)-1}
+	    iao (ja(p)+1)++;
+        \ENDFOR
+      \ENDFOR
+
+      {\em third: compute pointers from lengths} \\
+      \FORTO {i := 0} {nrow-1}
+	  iao(i+1) += iao(i)
+      \ENDFOR
+
+      {\em fourth: copy values} \\
+      \FORTO {i := 0} {nrow-1}
+        \FORTO {p := ia(i)} {ia(i+1)-1}
+          j := ja(p) \\
+	  q := iao(j) \\
+	  jao(q) := i \\
+	  ao(q) := a(p) \\
+	  iao (j)++
+        \ENDFOR
+      \ENDFOR
+
+      {\em fiveth: reshift pointers} \\
+      \FORTOSTEP {i := nrow-1} {0} {-1}
+	iao(i+1) := iao(i);
+      \ENDFOR
+      iao(0) := 0
+    \END
+ \end{algorithmic} \end{algorithm}
+ \vfill \pagebreak \mbox{} \vfill
+*/
+
+// NOTE: transposed matrix has always rows sorted by increasing column indexes
+//       even if original matrix has not
+template<class T>
+void
+csr_seq_rep<T>::build_transpose (csr_seq_rep<T>& b) const
+{
+  b.resize (col_ownership(), row_ownership(), nnz()); 
+
+  // first pass: set ib(*) to ib(0)
+  iterator ib = b.begin();
+  for (size_type j = 0, m = b.nrow(); j < m; j++) {
+    ib[j+1] = ib[0];
+  }
+  // second pass: compute lengths of row(i) of a^T in ib(i+1)-ib(0)
+  const_iterator ia = begin();
+  for (size_type i = 0, n = nrow(); i < n; i++) {
+    for (const_data_iterator p = ia[i], last_p = ia[i+1]; p != last_p; p++) {
+      size_type j = (*p).first;
+      ib [j+1]++;
+    }
+  }
+  // third pass: compute pointers from lengths
+  for (size_type j = 0, m = b.nrow(); j < m; j++) {
+    ib [j+1] += (ib[j]-ib[0]);
+  }
+  // fourth pass: store values
+  data_iterator q0 = ib[0];
+  for (size_type i = 0, n = nrow(); i < n; i++) {
+    for (const_data_iterator p = ia[i], last_p = ia[i+1]; p != last_p; p++) {
+      size_type j = (*p).first;
+      data_iterator q = ib[j];
+      (*q).first  = i;
+      (*q).second = (*p).second;
+      ib[j]++;
+    }
+  }
+  // fiveth: shift pointers
+  for (long int j = b.nrow()-1; j >= 0; j--) {
+    ib[j+1] = ib[j];
+  }
+  ib[0] = q0;
+}
+// ----------------------------------------------------------------------------
 // instanciation in library
 // ----------------------------------------------------------------------------
 template class csr_seq_rep<Float>;
