@@ -24,17 +24,7 @@
 #include "rheolef/csr.h"
 namespace rheolef {
 
-#ifdef TO_CLEAN
-template <class T, class M>
-inline
-vec<T,M>
-csr<T,M>::operator* (const vec<T,M>& x)
-{
-  vec<T,M> y (row_ownership(), T());
-  mult (x, y);
-  return y;
-}
-#endif // TO_CLEAN
+  // moved into csr.h
 
 } // namespace rheolef
 # endif // _RHEOLEF_CSR_VEC_EXPR_H
