@@ -29,6 +29,7 @@
 //
 #include "rheolef/distributed.h"
 #include "rheolef/dis_macros.h"
+#include "rheolef/catchmark.h"
 namespace rheolef {
 
 class odiststream {
@@ -43,11 +44,18 @@ public:
      odiststream(std::ostream& os, const communicator& comm = communicator())
        : _ptr_os(&os), _use_alloc(false), _comm(comm) {}
 
+     odiststream (std::string filename, std::string suffix = "",
+             const communicator& comm = communicator())
+       : _ptr_os(0), _use_alloc(false), _comm()
+       {
+         open (filename, suffix, comm);
+       }
+
      ~odiststream();
 
 // modifiers:
 
-     void open (std::string filename, std::string suffix,
+     void open (std::string filename, std::string suffix = "",
              const communicator& comm = communicator());
 
      void close();
@@ -131,11 +139,18 @@ public:
      idiststream (std::istream& is, const communicator& comm = communicator())
        : _ptr_is(&is), _use_alloc(false), _comm(comm) {}
 
+     idiststream (std::string filename, std::string suffix = "",
+             const communicator& comm = communicator())
+       : _ptr_is(0), _use_alloc(false), _comm()
+       {
+         open (filename, suffix, comm);
+       }
+
       ~idiststream();
 
 // modifiers:
 
-     void open (std::string filename, std::string suffix,
+     void open (std::string filename, std::string suffix = "",
              const communicator& comm = communicator());
 
      void close();
@@ -149,6 +164,12 @@ public:
 
 // internal accesses:
 public:
+#ifndef _RHEOLEF_HAVE_MPI
+     bool nop() { return false; }
+#else
+     bool nop() { return size_type(_comm.rank()) != io_proc(); }
+#endif //_RHEOLEF_HAVE_MPI
+
      std::istream& is() {
 	check_macro (_ptr_is != 0, "try to use an uninitialized idiststream");
 	return *_ptr_is;
@@ -173,7 +194,7 @@ idiststream&						\
 operator >> (idiststream& s, T& x)			\
 {							\
   if (s.do_load()) { (s.is()) >> x; }			\
-  mpi::broadcast(mpi::communicator(), x, 0);		\
+  mpi::broadcast (mpi::communicator(), x, s.io_proc());	\
   return s;						\
 }
 #else // _RHEOLEF_HAVE_MPI
@@ -209,12 +230,43 @@ define_sequential_idiststream_macro(doubledouble)
 #endif // _RHEOLEF_HAVE_DOUBLEDOUBLE
 
 // predefined distributed streams
-extern idiststream dcin;
-extern odiststream dcout;
+extern idiststream din;
+extern odiststream dout;
 extern odiststream dclog;
-extern odiststream dcerr;
+extern odiststream derr;
 
 bool dis_scatch (idiststream& ips, const communicator& comm, std::string ch);
 
+inline
+bool dis_scatch (idiststream& ips, std::string ch)
+{
+  return dis_scatch (ips, ips.comm(), ch);
+}
+inline
+idiststream&
+operator>> (idiststream& ids, const catchmark& m)
+{
+    if (ids.nop()) return ids;
+    ids.is() >> setmark(m.mark());
+    std::string label = "#" + m.mark();
+    if (!scatch(ids.is(),label)) {
+        warning_macro ("catchmark: label `"<< label <<"' not found on input");
+    }
+    return ids;
+}
+inline
+odiststream&
+operator<< (odiststream& ods, const catchmark& m)
+{
+    if (ods.nop()) return ods;
+    ods.os() << setmark(m.mark())
+             << "#" << m.mark() << std::endl;
+    return ods;
+}
+// system utilities:
+int dis_system (const std::string& command, const communicator& comm = communicator());
+bool dis_file_exists (const std::string& filename, const communicator& comm = communicator());
+
+
 } // namespace rheolef
 # endif // _RHEOLEF_DISTSTREAM_H
