@@ -25,24 +25,16 @@
 namespace rheolef { 
 
 /*Class:eye
-NAME: @code{eye} - identity matrix 
+NAME: @code{eye} - the identity matrix 
 DESCRIPTION:       
- Following matlab, the name EYE is used in place of I to denote identity matrices
- because I is often used as a subscript or as sqrt(-1). The dimensions of EYE are
- determined by context.  For example,
- @example
-      	    b = a + 3*EYE
- @end example
- adds 3 to the diagonal elements of A.
- Small caps 'eye' is the class name, and upper case 'EYE' is the const variable
- "eye<double>()", i.e. the identity matrix.
-
- The preconditioner interface is usefull when calling algorithms without any
- preconditioners, e.g.
- @example
-            int status = cg (a, x, b, EYE, 100, 1e-7);
- @end example
-
+  Following matlab, the name @code{eye()} is used in place of I to denote identity matrices
+  because I is often used as a subscript or as sqrt(-1). The dimensions of @code{eye()} are
+  determined by context.
+  The preconditioner interface is usefull when calling algorithms without any
+  preconditioners, e.g.
+  @example
+    int status = pcg (a, x, b, eye(), 100, 1e-7);
+  @end example
 AUTHOR: 
     Pierre Saramito
     | Pierre.Saramito@imag.fr
@@ -52,15 +44,13 @@ DATE:
 End:
 */
 //<eye:
-template<class T>
 class eye {
-  public:
-    eye () {}
-    const vec<T>& operator * (const vec<T>& x) const { return x; }
-    const vec<T>& solve (const vec<T>& x) const { return x; }
-    const vec<T>& trans_solve (const vec<T>& x) const { x; }
+public:
+  eye() {}
+  template<class T> const vec<T>& operator* (const vec<T>& x) const { return x; }
+  template<class T> const vec<T>& solve (const vec<T>& x) const { return x; }
+  template<class T> const vec<T>& trans_solve (const vec<T>& x) const { x; }
 };
-#define EYE eye<Float>()
 //>eye:
 
 // Remark: the shared libraries are sometime buggy
