@@ -233,9 +233,9 @@ mpi_scatter_init (
         receive_data.begin(),
         receive_max_size,
         istart,
-        to.procs.begin(),
-        to.starts.begin(),
-        to.indices.begin());
+        to.procs().begin(),
+        to.starts().begin(),
+        to.indices().begin());
     // ---------------------------------------------------------------------------
     // 12) allocate the entire receive(from) scatter context
     // ---------------------------------------------------------------------------
@@ -247,15 +247,15 @@ mpi_scatter_init (
     msg_from_context_pattern (
         msg_size.begin(),
         msg_size.end(),
-        from.procs.begin(),
-        from.starts.begin(),
+        from.procs().begin(),
+        from.starts().begin(),
         proc2from_proc.begin());
     // ---------------------------------------------------------------------------
     // 14) Computes the receive compresed message indices for receive(from)
     // ---------------------------------------------------------------------------
     // assume that indices are sorted by increasing order
     std::vector<size_type> start(send_nproc+1);
-    copy (from.starts.begin(), from.starts.end(), start.begin());
+    copy (from.starts().begin(), from.starts().end(), start.begin());
     msg_from_context_indices (
         owner.begin(),
         owner.end(),
@@ -264,7 +264,7 @@ mpi_scatter_init (
         my_proc,
         idy_maxval,
         start.begin(),
-        from.indices.begin());
+        from.indices().begin());
     // ---------------------------------------------------------------------------
     // 15) wait on sends
     // ---------------------------------------------------------------------------
