@@ -35,7 +35,7 @@ SYNOPSIS:
   @example
     template <class Matrix, class Vector, class Preconditioner, class Real>
     int pcg (const Matrix &A, Vector &x, const Vector &b,
-      const Preconditioner &M, int &max_iter, Real &tol, odiststream *p_dcerr=0);
+      const Preconditioner &M, int &max_iter, Real &tol, odiststream *p_derr=0);
   @end example
 
 EXAMPLE:
@@ -44,7 +44,7 @@ EXAMPLE:
   @example
     size_t max_iter = 100;
     double tol = 1e-7;
-    int status = pcg(a, x, b, EYE, max_iter, tol, &cerr);
+    int status = pcg(a, x, b, EYE, max_iter, tol, &derr);
   @end example
 
 DESCRIPTION:       
@@ -102,20 +102,20 @@ End:
 //<pcg:
 template <class Matrix, class Vector, class Vector2, class Preconditioner, class Real, class Size>
 int pcg(const Matrix &A, Vector &x, const Vector2 &Mb, const Preconditioner &M,
-	Size &max_iter, Real &tol, odiststream *p_dcerr = 0, std::string label = "cg")
+	Size &max_iter, Real &tol, odiststream *p_derr = 0, std::string label = "cg")
 {
     Vector b = M.solve(Mb);
     Real norm2_b = dot(Mb,b);
     if (norm2_b == Real(0)) norm2_b = 1;
     Vector Mr = Mb - A*x;
     Real  norm2_r = 0;
-    if (p_dcerr) (*p_dcerr) << "[" << label << "] #iteration residue" << std::endl;
+    if (p_derr) (*p_derr) << "[" << label << "] #iteration residue" << std::endl;
     Vector p;
     for (Size n = 0; n <= max_iter; n++) {
 	Vector r = M.solve(Mr);
         Real prev_norm2_r = norm2_r;
         norm2_r = dot(Mr, r);
-	if (p_dcerr) (*p_dcerr) << "[" << label << "] " << n << " " << ::sqrt(norm2_r/norm2_b) << std::endl;
+	if (p_derr) (*p_derr) << "[" << label << "] " << n << " " << ::sqrt(norm2_r/norm2_b) << std::endl;
         if (norm2_r <= sqr(tol)*norm2_b) {
           tol = ::sqrt(norm2_r/norm2_b);
           max_iter = n;
