@@ -23,12 +23,19 @@
 
 // include all for convenience
 #include "rheolef/vec.h"
-#include "rheolef/csr_vec_expr.h"
 #include "rheolef/vec_expr.h"
 #include "rheolef/vec_expr_ops.h"
+#include "rheolef/vec_concat.h"
+#include "rheolef/vec_range.h"
 #include "rheolef/csr.h"
+#include "rheolef/csr_vec_expr.h"
+#include "rheolef/csr_concat.h"
 #include "rheolef/dis_inner_product.h"
 #include "rheolef/solver.h"
 #include "rheolef/pcg.h"
+#include "rheolef/pgmres.h"
+#include "rheolef/dia.h"
+#include "rheolef/diag.h"
 #include "rheolef/eye.h"
+#include "rheolef/solver_abtb.h"
 #endif // _RHEOLEF_SKIT_H
