@@ -21,19 +21,61 @@
 // direct solver interface
 //
 #include "rheolef/solver.h"
+#include "solver_wrapper.h"
+#include "solver_mumps.h"
+#include "solver_trilinos_ifpack.h"
+#include "solver_no_trilinos_ifpack.h"
 #include "solver_pastix.h"
 
 namespace rheolef {
 
 template<class T, class M>
-solver_rep<T,M>::solver_rep (const csr<T,M>& a, const solver_option_type& opt)
- : _ptr(new_macro(rep(a,opt)))
+solver_rep<T,M>::solver_rep ()
+ : _ptr(0)
 {
 }
 template<class T, class M>
+solver_rep<T,M>::solver_rep (const csr<T,M>& a, const solver_option_type& opt)
+ : _ptr(0)
+{
+  typedef solver_wrapper_rep<T,M> rep;
+  _ptr = new_macro (rep(a,opt));
+}
+template<class T, class M>
+void
+solver_rep<T,M>::build_lu (const csr<T,M>& a, const solver_option_type& opt)
+{
+#ifdef _RHEOLEF_HAVE_MUMPS
+  typedef solver_mumps_rep<T,M> rep;
+#elif defined(_RHEOLEF_HAVE_PASTIX)
+  typedef solver_pastix_rep<T,M> rep;
+  opt.iterative = 0;
+#else // ! _RHEOLEF_HAVE_TRILINOS && !_RHEOLEF_HAVE_PASTIX
+  typedef solver_no_trilinos_ifpack_rep<T,M> rep;
+#endif // ! _RHEOLEF_HAVE_MUMPS
+  if (_ptr) delete_macro (_ptr);
+  _ptr = new_macro (rep(a,opt));
+}
+template<class T, class M>
+void
+solver_rep<T,M>::build_ilu (const csr<T,M>& a, const solver_option_type& opt)
+{
+#ifdef _RHEOLEF_HAVE_TRILINOS
+  typedef solver_trilinos_ifpack_rep<T,M> rep;
+#elif defined(_RHEOLEF_HAVE_PASTIX)
+  typedef solver_pastix_rep<T,M> rep;
+  opt.iterative = 1;
+#else // ! _RHEOLEF_HAVE_TRILINOS && !_RHEOLEF_HAVE_PASTIX
+  typedef solver_no_trilinos_ifpack_rep<T,M> rep;
+#endif
+  if (_ptr) delete_macro (_ptr);
+  _ptr = new_macro (rep(a,opt));
+}
+template<class T, class M>
 solver_rep<T,M>::~solver_rep ()
 {
-  delete_macro (_ptr);
+  if (_ptr) delete_macro (_ptr);
+  _ptr = 0;
 }
 template<class T, class M>
 void 
@@ -43,13 +85,13 @@ solver_rep<T,M>::update_values (const csr<T,M>& a)
 }
 template<class T, class M>
 vec<T,M>
-solver_rep<T,M>::solve (const vec<T,M>& b)
+solver_rep<T,M>::solve (const vec<T,M>& b) const
 {
   return _ptr->solve (b);
 }
 template<class T, class M>
 vec<T,M>
-solver_rep<T,M>::trans_solve (const vec<T,M>& b)
+solver_rep<T,M>::trans_solve (const vec<T,M>& b) const
 {
   return _ptr->trans_solve (b);
 }
