@@ -30,6 +30,7 @@
 //
 #include "rheolef/vec.h"
 #include "rheolef/dis_inner_product.h"
+#include "rheolef/dis_accumulate.h"
 #include "rheolef/pretty_name.h"
 
 #include <boost/mpl/bool.hpp>
@@ -37,6 +38,7 @@
 #include <boost/proto/debug.hpp>
 #include <boost/proto/context.hpp>
 #include <boost/proto/transform.hpp>
+#include <boost/proto/operators.hpp>
 #include <boost/utility/enable_if.hpp>
 #include <boost/typeof/std/vector.hpp>
 #include <boost/typeof/std/complex.hpp>
@@ -50,6 +52,84 @@ using proto::_;
 template <class Expr>
 struct vec_expr;
 
+// -----------------------------------------------------------
+// iterator begin
+// -----------------------------------------------------------
+template<class Iterator>
+struct vec_iterator_wrapper {
+    typedef Iterator iterator;
+    explicit vec_iterator_wrapper (Iterator iter1) : iter(iter1) {} 
+    mutable Iterator iter;
+};
+struct vec_begin : proto::callable {
+    template<class Sig>
+    struct result;
+
+    template<class This, class Container>
+    struct result<This(Container)> : proto::result_of::as_expr
+       <
+        vec_iterator_wrapper<
+	  typename boost::remove_reference<Container>::type::const_iterator>
+       > {};
+
+    template<class Container>
+    typename result<vec_begin(const Container&)>::type
+    operator ()(const Container& cont) const {
+        vec_iterator_wrapper<typename Container::const_iterator> iter (cont.begin());
+        return proto::as_expr (iter);
+    }
+};
+// Here is the grammar that replaces vector terminals with their begin iterators
+struct vec_grammar_begin
+  : proto::or_<
+        proto::when <proto::terminal<vec<_, _> >, vec_begin(proto::_value)>
+      , proto::when <proto::terminal<_> >
+      , proto::when <proto::nary_expr<_, proto::vararg<vec_grammar_begin> > >
+    >
+{};
+
+// Here is an evaluation context that dereferences iterator terminals.
+struct vec_dereference_context {
+    // Unless this is an iterator terminal, use the default evaluation context
+    template<class Expr, class EnableIf = void>
+    struct eval : proto::default_eval<Expr, const vec_dereference_context> {};
+
+    // Dereference iterator terminals.
+    template<class Expr>
+    struct eval<Expr,
+       typename boost::enable_if<
+         proto::matches<Expr, proto::terminal<vec_iterator_wrapper<_> > >
+        >::type
+       > {
+        typedef typename proto::result_of::value<Expr>::type IteratorWrapper;
+        typedef typename IteratorWrapper::iterator iterator;
+        typedef typename std::iterator_traits<iterator>::reference result_type;
+
+        result_type operator() (Expr &expr, const vec_dereference_context&) const {
+            return *proto::value(expr).iter;
+        }
+    };
+};
+// Here is an evaluation context that increments iterator
+// terminals.
+struct vec_increment_context {
+    // Unless this is an iterator terminal, use the default evaluation context
+    template<class Expr, class EnableIf = void>
+    struct eval : proto::null_eval<Expr, const vec_increment_context> {};
+
+    // advance iterator terminals.
+    template<class Expr> struct eval<Expr,
+	typename boost::enable_if<
+            proto::matches<Expr, proto::terminal<vec_iterator_wrapper<_> > >
+        >::type
+      > {
+        typedef void result_type;
+
+        result_type operator() (Expr &expr, const vec_increment_context&) const {
+            ++proto::value(expr).iter;
+        }
+    };
+};
 // -----------------------------------------------------------
 // Here is an evaluation context that indexes into a vec
 // expression and combines the result.
@@ -69,7 +149,18 @@ struct vec_subscript_context {
              return proto::value(expr)[ctx._i];
          }
     };
-    size_type _i;
+    // index iterator terminals
+    template<class Expr>
+    struct eval<Expr, typename boost::enable_if<
+              proto::matches<Expr, proto::terminal<vec_iterator_wrapper<_> > > >::type > {
+        typedef typename proto::result_of::value<Expr>::type IteratorWrapper;
+        typedef typename IteratorWrapper::iterator iterator;
+        typedef typename std::iterator_traits<iterator>::reference result_type;
+        result_type operator() (Expr &expr, const vec_subscript_context& ctx) const {
+            return *proto::value(expr).iter[ctx._i];
+        }
+    };
+    mutable size_type _i;
 };
 // -----------------------------------------------------------
 // get size
@@ -122,83 +213,6 @@ struct vec_check_size_context {
     size_type _size;
 };
 // -----------------------------------------------------------
-// iterator begin
-// -----------------------------------------------------------
-template<class Iterator>
-struct vec_iterator_wrapper {
-    typedef Iterator iterator;
-    explicit vec_iterator_wrapper (Iterator iter1) : iter(iter1) {} 
-    Iterator iter;
-};
-struct vec_begin : proto::callable {
-    template<class Sig>
-    struct result;
-
-    template<class This, class Container>
-    struct result<This(Container)> : proto::result_of::as_expr
-       <
-        vec_iterator_wrapper<
-	  typename boost::remove_reference<Container>::type::const_iterator>
-       > {};
-
-    template<class Container>
-    typename result<vec_begin(const Container&)>::type
-    operator ()(const Container& cont) const {
-        vec_iterator_wrapper<typename Container::const_iterator> iter (cont.begin());
-        return proto::as_expr (iter);
-    }
-};
-// Here is the grammar that replaces vector terminals with their begin iterators
-struct vec_grammar_begin : proto::or_<
-        proto::when <proto::terminal<vec<_, _> >, vec_begin(proto::_value)>
-      , proto::when <proto::terminal<_> >
-      , proto::when <proto::nary_expr<_, proto::vararg<vec_grammar_begin> > >
-    >
-{};
-
-// Here is an evaluation context that dereferences iterator terminals.
-struct vec_dereference_context {
-    // Unless this is an iterator terminal, use the default evaluation context
-    template<class Expr, class EnableIf = void>
-    struct eval : proto::default_eval<Expr, const vec_dereference_context> {};
-
-    // Dereference iterator terminals.
-    template<class Expr>
-    struct eval<Expr,
-       typename boost::enable_if<
-         proto::matches<Expr, proto::terminal<vec_iterator_wrapper<_> > >
-        >::type
-       > {
-        typedef typename proto::result_of::value<Expr>::type IteratorWrapper;
-        typedef typename IteratorWrapper::iterator iterator;
-        typedef typename std::iterator_traits<iterator>::reference result_type;
-
-        result_type operator() (Expr &expr, const vec_dereference_context&) const {
-            return *proto::value(expr).iter;
-        }
-    };
-};
-// Here is an evaluation context that increments iterator
-// terminals.
-struct vec_increment_context {
-    // Unless this is an iterator terminal, use the default evaluation context
-    template<class Expr, class EnableIf = void>
-    struct eval : proto::null_eval<Expr, const vec_increment_context> {};
-
-    // advance iterator terminals.
-    template<class Expr> struct eval<Expr,
-	typename boost::enable_if<
-            proto::matches<Expr, proto::terminal<vec_iterator_wrapper<_> > >
-        >::type
-      > {
-        typedef void result_type;
-
-        result_type operator() (Expr &expr, const vec_increment_context&) const {
-            ++proto::value(expr).iter;
-        }
-    };
-};
-// -----------------------------------------------------------
 // A grammar which matches all the assignment operators,
 // so we can easily disable them.
 // -----------------------------------------------------------
@@ -244,14 +258,15 @@ struct vec_expr : proto::extends<Expr, vec_expr<Expr>, vec_domain> {
 
 // typedefs :
 
+    typedef proto::extends<Expr, vec_expr<Expr>, vec_domain> base_type;
+
     typedef vec_get_size_context::size_type size_type;
-    typedef typename proto::result_of::eval<const Expr, const vec_subscript_context>::type element_type;
     typedef typename boost::result_of<vec_grammar_begin(const Expr&)>::type raw_iterator;
 
 // allocators:
 
     explicit vec_expr (const Expr& expr)
-      : proto::extends<Expr, vec_expr<Expr>, vec_domain>(expr)
+      : base_type(expr)
     {}
 
 // accessors:
@@ -267,10 +282,6 @@ struct vec_expr : proto::extends<Expr, vec_expr<Expr>, vec_domain> {
     size_type dis_size() const {
         return ownership().dis_size();
     }
-    element_type operator[] (size_type i) const {
-        const vec_subscript_context get_subscript(i);
-        return proto::eval(*this, get_subscript);
-    }
     struct const_iterator_begin : raw_iterator {
 
        typedef std::input_iterator_tag        iterator_category;
@@ -279,9 +290,16 @@ struct vec_expr : proto::extends<Expr, vec_expr<Expr>, vec_domain> {
        typedef value_type* pointer;
        typedef std::ptrdiff_t difference_type;
 
-       const_iterator_begin (const vec_expr<Expr>& expr) 
-	 : raw_iterator(vec_grammar_begin() (expr)) {
-       }
+#if BOOST_VERSION < 104601
+       const_iterator_begin (const vec_expr<Expr>& expr)
+	 : raw_iterator(vec_grammar_begin() (expr))
+       {}
+#else // BOOST_VERSION < 104601
+       const_iterator_begin (const vec_expr<Expr>& expr)
+	 : raw_iterator(vec_grammar_begin() (expr.proto_expr_))
+	   // ICI: il faut recuperer base_type(expr) de type Expr au lieu vec_expr<Expr>
+       {}
+#endif // BOOST_VERSION < 104601
        const_iterator_begin& operator++ () { 	
          const vec_increment_context inc = {};
          proto::eval (*this, inc);
@@ -292,22 +310,30 @@ struct vec_expr : proto::extends<Expr, vec_expr<Expr>, vec_domain> {
 	 operator++();
 	 return tmp;
        }
-       reference operator* () {
+       reference dereference () const {
          const vec_dereference_context deref = {};
          typedef typename proto::result_of::eval<raw_iterator,vec_dereference_context>::type my_ref;
          reference value = proto::eval(*this, deref);
          return value;
        }
+       reference operator* () const { return dereference(); }
+       reference operator* ()       { return dereference(); }
     };
     typedef const_iterator_begin const_iterator;
     const_iterator begin() const { return const_iterator(*this); }
 
+// random access:
+
+    typedef typename proto::result_of::eval<const Expr, const vec_subscript_context>::type value_type;
+    value_type operator[] (size_type i) const {
+        const vec_subscript_context get_subscript(i);
+        return proto::eval(*this, get_subscript);
+    }
 private:
     // hide this:
     using proto::extends<Expr, vec_expr<Expr>, vec_domain>::operator[];
 };
-// Define a trait type for detecting vec terminals, to
-// be used by the BOOST_PROTO_DEFINE_OPERATORS macro below.
+// Define a trait type for detecting vec terminals
 template<class T>
 struct is_vec : mpl::false_ {};
 
@@ -315,16 +341,18 @@ template<class T, class M>
 struct is_vec<vec<T, M> > : mpl::true_ {};
 
 namespace vec_detail {
-    template<class FwdIter, class Expr, class Op>
-    void evaluate (FwdIter begin, FwdIter end, const Expr& expr, Op op) {
+    template<class ForwardIterator, class Expr, class Op>
+    void evaluate (ForwardIterator begin, ForwardIterator end, const Expr& expr, Op op) {
         const vec_increment_context   inc   = {};
         const vec_dereference_context deref = {};
         typename boost::result_of<vec_grammar_begin(const Expr&)>::type expr_begin
 	  = vec_grammar_begin() (expr);
+#ifndef TRY_VEC_EXPR
         for (; begin != end; ++begin) {
             op (*begin, proto::eval(expr_begin, deref));
             proto::eval (expr_begin, inc);
         }
+#endif // TRY_VEC_EXPR
     }
     struct assign_op {
         template<class T, class U>
@@ -339,7 +367,7 @@ template<class T, class M>
 template<class Expr>
 inline
 vec<T,M>& 
-vec<T,M>::operator= (const Expr& expr) {
+vec<T,M>::operator= (const vec_expr<Expr>& expr) {
     // get and check sizes:
     const vec_get_size_context get_size;
     proto::eval (proto::as_expr<vec_domain>(expr), get_size);
@@ -362,6 +390,20 @@ vec<T,M>::vec (const Expr& expr)
 {
     operator= (expr);
 }
+template<class T, class M>
+inline
+T
+norm2 (const vec<T,M>& x)
+{
+  return dot(x,x);
+}
+template<class T, class M>
+inline
+T
+norm (const vec<T,M>& x)
+{
+  return sqrt(norm2(x));
+}
 
 } // namespace rheolef
 #endif // _RHEOLEF_VEC_EXPR_H
