@@ -27,7 +27,7 @@ int main() { return 0; }
 using namespace rheolef;
 using namespace std;
 int main(int argc, char**argv) {
-    environment distributed(argc, argv);
+    environment rheolef(argc, argv);
     size_t n = (argc < 2) ? 5 : atoi(argv[1]);
     communicator comm;
     distributor ownership (n, comm, (comm.rank() == 0 ? n : 0));
@@ -38,6 +38,6 @@ int main(int argc, char**argv) {
         }
     }
     a.dis_entry_assembly();
-    dcout << a;
+    dout << a;
 }
 #endif // _RHEOLEF_HAVE_MPI
