@@ -28,7 +28,7 @@ using namespace std;
 using namespace rheolef;
 int main(int argc, char**argv)
 {
-    environment distributed(argc, argv);
+    environment rheolef(argc, argv);
     size_t n = (argc < 2) ? 5 : atoi(argv[1]);
     asr<Float> a(n,n);
     size_t first = a.row_first_index();
@@ -39,6 +39,6 @@ int main(int argc, char**argv)
         }
     }
     a.dis_entry_assembly();
-    dcout << a;
+    dout << a;
 }
 #endif // _RHEOLEF_HAVE_MPI
