@@ -22,7 +22,7 @@
 using namespace rheolef;
 using namespace std;
 int main(int argc, char**argv) {
-    environment distributed(argc, argv);
+    environment rheolef(argc, argv);
     size_t n = (argc < 2) ? 5 : atoi(argv[1]);
     asr<Float> a(n,n);
     size_t first = a.row_first_index();
@@ -33,5 +33,5 @@ int main(int argc, char**argv) {
         }
     }
     a.dis_entry_assembly();
-    dcout << a;
+    dout << a;
 }
