@@ -22,8 +22,8 @@
 using namespace rheolef;
 using namespace std;
 int main(int argc, char**argv) {
-  environment distributed(argc, argv);
+  environment rheolef(argc, argv);
   csr<Float> a;
-  dcin  >> a; 
-  dcout << a;
+  din  >> a; 
+  dout << a;
 }
