@@ -19,9 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 # -------------------------------------------------------------------------
-SRCDIR=${SRCDIR-"."}
 TOP_SRCDIR=${TOP_SRCDIR-"../.."}
-NPROC_MAX=2
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
 run "./get_first_tst 2> /dev/null >/dev/null"
