@@ -24,8 +24,10 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"7"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
-if test "$MPIRUN" = ""; then
+if test "$MPIRUN" = "" || test "$NPROC_MAX" -le 2; then
   echo "      (skipped)"
 else
   run "$MPIRUN -np 2 ./mpi_boost_nonblocking_array_of_idxset2_tst 2>/dev/null | sort -n | diff -w -B mpi_boost_nonblocking_array_of_idxset2_tst.valid -"
+  status=$?
+  exit $status
 fi
