@@ -24,8 +24,14 @@ TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 NPROC_MAX=${NPROC_MAX-"7"}
 . "${TOP_SRCDIR}/config/loop_mpirun.sh"
 
-if test "$MPIRUN" = ""; then
-  echo "      (skipped)"
-else
-  run "$MPIRUN -np 2 ./mpi_boost_nonblocking_idxset_tst 2>/dev/null | sort -n | diff -w -B mpi_boost_nonblocking_idxset_tst.valid -"
+#echo "    (skipped for foehn)"	# TODO
+#exit 0
+
+if test "$MPIRUN" = "" || test "$NPROC_MAX" -lt 2; then
+  echo "      skipped (requires np >= 2)"
+else 
+  run "rm -f ./mpi_boost_nonblocking_idxset_tst-[01]"
+  run "$MPIRUN -np 2 ./mpi_boost_nonblocking_idxset_tst 2>/dev/null && cat ./mpi_boost_nonblocking_idxset_tst-[01] | diff -Bw mpi_boost_nonblocking_idxset_tst.valid -"
+  status=$?
+  exit $status
 fi
