@@ -953,7 +953,7 @@ void Geometry::AfterRead()
 		     << edges[i].v[1-jj]->r.y -edges[i].Adj[jj]->v[1-edges[i].SensAdj[jj]]->r.y <<  endl;
 		*/
 	       }
-	       
+	       else {tg=R2(0,0); if (verbosity>6) cout << "Corner without tangent, edge " <<i << " end " << jj << endl; }
 	       //else ;// a Corner with no tangent => nothing to do    
              } // a tg 
              else 
@@ -1052,7 +1052,13 @@ void Geometry::AfterRead()
     for(int i=0;i<NbOfCurves ;i++)
      {
         cout << " Curve " << i << " begin e=" << Number(curves[i].be) << " k=" << curves[i].kb 
-             << "  end e= " << Number(curves[i].ee) << " k=" << curves[i].ke << endl;
+		<< " vert=" << Number(edges[Number(curves[i].be)].v[curves[i].kb])
+		<< " Corner?=" << ((edges[Number(curves[i].be)].v[curves[i].kb])->Corner()?"y":"n")
+		<< " tg=" << edges[Number(curves[i].be)].tg[curves[i].kb] 
+             << "  end e= " << Number(curves[i].ee) << " k=" << curves[i].ke 
+		<< " vert=" << Number(edges[Number(curves[i].ee)].v[curves[i].ke])
+		<< " Corner?=" << ((edges[Number(curves[i].ee)].v[curves[i].ke])->Corner()?"y":"n")
+		<< " tg=" << edges[Number(curves[i].ee)].tg[curves[i].ke] << endl;
      }
     delete []ev;
     delete []hv;
