@@ -20,16 +20,13 @@
 #
 # -------------------------------------------------------------------------
 SRCDIR=${SRCDIR-"."}
-tmp=${TMPDIR-"/tmp"}/rheolef-fdstest1-$$.valid
-cat > $tmp << EOF
-read 20 chars and print them (2 twice due to use of unget())
-abcdefghghijklmnopqr
-EOF
-echo  "      echo abcdefghijklmnopqrst | ./fdstest1"
-echo abcdefghijklmnopqrst | ./fdstest1 | diff $tmp -
+TOP_SRCDIR=${TOP_SRCDIR-"../.."}
+NPROC_MAX=${NPROC_MAX-"8"}
+. "${TOP_SRCDIR}/config/loop_mpirun.sh"
+
+status=0
+
+run "echo abcdefghijklmnopqrst | ./fdstest1 | diff $SRCDIR/fdstest1.valid -"
 status=$?
-/bin/rm -f $tmp
+
 exit $status
-
-
-
