@@ -19,9 +19,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 # -------------------------------------------------------------------------
+TOP_SRCDIR=${TOP_SRCDIR-"../.."}
 SRCDIR=${SRCDIR-"."}
-echo  "      ./fdstest3 $SRCDIR/fdstest3.cc"
-./fdstest3 $SRCDIR/fdstest3.cc 2>/dev/null | sed -e 's/^> //' -e '$d' | diff $SRCDIR/fdstest3.cc -
+. "${TOP_SRCDIR}/config/loop_mpirun.sh"
+
+run "./fdstest3 $SRCDIR/fdstest3.cc 2>/dev/null | sed -e 's/^> //' -e '$d' | diff -Bw $SRCDIR/fdstest3.cc -"
 status=$?
 exit $status
-
