@@ -20,6 +20,7 @@
 /// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 /// 
 /// =========================================================================
+#include "rheolef/iorheo.h"
 
 namespace rheolef { 
 bool scatch (std::istream& in, const std::string& ch);
@@ -47,6 +48,7 @@ End: */
 class catchmark {
     public:
 	catchmark(const std::string& x);
+	const std::string& mark() const { return _mark; }
 	friend std::istream& operator >> (std::istream& is, const catchmark& m);
 	friend std::ostream& operator << (std::ostream& os, const catchmark& m);
     protected:
