@@ -31,7 +31,9 @@ public:
     heap_object(size_type sizeof_bucket = sizeof(T));
     ~heap_object();
     T* new_bucket() const;
+#ifdef TO_CLEAN
     T* malloc() const { return new_bucket(); } // for boost::object_pool compatibility
+#endif // TO_CLEAN
     size_type size() const { return _counter; }
     void reinitialize (size_type sizeof_bucket = sizeof(T));
     void clear ();
