@@ -26,6 +26,7 @@
 namespace rheolef { 
 /*Class:iorheo
 NAME: @code{iorheo} - input and output functions and manipulation 
+@cindex graphic render
 @clindex iorheo
 @clindex csr
 @clindex geo
@@ -47,7 +48,7 @@ NAME: @code{iorheo} - input and output functions and manipulation
 @toindex @code{vtk}
 @toindex @code{geomview}
 @toindex @code{PlotM}
-EXAMPLE:
+SMALL PIECES OF CODE:
     input geo in standard file format:
     @example
     	cin >> g;
@@ -147,7 +148,8 @@ FILE FORMAT MANIPULATORS:
 @fiindex @file{.face} tetgen mesh boundary faces
 @fiindex @file{.v} grummp tridimensionnal mesh
 @fiindex @file{.m} grummp bidimensionnal mesh
-@fiindex @file{.msh} gmsh mesh
+@fiindex @file{.gmsh} gmsh mesh
+@fiindex @file{.gmsh_pos} gmsh metric mesh
 @fiindex @file{.hb} Harwell-Boeing matrix
 @fiindex @file{.geo} mesh
 @fiindex @file{.field} field
@@ -190,9 +192,13 @@ FILE FORMAT MANIPULATORS:
 	uses @file{.mmg3d} Cecile Dobrzynski's tridimensional anisotropic
 	mesh generator file format for geo input/output operation. 
     @itemx gmsh
-	uses @file{.msh} 
+	uses @file{.gmsh} 
 	gmsh Christophe Geuzaine and Jean-François Remacle 
         mesh generator file format for geo input/output operation. 
+    @itemx gmsh_pos
+	uses @file{.gmsh_pos} 
+	gmsh Christophe Geuzaine and Jean-François Remacle 
+        mesh metric file format for geo adapt input/output operation. 
     @itemx grummp
 	uses @file{.m} (bidimensional) or @file{.v} (tridimensionnal)
 	Carl Ollivier-Gooch 's mesh generator file format for geo input/output
@@ -303,13 +309,21 @@ VALUATED MANIPULATORS:
     	cout << velocity << plotmtv << vectorscale(0.1) << uh;
     @end example
 @clindex catchmark
-    See also @ref{catchmark class} for input-output of vector-valued fields.
+    See also @ref{catchmark algorithm} for input-output of vector-valued fields.
     @table @code
     @itemx isovalue @var{float}
     @itemx n_isovalue @var{int}
     @itemx n_isovalue_negative @var{int}
     @itemx vectorscale @var{float}
     @itemx subdivide @var{float}
+@cindex image file format
+@cindex graphic render
+@fiindex @file{.png} image 
+@fiindex @file{.jpg} image
+@fiindex @file{.pdf} image
+    @itemx image_format @var{string}
+	The argument is any valid image format, such as @code{png}, @code{jpg} or @code{pdf},
+	that could be handled by the corresponding graphic render.
     @end table
 End:*/
 
@@ -331,6 +345,7 @@ public:
 
 		transpose,			// avoid hb-matrix(csc) to/in csr conversion: 
 						// thus, handle the transposed matrix
+		upgrade,	 		// rheolef upgrade file format version
 
 		// color plot options
 
@@ -349,8 +364,10 @@ public:
 		dump,				// Dump format (debug)
 
 		bamg,				// bamg F. hecht mesh & metric format
+		peschetola,			// Custom meshes by Valentina Peschetola
 		grummp,				// grummp mesh file format
-		gmsh,				// gmsh mesh & metric file format
+		gmsh,				// gmsh mesh file format
+		gmsh_pos,			// gmsh metric mesh file format
 		mmg3d,				// mmg3d mesh format
 		tetgen,				// tetgen mesh format
 		qmg,				// qmg mesh file format
@@ -392,6 +409,7 @@ public:
                 lattice, 	  	       	// subdivide each element by two, as in P2-iso-P1, for geo/vtkdata format
                 label, 	  	 	      	// write labels on plotmtv contour lines
 		autovscale,			// adapt vscale to actual length of vectors and meshsize
+		reader_on_stdin,		// when read on stdin, add -persist to gnuplot
 
 		last
 	};
