@@ -49,6 +49,11 @@ namespace rheolef {
 using namespace std;
 namespace io = boost::iostreams;
 
+std::string
+get_tmpdir() {
+ char* c_tmpdir = std::getenv ("TMPDIR");
+ return (c_tmpdir == 0) ? "/tmp" : c_tmpdir;
+}
 string
 itos (string::size_type i)
 {
@@ -156,9 +161,15 @@ irheostream::open (const string& name, const string& suffix)
 
     // get full file path with the optional '.gz' suffix:
     string full_name = get_full_name_from_rheo_path (name, suffix);
-    bool has_gz = has_suffix (full_name, "gz");
-
+    if (full_name.length() == 0) {
+      if (suffix.length() != 0) {
+        error_macro ("file \"" << name << "[." << suffix << "[.gz]]\" not found");
+      } else {
+        error_macro ("file \"" << name << "[.gz]\" not found");
+      }
+    }
     // open the file[.gz]:
+    bool has_gz = has_suffix (full_name, "gz");
     if (!has_gz) {
       _ifs.open (full_name.c_str(), ios_base::in);
     } else {
